#ifndef ANALYSISALGORITHM_H
#define ANALYSISALGORITHM_H

#include <memory>
#include <string>
#include <boost/thread.hpp>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>
#include "Sct/IoExceptions.h"
#include "Sct/LogicErrors.h"

namespace SctData {
class TestResult;
}
class TestData;

namespace SctAnalysis {
  
/**
  An abstract base class for all the analysis algorithms.
  @author Matthew Palmer
  @date 11 June 2003  
  */
class AnalysisAlgorithm {
public:
    /**
       Default constructor, to be used in creation of map.
     */
    AnalysisAlgorithm(const TestData& testData, const string& moduleName) throw() : 
	m_modulename(moduleName), m_test(testData), m_done(false) {}
    /**
       Destructor.
      */
    virtual ~AnalysisAlgorithm() throw();
    /**
       get a pointer to the cached TestData;
    */
    const TestData& getTestData() const throw() {return m_test;}

    /**
      Get the name of the module which this algorithm is working on.
    */
    const std::string getModuleName() const throw() {return m_modulename;}

    /**
       Must be overridden by sub-classes. Returns a shared pointer to a concrete AnalysisAlgorithm
       of the correct type. Primarily used by the map to get an AnalysisAlgorithm of the correct type.
    */
    virtual boost::shared_ptr<AnalysisAlgorithm> clone(const TestData& testData, const string& moduleName) const throw() =0;
    /**
       Notify algorithm that it may add a named FitScanResult to this TestResult. 
       Default is to do nothing, but concrete classes should react as they see fit!
       @param name the IS name of the FitScanResult.
       @param t the test result to add it to.
    */
    virtual void canAddFitScanResult(const std::string& name) throw(Sct::LogicError, Sct::IoError) {}
    /**
       Notify algorithm that it may add a named RawScanResult to this TestResult. 
       Default is to do nothing, but concrete classes can react as they see fit!
       @param name the IS name of the RawScanResult.
       @param t  the test result to add it to.
    */
    virtual void canAddRawScanResult(const std::string& name) throw(Sct::LogicError, Sct::IoError) {}
    /**
       Method indicates when a particular analysis is finished.
    */
    bool isDone() const throw() {return m_done;}
    /**
       Access to the TestResult if required.
     */
    boost::shared_ptr<SctData::TestResult> getTestResult() const{return m_result;}
    
    /**
      Access the mutex
      */
    boost::recursive_mutex& getMutex() const {return m_access;}
protected:
    /**
       Allows concrete classes to set the TestResult member after they create one
       of the appropriate type.
     */
    void setTestResult(boost::shared_ptr<SctData::TestResult> testresult){m_result=testresult;}
    /**
       Used by other methods to ensure that a TestResult has been made.
       @throw LogicError if TestResult member is not initialised.
    */
    void checkTestResultExists() const throw(Sct::LogicError);
    /**
       Convenience method can called by derived class when notification is given that
       a new FitScanResult is available.
     */
    void addFit(const std::string& name) throw(Sct::LogicError, Sct::IoError);
    /**
       Convenience method can called by derived class when notification is given that
       a new RawScanResult is available.
     */
    void addRaw(const std::string& name) throw(Sct::LogicError, Sct::IoError);
    /**
      Useful function that initializes TestResult from Test and module name.
      @note TestResult must have been created first!
    */
    void initializeTestResult() throw(Sct::LogicError);
    /**
       Method callable by published the TestResult, and sets `done'.
    */
    void finish() throw(Sct::IoError, Sct::LogicError);
    
    AnalysisAlgorithm() ;
    
private:
    const std::string m_modulename;              ///< name of the module being worked on.
    boost::shared_ptr<SctData::TestResult> m_result;  ///< pointer to the result. @note it is up to the Concrete class to make one!
    const TestData& m_test;      ///< pointer to the testdata.
    bool m_done;                      ///< true if finish() has been called.
    mutable boost::recursive_mutex m_access;      ///< access to the algorithm can be locked for thread-safeness.
};
}

#endif //#ifndef ANALYSISALGORITHM_H
