package SctData;

import hep.aida.*;
import hep.aida.ref.histogram.*;
import Sct.*;

/**
 * A RawScanResult represents the raw data for a single scan. Note that this object (like all others in the package) owns its
 * members (i.e. it will delete them when it is deleted). This can cause problems with the 4 argument constructor if you give
 * it local vars!!!! The get***Occupancy functions all return a TH1D with a constant name.
 * It is up to the calling code to rename it as required if it does not want the hist deleted the next time
 * any of these functions is called.
 * @author Matthew Palmer
 */
public class RawScanResult extends ScanResult {
    public RawScanResult(ScanHeader header, ModuleConfiguration moduleConfig, ScanPoints points, IHistogram2D scanData) {
        super(header, moduleConfig, points);
        this.scanData = scanData;
    }

    public String getClassName() {
        return "SctData.RawScanResult";
    }

    public String getUniqueID() {
        return "RawScanResult." + header.getUniqueID();
    }

    public static String getUniqueID(ScanHeader header) {
        return "RawScanResult." + header.getUniqueID();
    }

    /** Gets the scan data as a ROOT histogram. */
    public IHistogram2D getData() {
        return scanData;
    }

    /** Utility function that gets a general occupancy hist. Returns a TH1D containing the occupancy. */
    
    public static RawScanResult read(IStream s, ObjectManager o) throws java.io.IOException {
        return new RawScanResult(s, o);
    }
    
    public void write(OStream s, ObjectManager o) throws java.io.IOException {
        super.write(s, o);
        
        int nPoints = getPoints().getNPoints();
        int size = (Parameters.nChannelLink + 2) * (nPoints + 2);
        s.writeInt("Size", size, false);
        s.writeShort("Type", ROOTHIST, false);
        s.writeShort("Width", WIDTH_64BIT, false);
        
        
        //Copy data into arrays as necessary for Root format.  Ignore under and overflow bins
        double[] link0Data = new double[size];
        double[] link1Data = new double[size];
        
        for (int i=0; i<Parameters.nChannelLink; ++i) {
            for (int j=0; j<nPoints; ++j) {
                link0Data[(Parameters.nChannelLink+2)*(j+1)+i+1] = scanData.binHeight(i, j);                
            }
        }
        for (int i=0; i<Parameters.nChannelLink; ++i) {
            for (int j=0; j<nPoints; ++j) {
                link1Data[(Parameters.nChannelLink+2)*(j+1)+i+1] = scanData.binHeight(i+Parameters.nChannelLink, j);
            }
        }
        
        s.writeDoubleArray("Link0Data", link0Data);
        s.writeDoubleArray("Link1Data", link1Data);
    }
    
    private RawScanResult(IStream s, ObjectManager o) throws java.io.IOException {
        super(s, o);
        FixedAxis xAxis = new FixedAxis(Parameters.nChannelModule, -0.5, Parameters.nChannelModule+0.5);
        Histogram2D data = new Histogram2D(getUniqueID(), getUniqueID(), xAxis, getPoints().getAxis());
        scanData = data;
        
        readData(s, data);
    }
    
    private void readData(IStream s, Histogram2D data) throws java.io.IOException {
        int size = s.readInt("Size");
        short type = s.readShort("Type");
        short width = s.readShort("Width");
        
        switch (type) {
            case SLICE:
                readSliceData(s, data, size, width);
                return;
                
            case ROOTHIST:
                readRootData(s, data, size, width);
                return;
                
            default:
                throw new java.io.StreamCorruptedException("Unsupported raw data type: " + type);
        }
    }
    
    private void readSliceData(IStream s, Histogram2D data, int size, short width) throws java.io.IOException {
        
    }
    
    //Read in ROOT data - copy into arrays for internal rep
    //Ignore under and overflow bins.
    private void readRootData(IStream s, Histogram2D data, int size, short width) throws java.io.IOException {
        double [] link0Data = s.readDoubleArray("Link0Data");
        double [] link1Data = s.readDoubleArray("Link1Data");
        int nPoints = getPoints().getNPoints();
        
        if (link0Data.length != link1Data.length || link0Data.length != (Parameters.nChannelLink + 2) * (nPoints+2) ) {
            throw new java.io.StreamCorruptedException("Data arrays are not correct length.  Expected: " + 
                      (Parameters.nChannelLink + 2) * (nPoints+2) + " Link0Data length: " + 
                      link0Data.length + " Link1Data length: " + link1Data.length);
        }
        
        double [][] heights = new double[Parameters.nChannelModule+2][nPoints+2];
        double [][] errors = new double[Parameters.nChannelModule+2][nPoints+2];                

        IAxis xAxis = data.xAxis();
        IAxis yAxis = data.yAxis();
        
        for (int i=0; i<Parameters.nChannelModule; ++i) {
            for (int j=0; j<nPoints; ++j) {
                heights[i+1][j+1] = i<Parameters.nChannelLink ? link0Data[(j+1)*(Parameters.nChannelLink+2)+i+1] : link1Data[(j+1)*(Parameters.nChannelLink+2)+i+1-Parameters.nChannelLink];
                errors[i+1][j+1] = Math.sqrt(heights[i][j]);
            }
        }
                
        data.setContents(heights, errors, null, null, null, null, null);
    }
    
    private static final short SLICE = (short)2;
    private static final short SCURVE = (short)4;
    private static final short SLICE_COMPRESSED = (short)6;
    private static final short SCURVE_COMPRESSED = (short)8;
    private static final short FIT = (short)10;
    private static final short HITS_PER_EVENT = (short)12;
    private static final short ROOTHIST = (short)14;
    private static final short RAWHIST = (short)16;
    
    private static final short WIDTH_64BIT = (short)64;
    private static final short WIDTH_32BIT = (short)32;
    private static final short WIDTH_16BIT = (short)16;
    
    private IHistogram2D scanData;
}
