#ifndef SCANHEADER_H
#define SCANHEADER_H

#include <string>
#include <boost/shared_ptr.hpp>
#include "Sct/Serializable.h"
#include "Sct/Streamable.h"

using namespace std;
using boost::shared_ptr;

class TH1;
class TH2;

namespace SctData {

class ConfigurationVariable;
class DataStatus;

/**
 * This class contains the identifying information about a scan result.
 * It is entirely defined by a run number, a scan number, a module name and a scan type.
 * The combination of the first 3 of those should be guarenteed to be unique.
 * @author Matthew Palmer
 */
class ScanHeader : public virtual Sct::Serializable {
public:
    //Construct a scan from it's identifying information.
    ScanHeader(unsigned int scanNumber, unsigned int runNumber, string moduleName, const ConfigurationVariable& variable) throw() ;

    ScanHeader() throw() ;		///< IO use only.
    
    /**
     * Deletes this object and any references it holds.
     */
    virtual ~ScanHeader() throw();

    //Streamable and Serializable overrides
    virtual string getClassName() const throw() ;
    virtual string getUniqueID() const throw();

    static string getUniqueID(unsigned int runNumber, unsigned int scanNumber, const string& moduleName) throw();

    /**
     * Retrieves the current status of this scan.
     * See the DataStatus class for more information.
     * @return The status - an item from the DataStatus class.
     */
    //const DataStatus& getStatus() const;

    /**
      @return the scan number.
    */
    unsigned int getScanNumber() const throw() {
        return scanNumber;
    }  		    
    
    /** @return the run number 
      */
    unsigned int getRunNumber() const throw() {
        return runNumber;
    } 
    
    /** @return the module name 
    */
    const string& getModuleName() const throw() {
        return moduleName;
    }  
    
    /** @return the scan type
    */
    const ConfigurationVariable& getVariable() const throw() {
        return *variable;
    }         

    /**
     * Equality operator.
     * @return true if scan has the same scan number, run number, module number and scan type as this one.
     */
    bool operator==(const ScanHeader& scan) const throw() ;


protected:
    unsigned int scanNumber; 				///<The scan number;
    unsigned int runNumber; 				///<The run number
    string moduleName; 			   	///<The moduleName.
    const ConfigurationVariable* variable; 		///<The variable that was modified in this scan.

    friend class ScanHeaderIOHelper;
};
}
#endif //#ifndef SCANHEADER_H
