#include "Comp.h"
#include "Sct/IS/IOManagerIS.h"
#include "Sct/Exception.h"
#include "Sct/LogicErrors.h"
#include "Sct/SctNames.h"
#include "Sct/SctParameters.h"
#include "SctData/FitScanResult.h"
#include "SctData/FitObject.h"
#include <is/isinfo.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <vector>
#include <TFile.h>
#include <TApplication.h>
#include <TGraph.h>

using namespace Sct;
using namespace Sct::IS;
using namespace boost;
using namespace SctData;
using namespace std;

bool debug = false;
double meanTol = 1;
double sigmaTol = 0.37;
string lastScanName = "FittedData.SctData::FitScanResult.551.53.SCTTestAPI_PseudoModule";

static TApplication& getRootApp() {
    //Create TApplication - necessary to prevent batch mode and no graphics on later ROOT versions
    static TApplication myapp("myapp", 0, 0);
    return myapp;
}

vector<shared_ptr<FitScanResult> > readAll() {
    vector<shared_ptr<FitScanResult> > fits;
        
    ISInfoIterator it( SctNames::getPartition(), "FittedData", ".*" );
    ISInfoDictionary dict(SctNames::getPartition());
    while (it()) {
	shared_ptr<Serializable> ob = IOManagerIS::instance().read(it);
	shared_ptr<FitScanResult> result = dynamic_pointer_cast<FitScanResult>(ob);
	if (!result) 
	    throw IllegalStateError(string("Failed to read fit: ") + it.name(), __FILE__, __LINE__);
	fits.push_back(result);
    }
    
    return fits;
}

TFile* loadSCTDAQData(FitScanResult& fit) {
    ostringstream oss;
    oss << "$SCT_DAQ_ROOT/installed/share/SctTestApiData/strun" << fit.getHeader().getRunNumber() << "_" << fit.getHeader().getScanNumber() << ".root";
    if (debug) cout << oss.str() << endl;    
    return new TFile(oss.str().c_str());
}


TH1F* getSigmaGraph(unsigned int link, TFile* f) {
    ostringstream oss;
    oss << "h_sigma" << link << ";" << 1;
    if (debug) cout << oss.str() << endl;
    return (TH1F*)f->Get(oss.str().c_str());
}

TH1F* getMeanGraph(unsigned int link, TFile* f) {
    ostringstream oss;
    oss << "h_mean" << link << ";" << 1;
    if (debug) cout << oss.str() << endl;    
    return (TH1F*)f->Get(oss.str().c_str());
}

TH1F* getChiSqGraph(unsigned int link, TFile* f) {
    ostringstream oss;
    oss << "h_chisq" << link << ";" << 1;
    if (debug) cout << oss.str() << endl;    
    return (TH1F*)f->Get(oss.str().c_str());
}


bool compareFits(FitScanResult& fit) {
    Comp cMean(meanTol);
    Comp cSigma(sigmaTol);
    int meanFailCount = 0;
    int sigmaFailCount = 0;
    TFile* f = loadSCTDAQData(fit);
    
    cout << endl << endl << "Comparing " << fit.getUniqueID() << endl;
    
    for (unsigned int link = 0; link<nLinkModule; ++link) {
	TH1F* gMean = getMeanGraph(link, f);
	TH1F* gSigma = getSigmaGraph(link, f);
	TH1F* gChiSq = getChiSqGraph(link, f);
	//Check numbers of points etc
	if (debug) cout << gMean->GetNbinsX() << endl;
	if (gMean->GetNbinsX() != nChannelLink) {
	    throw IoException("Incorrect data in TH1F", __FILE__, __LINE__);
	}	
	
	for (unsigned int i=0; i<nChannelLink; ++i) {
	    FitObject& fitOb = fit.getChannelFit(link, i);
	    //Note SCTDAQ stores in bin 0 - the underflow bin!
	    double mean = gMean->GetBinContent(i);
	    double sigma = gSigma->GetBinContent(i);	 
	    double chisq = gChiSq->GetBinContent(i);
	    int fitMeanIndex = fitOb.getParIndex("Mean");
	    int fitSigmaIndex = fitOb.getParIndex("Sigma");	    
	    double ourChiSq = fitOb.getChiSquared() / (fitOb.getNDF() + fitOb.getNPar());
	    
	    if (cMean.comp(fitOb.getParameter(fitMeanIndex), mean)) {
		cout << "Channel: " << (link*nChannelLink+i) << " Mean different. SCTDAQ: " << mean << " +- " 
		     << gMean->GetBinError(i) << " Chi/NdF: " << chisq << " Us: " << fitOb.getParameter(fitMeanIndex) 
		     << " +- " << fitOb.getParError(fitMeanIndex) << " Chi/NdF: " << ourChiSq << endl; 
		++meanFailCount;
	    } else if (debug) {
		cout << "Channel: " << (link*nChannelLink+i) << " Mean same. SCTDAQ: " << mean << " +- " 
		     << gMean->GetBinError(i) << " Chi/NdF: " << chisq << " Us: " << fitOb.getParameter(fitMeanIndex) 
		     << " +- " << fitOb.getParError(fitMeanIndex) << " Chi/NdF: " << ourChiSq << endl;  
	    }
	    
	    if (cSigma.comp(fitOb.getParameter(fitSigmaIndex), sigma)) {
		cout << "Channel: " << (link*nChannelLink+i) << " Sigma different. SCTDAQ: " << sigma << " +- " 
		     << gSigma->GetBinError(i) << " Chi/NdF: " << chisq << " Us: " << fitOb.getParameter(fitSigmaIndex) 
		     << " +- " << fitOb.getParError(fitSigmaIndex) << " Chi/NdF: " << ourChiSq <<endl; 
		
		++sigmaFailCount;
	    } else if (debug) {
		cout << "Channel: " << (link*nChannelLink+i) << " Sigma same. SCTDAQ: " << sigma << " +- " 
		     << gSigma->GetBinError(i) << " Chi/NdF: " << chisq << " Us: " << fitOb.getParameter(fitSigmaIndex) 
		     << " +- " << fitOb.getParError(fitSigmaIndex) << " Chi/NdF: " << ourChiSq <<endl; 
	    }
	    
	}
	delete gMean;
	delete gSigma;
    }
    f->Close();
    cout << "Summary.  " << endl << "Av Mean diff: " << cMean.mean() << " +- " << cMean.stddev() << endl;
    cout << "Av Sigma diff: " << cSigma.mean() << " +- " << cSigma.stddev() << endl;
    cout << meanFailCount << " channels failed mean test" << endl;
    cout << sigmaFailCount << " channels failed sigma test" << endl;
    return (meanFailCount + sigmaFailCount < 80);
}


bool compare(vector<shared_ptr<FitScanResult> > fits) {    
    bool retVal = true;
    for (unsigned int i=0; i<fits.size(); ++i) {
	if (!compareFits(*fits[i])) retVal = false;
    }
    return retVal;
}


void waitFor(string name, int nsecs) {
    ISInfoDictionary d(SctNames::getPartition());
    int tries = 0;
    while (!d.contains(name.c_str()) && tries < nsecs) {
	if (!sleep(1))
	    ++tries;
    }
    if (tries == nsecs) {
	throw IllegalStateError("Didn't wait long enough", __FILE__, __LINE__);
    }
    if (debug) {
	cout << "Waited for " << tries << " secs" << endl;
    }
}


int main(int argc, char** argv) {
    Sct::setExceptionHandlers(argv[0]);
    getRootApp();
    try {
	waitFor(lastScanName, 40);
	vector<shared_ptr<FitScanResult> > fits = readAll();
	
	if (compare(fits)) {
	    if (debug) cout << "Success!" << endl;
	    return 0;
	}
	else {
	    cout << "Failed" << endl;
	    return 1;
	}
	
    } catch (Throwable& e) {
	e.sendToMrs(MRS_FATAL);
	return 1;
    }
}
