//#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xinclude.h>

#include <fstream.h>

#include <mrs/message.h>
#include <is/isinfotmpl.h>

#include <stdexcept>

#include <time.h>


#include "configXMLImpl.h"

using namespace SctConfiguration;

/*
  Parse the xml configuration and do XInclude processing
  Configuration file loaded from environment variable SCTDAQ_ROD_CONFIGURATION_PATH
*/
ConfigurationXMLImpl::ConfigurationXMLImpl() : document(0), xpathContext(0), log("conf",2), lastFilename("") {
  const char *envname = "SCTDAQ_ROD_CONFIGURATION_PATH";

  char *filename = getenv(envname);

  document = 0;
  xpathContext = 0;

  if(filename) {
    loadConfiguration(filename);
  } else {
    std::cerr << "Configuration Environment: " << envname << " not set, don't know where to load config from\n";
    log << "Configuration Environment: " << envname << " not set, don't know where to load config from\n";
    throw ConfigurationException("No configuration");
  }

  IPCPartition partition("SCT");

  MRSStream mrsStream(partition);

  mrsStream << "Configuration running " << MRS_INFORMATION << MRS_QUALIF("sctConf") << ENDM;

//   mrsStream << "END_RUN" << MRS_INFORMATION << MRS_TEXT("End of run") << MRS_QUALIF("SCTAPI") << MRS_QUALIF("RUNCONTROL") << MRS_OPT_PARAM<int>("ErrorNumber", "Error Number is: %d", 2) << ENDM; 
//   mrsStream << "message1" << MRS_INFORMATION << MRS_QUALIF("A QUALIFIER") << MRS_QUALIF("FRONTEND") << MRS_PARAM<int>("param11",30) << MRS_PARAM<float>("param12",3.12) << MRS_OPT_PARAM<float>("Time", "Time is: %f PM", 2.1512) << ENDM; 

  ISInfoDictionary dict(partition);

  ISInfoString isFilename(filename);

//   ISInfoString getFilename;
//   dict.findValue("ConfigurationServer.Filename", getFilename);
//   std::cout << "Found " << getFilename.getValue() << " in IS\n";

  if(dict.contains("ConfigurationServer.Filename")) {
    dict.update("ConfigurationServer.Filename", isFilename);

    std::cout << "*** Filename already in IS, will run anyway but there could be another service running?\n";
  } else {
    dict.insert("ConfigurationServer.Filename", isFilename);
  }

  std::cout << "Put " << isFilename.getValue() << " into  IS\n";
}

void ConfigurationXMLImpl::modifyPowerParam(int MUR, int number, std::string name, float value) {
  log << "Change module's (" << MUR << ", " << number << ") Power Param (" << name << ") to " << value << std::endl;

  unsigned int partition, crate, channel;
  translateToPowerSupply(MUR, number, partition, crate, channel);

  char expression[1000];
  sprintf(expression, 
          "/configuration/power/crate[@id=\"%d\"]/channel[@id=\"%d\"]/param[@name=\"%s\"]", 
          crate, channel, name.c_str());

  xmlNodePtr paramNode;

  try {
    paramNode = getQueryNode(expression, "Modify power param");
  } catch(ConfigurationException &c) {
    // Create param node 
    xmlNodePtr root = xmlDocGetRootElement(document);
    xmlNodePtr power = getOrNewChildNode(root, "power");
    xmlNodePtr crateNode = getOrNewChildWithIntAttr(power, "crate", "id", crate);
    xmlNodePtr channelNode = getOrNewChildWithIntAttr(crateNode, "channel", "id", channel);
    paramNode = getOrNewChildNode(channelNode, "param");

    xmlSetProp(paramNode, (xmlChar *)"name", (xmlChar *)name.c_str());
  }

  char buffer[25];
  snprintf(buffer, 25, "%f", value);
  xmlNodeSetContentLen(paramNode, (xmlChar *)buffer, strlen(buffer));
}

void ConfigurationXMLImpl::modifyDefaultPowerParam(std::string name, float value) {
  log << "Change Default Power Param (" << name << ") to " << value << std::endl;

  char expression[1000];
  sprintf(expression, 
          "/configuration/power/defaults/param[@name=\"%s\"]", name.c_str());
  xmlNodePtr paramNode;

  try {
    paramNode = getQueryNode(expression, "Modify default power param");
  } catch(ConfigurationException &c) {
    // Create param node 
    xmlNodePtr root = xmlDocGetRootElement(document);
    xmlNodePtr power = getOrNewChildNode(root, "power");
    xmlNodePtr defaults = getOrNewChildNode(power, "defaults");
    paramNode = getOrNewChildNode(defaults, "param");

    xmlSetProp(paramNode, (xmlChar *)"name", (xmlChar *)name.c_str());
  }

  char buffer[25];
  snprintf(buffer, 25, "%f", value);
  xmlNodeSetContentLen(paramNode, (xmlChar *)buffer, strlen(buffer));
}

float ConfigurationXMLImpl::getPowerParam(int MUR, int number, std::string name) {
  log << "Get power param " << MUR << ", " << number << ", " << name << std::endl;

  char expression[1000]; 
  char *content;

  try {
    unsigned int partition, crate, channel;
    translateToPowerSupply(MUR, number, partition, crate, channel);

    sprintf(expression, 
            "/configuration/power/crate[@id=\"%d\"]/channel[@id=\"%d\"]/param[@name=\"%s\"]", 
            crate, channel, name.c_str());

    xmlNodePtr paramNode = getQueryNode(expression, "Get power param");

    content = (char *)xmlNodeGetContent(paramNode);
  } catch(ConfigurationException &c) {
    sprintf(expression, 
            "/configuration/power/defaults/param[@name=\"%s\"]", 
            name.c_str());
    
    xmlNodePtr paramNode = getQueryNode(expression, "Get default power param");

    content = (char *)xmlNodeGetContent(paramNode);
  }

  return strtod(content, NULL);
}

/*
  Load configuration from named file
*/
void ConfigurationXMLImpl::loadConfiguration(const std::string &fname) {
  std::string realFilename = fname;
  if(realFilename == "") {
    if(lastFilename != "") {
      realFilename = lastFilename;
    } else {
      log << "Passed bad filename \"\"!\n";
      std::cerr << "Bad filename!\n";
      throw ConfigurationException("Empty file name");
    }
  }

  log << "Load configuration from " << realFilename << std::endl;

  if(document) {
    // Delete XXX previous one first
  }

  document = xmlParseFile(realFilename.c_str());

  if(document) {
    //    processFileLinks(document);
    xmlXIncludeProcess(document);
    //    std::cout << "XIncludes : " << includeCount << std::endl; //processFileLinks(document);

    saveConfiguration();
  } else {
    log << "Configuration Document not parsed!\n";
    std::cerr << "Configuration Document not parsed!\n";
    throw ConfigurationException("File not found or bad file");
  }

  xmlXPathInit();
  if(xpathContext) {
    xmlXPathFreeContext(xpathContext);
  }
  xpathContext = xmlXPathNewContext(document);

  lastFilename = realFilename;
}

/*
  Free the configuration xml
*/
ConfigurationXMLImpl::~ConfigurationXMLImpl() {
  saveConfiguration();
  if(xpathContext) {
    xmlXPathFreeContext(xpathContext);
  }
  if(document) {
    xmlFreeDoc(document);
  }
  xmlCleanupParser();


  IPCPartition partition("SCT");

  ISInfoDictionary dict(partition);

  if(dict.contains("ConfigurationServer.Filename")) {
    dict.remove("ConfigurationServer.Filename");
  } else {
    std::cout << "*** No Filename in IS, don't need to remove it?\n";
  }
}

/*
  Lookup the partitions defined in the configuration and return a list of integer ID's
*/
std::list<unsigned int> ConfigurationXMLImpl::listPartitions() {
  log << "List partitions\n";

  std::list<unsigned int> result;

  char expression[1000];

  sprintf(expression, "/configuration/partition");

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
        xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

        try {
          int thisId = getAttrAsInt(curr, "id");
          result.push_back(thisId);
        } catch(ConfigurationException &c) {
          log << "This partition has no id attribute\n";
        }
      }
    } else {
      log << "Bad partition data\n";
    }

    xmlXPathFreeObject(queryResult);
  } else {
    log << "No result\n";
  }

  return result;
}

/*
  Lookup the crates defined in the specified partition and return a list of integer ID's
*/
std::list<unsigned int> ConfigurationXMLImpl::listCratesInPartition(unsigned int partition) {
  log << "List crates in partition " << partition << std::endl;

  std::list<unsigned int> result;

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate", partition);

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
        xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];
        try {
          result.push_back(getAttrAsInt(curr, "id"));
        } catch(ConfigurationException &c) {
          log << "This crate has no id attribute\n";
        }
      }
    } else {
      log << "Bad crate data\n";
    }

    xmlXPathFreeObject(queryResult);
  } else {
    log << "No result\n";
  }

  return result;
}

/*
  Lookup the RODs defined in the specified crate and return a list of integer ID's
*/
std::list<unsigned int> ConfigurationXMLImpl::listRodsInCrate(unsigned int partition, unsigned int crate) {
  log << "List rods in (" << partition << ", " << crate << ")\n";

  std::list<unsigned int> result;

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate[@id=%d]/rod", partition, crate);

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
        xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

        try {
          result.push_back(getAttrAsInt(curr, "id"));
        } catch(ConfigurationException &c) {
          log << "This rod has no id attribute\n";
        }
      }
    } else {
      log << "Bad ROD data\n";
    }
    xmlXPathFreeObject(queryResult);
  } else {
    log << "No result rod\n";
  }

  return result;
}

/*
  Return integer id's of the MURs in the specified rod.
*/
std::list<unsigned int> ConfigurationXMLImpl::listMURSInRod(unsigned int partition, unsigned int crate, unsigned int rod) {
  log << "List MURs in (" << partition << ", " << crate << ", " << rod << ")\n";
  std::list<unsigned int> result;

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]/MUR", partition, crate, rod);

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
        xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

        try {
          result.push_back(getAttrAsInt(curr, "id"));
        } catch(ConfigurationException &c) {
          log << "This MUR has no id attribute\n";
        }
      }
    } else {
      log << "Bad MUR data\n";
    }

    xmlXPathFreeObject(queryResult);
  } else {
    log << "No MUR result\n";
  }

  return result;
}

/*
  Return a list of serial numbers associated with the specified MUR
*/
std::list<std::string> ConfigurationXMLImpl::listModulesInMUR(unsigned int partition, unsigned int MUR) {
  log << "List modules in (" << partition << ", " << MUR << ")\n";
  std::list<std::string> result;

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate/rod/MUR[@id=%d]/module/text()", partition, MUR);

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      if(queryResult->nodesetval->nodeNr <= 6) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          result.push_back((char *)curr->content);
        }
      } else {
        log << "Config: Too many modules in MUR " << MUR << " (" << queryResult->nodesetval->nodeNr << ")\n";
      }
    } else {
      log << "Config: Can't find modules in MUR " << MUR << " of partition " << partition << "\n";
    }
    xmlXPathFreeObject(queryResult);
  } else {
    log << "No module/MUR result\n";
  }

  return result;
}

/*
  Return a list of serial numbers of all modules that have configuration

  This includes those that are not attached to an MUR 
*/
std::list<std::string> ConfigurationXMLImpl::listAllModules() {
  log << "List all modules\n";
  std::list<std::string> result;

  char expression[1000];

  sprintf(expression, "/configuration/modules/module/sn/text()");

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      if(queryResult->nodesetval->nodeNr > 0) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          result.push_back((char *)curr->content);
        }
      } else {
        log << "Config: Found 0 modules!\n";
      }
    } else {
      log << "Config: Can't find any modules!\n";
    }
    xmlXPathFreeObject(queryResult);
  } else {
    log << "No module/MUR result\n";
  }

  return result;
}

/*
  Return a list of serial numbers of modules that have configuration but are not attached to an MUR.
*/
std::list<std::string> ConfigurationXMLImpl::listUnusedModules() {
  log << "List unattached modules\n";
  std::list<std::string> result;

  char expression[1000];

  sprintf(expression, "/configuration/modules/module/sn/text()");

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      if(queryResult->nodesetval->nodeNr > 0) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          std::string sn = (char *)curr->content;

          try {
            unsigned int MUR, module;
            translateFromSN(sn, MUR, module);
          } catch(ConfigurationException &e) {
            result.push_back(sn);
          }
        }
      } else {
        log << "Config: Found 0 modules!\n";
      }
    } else {
      log << "Config: Can't find any modules!\n";
    }
    xmlXPathFreeObject(queryResult);
  } else {
    log << "No module/MUR result\n";
  }

  return result;
}

/*
  Return a list of serial numbers associated with the redundancy links to the specified MUR
*/
std::list<std::string> ConfigurationXMLImpl::listRModulesInMUR(unsigned int partition, unsigned int MUR) {
  log << "List rmodules in MUR (" << partition << ", " << MUR << ")\n";
  std::list<std::string> result;

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate/rod/MUR[@id=%d]/rmodule/text()", partition, MUR);

  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

  if(queryResult) {
    if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
      if(queryResult->nodesetval->nodeNr <= 6) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          result.push_back((char *)curr->content);
        }
      } else {
        log << "Config: Too many redundant modules in MUR " << MUR << " (" << queryResult->nodesetval->nodeNr << ")\n";
      }
    } else {
      log << "Config: Can't find redundant modules for MUR " << MUR << " of partition " << partition << "\n";
    }
    xmlXPathFreeObject(queryResult);
  } else {
    log << "No redundant module/MUR result\n";
  }

  return result;
}

/*
  Find the configuration associated with the specified ROD.
  Return a RodConfig structure.
*/
RodConfig ConfigurationXMLImpl::getRodConfig(unsigned int partition, unsigned int crate, unsigned int rod) {
  log << "Get Rod Configuration " << partition << ", " << crate << ", " << rod << std::endl;
  RodConfig result;

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]", partition, crate, rod);

  xmlNodePtr c = getQueryNode(expression, "ROD configuration"); // queryResult->nodesetval->nodeTab[0];

  result = parseRodConfig(c);

  return result;
}

/*
  Find the configuration associated with the specified TIM.
*/
TimConfig ConfigurationXMLImpl::getTimConfig(unsigned int partition, unsigned int crate) {
  log << "Get TIM Configuration " << partition << ", " << crate << std::endl;
  TimConfig result;

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate[@id=%d]/tim", partition, crate);

  xmlNodePtr c = getQueryNode(expression, "TIM configuration");

  result = parseTimConfig(c);

  return result;
}

MURType ConfigurationXMLImpl::getMURType(unsigned int MUR) {
  log << "Find type of MUR: " << MUR << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/geography/barrel/row/MUR[@id=%d]", MUR);

  try {
    getQueryNode(expression, "Check is Barrel");

    return BARREL;
  } catch (ConfigurationException &c) {
    // Not a barrel MUR
  }

  sprintf(expression, "/configuration/geography/endcap/quadrant/MUR[@id=%d]", MUR);

  try {
    getQueryNode(expression, "Translation to Endcap");

    return ENDCAP;
  } catch(ConfigurationException &c) {
    // Not an endcap MUR!
  }

  sprintf(expression, "/configuration//MUR[@id=%d]", MUR);

  try {
    getQueryNode(expression, "Is defined");

    return UNMAPPED;
  } catch(ConfigurationException &c) {
    // Not defined!
    return UNKNOWN;
  }
}

void ConfigurationXMLImpl::translateToROD(unsigned int MUR, unsigned int module,
                                   unsigned int &partition, unsigned int &crate,
                                   unsigned int &rod, unsigned int &channel) {
  log << "Translate to Rod " << MUR << ", " << module << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/partition/crate/rod/MUR[@id=%d]/module[@id=\"%d\"]", MUR, module);
  xmlNodePtr moduleNode = getQueryNode(expression, "Translate to ROD");

  // MUR order
  int order = getAttrAsInt(moduleNode->parent, "order");

  rod = getAttrAsInt(moduleNode->parent->parent, "id");
  crate = getAttrAsInt(moduleNode->parent->parent->parent, "id");
  partition = getAttrAsInt(moduleNode->parent->parent->parent->parent, "id");

  channel = order * 6 + module - 1;

  log << " Translate to Rod " << partition << ", " << crate << ", " << rod << ", " << channel << "\n";
}

void ConfigurationXMLImpl::translateFromROD(unsigned int partition, unsigned int crate,
                                     unsigned int rod, unsigned int channel,
                                     unsigned int &MUR, unsigned int &module) {
  log << "Translate from Rod " << partition << ", " << crate << ", " << rod << ", " << channel << "\n";

  unsigned int order = channel / 6;
  unsigned int id = (channel % 6) + 1;

  char expression[1000];
  sprintf(expression,
          "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]/MUR[@order=%d]",
          partition, crate, rod, order);

  xmlNodePtr moduleNode = getQueryNode(expression, "Translation from ROD");

  module = id; // getAttrAsInt(moduleNode, "id");
  MUR = getAttrAsInt(moduleNode, "id");

  log << " Translate from Rod " << MUR << ", " << module << std::endl;
}

void ConfigurationXMLImpl::translateToRROD(unsigned int MUR, unsigned int module,
                                    unsigned int &partition, unsigned int &crate,
                                    unsigned int &rod, unsigned int &channel) {
  log << "Translate to redundant Rod " << MUR << ", " << module << std::endl;

  std::string sn;

  translateToSN(MUR, module, sn);

  char expression[1000];
  sprintf(expression, "/configuration/partition/crate/rod/MUR/rmodule[text()=\"%s\"]", sn.c_str());
  xmlNodePtr moduleNode = getQueryNode(expression, "Translate to Redundant ROD");

  // MUR order
  module = getAttrAsInt(moduleNode, "id");
  int order = getAttrAsInt(moduleNode->parent, "order");

  rod = getAttrAsInt(moduleNode->parent->parent, "id");
  crate = getAttrAsInt(moduleNode->parent->parent->parent, "id");
  partition = getAttrAsInt(moduleNode->parent->parent->parent->parent, "id");

  channel = order * 6 + module - 1;

  log << " Translate to redundant Rod " << partition << ", " << crate << ", " << rod << ", " << channel << "\n";
}

void ConfigurationXMLImpl::translateFromRROD(unsigned int partition, unsigned int crate,
                                      unsigned int rod, unsigned int channel,
                                      unsigned int &MUR, unsigned int &module) {
  log << "Translate from redundant Rod " << partition << ", " << crate << ", " << rod << ", " << channel << "\n";

  unsigned int order = channel / 6;
  unsigned int id = (channel % 6) + 1;

  char expression[1000];
  sprintf(expression,
          "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]/MUR[@order=%d]/rmodule[@id=%d]/text()",
          partition, crate, rod, order, id);

  xmlNodePtr moduleNode = getQueryNode(expression, "Translation from Redundant ROD");

//   std::cout << (char *)(moduleNode->content) << std::endl;

  translateFromSN((char *)moduleNode->content, MUR, module);

  log << " Translate from redundant Rod " << MUR << ", " << module << "\n";
}

void ConfigurationXMLImpl::translateToSN(unsigned int MUR, unsigned int module,
                                  std::string &sn) {
  log << "Translate to serial number " << MUR << ", " << module << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/partition/crate/rod/MUR[@id=%d]/module[@id=\"%d\"]/text()", MUR, module);
  xmlNodePtr moduleNode = getQueryNode(expression, "Translation to SN");

  sn = (char *)moduleNode->content;

  log << " Translate to serial number " << sn << std::endl;
}

void ConfigurationXMLImpl::translateFromSN(std::string sn,
                                    unsigned int &MUR, unsigned int &module) {
  log << "Translate from serial number " << sn << "\n";

  char expression[1000];
  sprintf(expression, "/configuration/partition/crate/rod/MUR/module[text()=\"%s\"]", sn.c_str());

  xmlNodePtr moduleNode = getQueryNode(expression, "Translation from SN");

  module = getAttrAsInt(moduleNode, "id");
  MUR = getAttrAsInt(moduleNode->parent, "id");

  log << " Translate From serial number " << MUR << ", " << module << std::endl;
}

void ConfigurationXMLImpl::translateToBarrel(unsigned int MUR, unsigned int module,
                                      unsigned int &barrel, unsigned int &row, int &number) {
  log << "Translate to Barrel " << MUR << ", " << module << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/geography/barrel/row/MUR[@id=%d]", MUR);
  xmlNodePtr moduleNode = getQueryNode(expression, "Translation to Barrel");

  int position = getAttrAsInt(moduleNode, "position");
  row = getAttrAsInt(moduleNode->parent, "id");
  barrel = getAttrAsInt(moduleNode->parent->parent, "id");
  number = position * module;

  log << " Translate to Barrel " << barrel << ", " << row << ", " << number << std::endl;
}

void ConfigurationXMLImpl::translateFromBarrel(unsigned int barrel, unsigned int row, int number,
                                        unsigned int &MUR, unsigned int &module) {
  log << "Translate from barrel " << barrel << ", " << row << ", " << number << "\n";

  int position;

  if(number < 0) {
    position = -1;
  } else {
    position = 1;
  }

  char expression[1000];
  sprintf(expression,
          "/configuration/geography/barrel[@id=%d]/row[@id=%d]/MUR[@position=%d]",
          barrel, row, position);

  xmlNodePtr moduleNode = getQueryNode(expression, "Translation from Barrel");

  module = (unsigned int)(number * position);
  MUR = getAttrAsInt(moduleNode, "id");

  log << " Translate from Barrel " << MUR << ", " << module << std::endl;
}

/* Endcap Internals:
   position of MUR in quadrant
   number is left to right top to bottom (from 0)
      0000001222222   (0-12)
        4444455555    (13-22)
        1111133333    (23-32)
*/
void ConfigurationXMLImpl::translateToEndcap(unsigned int MUR, unsigned int module,
                                             unsigned int &disk, unsigned int &ring, unsigned int &number) {
  log << "Translate to Endcap " << MUR << ", " << module << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/geography/endcap/quadrant/MUR[@id=%d]", MUR);
  xmlNodePtr moduleNode = getQueryNode(expression, "Translation to Endcap");

  int position = getAttrAsInt(moduleNode, "position");
  ring = getAttrAsInt(moduleNode->parent, "id");
  disk = getAttrAsInt(moduleNode->parent->parent, "id");

  switch(position) {
  case 0: number = module; break;
  case 1: number = (module==0)?6:module + 22; break;
  case 2: number = module + 7; break;
  case 3: number = module + 27; break;
  case 4: number = module + 12; break;
  case 5: number = module + 17; break;
  }

  log << " Translate to endcap " << disk << ", " << ring << ", " << number << "\n";
}

void ConfigurationXMLImpl::translateFromEndcap(unsigned int disk, unsigned int ring, unsigned int number,
                                        unsigned int &MUR, unsigned int &module) {
  int position;

  log << "Translate from endcap " << disk << ", " << ring << ", " << number << "\n";

  if(number < 6) {
    position = 0; module = number;
  } else if(number == 6) {
    position = 1; module = 0;
  } else if(number < 13) {
    position = 2; module = number - 7;
  } else if(number < 18) {
    position = 4; module = number - 12;
  } else if(number < 23) {
    position = 5; module = number - 17;
  } else if(number < 28) {
    position = 1; module = number - 22;
  } else if(number < 33) {
    position = 3; module = number - 27;
  } else {
    log << "Number too big in translate from endcap\n";
    throw ConfigurationException("Number to big in translate from endcap");
  }

  char expression[1000];
  sprintf(expression,
          "/configuration/geography/endcap[@id=%d]/quadrant[@id=%d]/MUR[@position=%d]",
          disk, ring, position);

  xmlNodePtr moduleNode = getQueryNode(expression, "Translation from Endcap");

  MUR = getAttrAsInt(moduleNode, "id");

  log << " Translate from Endcap " << MUR << ", " << module << std::endl;
}

void ConfigurationXMLImpl::translateToPowerSupply(unsigned int MUR, unsigned int module,
                                           unsigned int &partition, 
                                           unsigned int &crate, unsigned int &channel) {  
  log << "Translate to Power Supply " << MUR << ", " << module << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/power/crate/channel[@MUR=%d and @module=%d]", MUR, module);
  xmlNodePtr moduleNode = getQueryNode(expression, "Translation to Power Supply");

  channel = getAttrAsInt(moduleNode, "id");
  crate = getAttrAsInt(moduleNode->parent, "id");
  partition = 0; // getAttrAsInt(moduleNode->parent->parent, "id");

  log << " Translate to Power Supply " << partition << ", " << crate << ", " << channel << std::endl;
}

void ConfigurationXMLImpl::translateFromPowerSupply(unsigned int partition, 
                              unsigned int crate, unsigned int channel, 
                              unsigned int &MUR, unsigned int &module) {
  log << "Translate from Power Supply " << partition << ", " << crate << ", " << channel << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/power/crate[@id=%d]/channel[@id=%d]", crate, channel);
  xmlNodePtr moduleNode = getQueryNode(expression, "Translation from Power Supply");

  MUR = getAttrAsInt(moduleNode, "MUR");
  module = getAttrAsInt(moduleNode, "module");

  log << " Translate from Power Supply " << MUR << ", " << module << std::endl;
}


/*
  Return the configuration for the given module
*/
ABCDModule ConfigurationXMLImpl::getModuleConfig(std::string module) {
  log << "Get module configuration " << module << std::endl;
  
  char expression[1000];

  sprintf(expression, "/configuration/modules/module[sn=\"%s\"]", module.c_str());

  xmlNodePtr moduleNode = getQueryNode(expression, "Module Config");

  return parseModuleConfig(moduleNode);
}

/*
  Parse the xml tree provided and return the module configuration
*/
const ABCDModule ConfigurationXMLImpl::parseModuleConfig(xmlNodePtr moduleNode) {
  ABCDModule result;

  for(unsigned int i=0; i<sizeof(result); i++) {
    ((char*)&result)[i] = 0;
  }

  if(moduleNode) {
    //    xmlElemDump(stdout, document, moduleNode);

    for(xmlNodePtr child = moduleNode->children;
        child;
        child = child->next) {
      if(child->type == XML_ELEMENT_NODE) {
        char * content = (char *)xmlNodeGetContent(child);

        //        std::cout << "Tag in module " << (char *)child->name << std::endl;;

        if(strcmp((char *)child->name, "active") == 0) {
          result.active = atoi(content);
        } else if(strcmp((char *)child->name, "group") == 0) {
          result.groupId = atoi(content);
        } else if(strcmp((char *)child->name, "select") == 0) {
          result.select = atoi(content);
        } else if(strcmp((char *)child->name, "config") == 0) {
          log << "Using old active/select format\n";
          // Old config format
          if(xmlGetProp(child, (xmlChar *)"active") &&
             xmlGetProp(child, (xmlChar *)"select")) {
            result.active = atoi((char *)xmlGetProp(child, (xmlChar *)"active"));
            result.select = atoi((char *)xmlGetProp(child, (xmlChar *)"select"));
          } else {
            log << "module config without active or select tags\n";
          }
        } else if(strcmp((char *)child->name, "chip") == 0) {
          try {
            int id = getAttrAsInt(child, "id", 10);   // This is stored with a leading 0 so it shouldn't be octal!!!
            int active = getAttrAsInt(child, "active");
            int address = getAttrAsInt(child, "address");

            result.chip[id].active = active;
            result.chip[id].address = address;

            for(xmlNodePtr chipChild = child->children;
                chipChild;
                chipChild = chipChild->next) {
              char * chipContent = (char *)xmlNodeGetContent(chipChild);

              //            std::cout << "Tag in chip " << (char *)chipChild->name << std::endl;

              if(chipChild->type != XML_ELEMENT_NODE) {
              } else if(strcmp((char *)chipChild->name, "config") == 0) {
                char *eptr = chipContent;
                result.chip[id].basic.config.padding = 0;
                for(int i=0; i<11; i++) {
                  int value;
                  char *cptr = eptr;
                  value = strtol(cptr, &eptr, 0);
//                perror("Error converting");
                  switch(i) {
                  case 0: result.chip[id].basic.config.readoutMode = value; break;
                  case 1: result.chip[id].basic.config.calibMode = value; break;
                  case 2: result.chip[id].basic.config.trimRange = value; break;
                  case 3: result.chip[id].basic.config.edgeDetect = value; break;
                  case 4: result.chip[id].basic.config.mask = value; break;
                  case 5: result.chip[id].basic.config.accumulate = value; break;
                  case 6: result.chip[id].basic.config.inputBypass = value; break;
                  case 7: result.chip[id].basic.config.outputBypass = value; break;
                  case 8: result.chip[id].basic.config.master = value; break;
                  case 9: result.chip[id].basic.config.end = value; break;
                  case 10: result.chip[id].basic.config.feedThrough = value; break;
                  }
                }
              } else if(strcmp((char *)chipChild->name, "mask") == 0) {
                // Should be more robust?
                char *eptr = chipContent;
                for(int m=0; m<4; m++) {
                  char *cptr = eptr;
                  result.chip[id].basic.mask[m] = strtoul(cptr, &eptr, 0);
                }
              } else if(strcmp((char *)chipChild->name, "reg") == 0) {
                try {
                  std::string name = getAttrAsString(chipChild, "name");

                  if(strcmp(name.c_str(), "vthr") == 0) {
                    result.chip[id].basic.vthr = strtol(chipContent, NULL, 0);
                  } else if(strcmp(name.c_str(), "vcal") == 0) {
                    result.chip[id].basic.vcal = strtol(chipContent, NULL, 0);
                  } else if(strcmp(name.c_str(), "delay") == 0) {
                    result.chip[id].basic.delay = strtol(chipContent, NULL, 0);
                  } else if(strcmp(name.c_str(), "preamp") == 0) {
                    result.chip[id].basic.preamp = strtol(chipContent, NULL, 0);
                  } else if(strcmp(name.c_str(), "shaper") == 0) {
                    result.chip[id].basic.shaper = strtol(chipContent, NULL, 0);
                  } else {
                    log << "Unknown tag for reg value " << name << std::endl;
                  }
                } catch(ConfigurationException &c) {
                  log << "chip reg with no name attribute\n";
                }
              } else if(strcmp((char *)chipChild->name, "rc_function") == 0) {
                try {
                  result.chip[id].caldata.rc_function = getAttrAsInt(chipChild, "type");

                  char *token = strtok(chipContent, " ");
                  int nextVal = -1;
                  do {
                    if(strcmp(token, "p0") == 0) {
                      nextVal = 0;
                    } else if(strcmp(token, "p1") == 0) {
                      nextVal = 1;
                    } else if(strcmp(token, "p2") == 0) {
                      nextVal = 2;
                    } else {
                      if(nextVal < 0) {
                        log << "Bad rc function parameters " << token << std::endl;
                      } else {
                        result.chip[id].caldata.rc_params[nextVal] = strtod(token, NULL);
                      }
                      nextVal = -1;
                    }
                  } while((token = strtok(NULL, " ")) != 0);
                } catch(ConfigurationException &c) {
                  log << "No type attribute on rc_function\n";
                }
              } else if(strcmp((char *)chipChild->name, "c_factor") == 0) {
                result.chip[id].caldata.c_factor = strtod(chipContent, NULL);
              } else if(strcmp((char *)chipChild->name, "target") == 0) {
                double val = strtod(chipContent, NULL);
                result.chip[id].target = ((int)((val/2.5)+0.5));
              } else if(strcmp((char *)chipChild->name, "trim") == 0) {
                try {
                  int start = getAttrAsInt(chipChild, "start", 16);
                  int end = getAttrAsInt(chipChild, "end", 16);

                  int length = end-start+1;

                  if((length%8) != 0) log << "Can't interpret trim values not in blocks of 8\n";
                  else {
                    char *endPtr = chipContent;
                    char *startPtr = chipContent;
                    int curr = start;
                    do {
                      startPtr = endPtr;
                      unsigned long trimValues = strtoul(startPtr, &endPtr, 0);

                      if(curr > end) {
                        if(startPtr != endPtr) {
                          log << "Too many numbers in trim (range " << start << "-" << end << ")\n";
                        }
                        break;
                      }

                      for(int i=0; i<8; i++) {
                        unsigned char val = (trimValues >> ((7-i)*4)) & 0x0000000f;

                        result.chip[id].trim[curr+i] = val;
//                          if(val>9) val += 'a'-10;
//                          else val += '0';
                      }
                      curr += 8;
                    } while(*endPtr!=0);
                  }
                } catch(ConfigurationException &c) {
                  log << "No start and end tags on trim\n";
                } // End trim attribute check
              } // End if over chip tags

              free(chipContent);
            } // End loop over chip tags
          } catch(ConfigurationException &c) {
            log << "Bad attributes to chip\n";
          } // End chip attribute check
        } else {
//           std::cout << "Unrecognised tag in module " << child->name << "\n";
        } // End if over module tags

        free(content);
      } // Tag not an element
    } // End loop over module tags
  } else {
    log << "Module configuration not found\n";
    throw ConfigurationException("Module configuration not found");
  }

  return result;
}

char *ConfigurationXMLImpl::getFibreMappings(unsigned int partition, unsigned int crate, unsigned int rod) {
  log << "Get BOC fibre mappings\n";
  
  char *result = new char[48*3];

  for(int channel = 0; channel < 48; channel++) {
    char expression[1000];

    int order = channel/6;
    int chan = (channel % 6) + 1;

    // Defaults
    result[channel * 3 + 0] = channel;
    result[channel * 3 + 1] = channel * 2;
    result[channel * 3 + 2] = channel * 2 + 1;

    sprintf(expression,
            "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]/MUR[@order=%d]/channel[@id=%d]",
            partition, crate, rod, order, chan);

    try {
      xmlNodePtr c = getQueryNode(expression, "BOC Channel mappings");
      
      xmlNodePtr node;
      try {
        node = getChildNode(c, "output");
        result[channel * 3 + 0] = getAttrAsInt(node, "fibre");
      } catch(ConfigurationException &c) {
        // Leave as default
      }

      node = getChildNode(c, "stream0");
      try {
        result[channel * 3 + 1] = getAttrAsInt(node, "fibre");
      } catch(ConfigurationException &c) {
        //        std::cout << "Bad stream0 tag in channel " << channel << std::endl;
      }

      node = getChildNode(c, "stream1");
      try {
        result[channel * 3 + 2] = getAttrAsInt(node, "fibre");
      } catch(ConfigurationException &c) {
        //        std::cout << "Bad stream1 tag in channel " << channel << std::endl;
      }
    } catch(ConfigurationException &c) {
      //      std::cout << "Bad configuration for channel " << channel << std::endl;
    }
  }

  return result;
}

void ConfigurationXMLImpl::setFibreMapping(unsigned int partition, unsigned int crate, unsigned int rod, 
                                           unsigned int channel, unsigned int tx, unsigned int rx0, unsigned int rx1) {
  log << "Set BOC fibre mappings for " << partition << ", " << crate << ", " << rod << ", " << channel << "\n";

  unsigned int MUR, module;
  translateFromROD(partition, crate, rod, channel, MUR, module);

  char expression[1000];

  sprintf(expression,
          "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]/MUR[@id=%d]",
          partition, crate, rod, MUR);

  // If this fails just propogate the exception
  xmlNodePtr murNode = getQueryNode(expression, "Set fibre mapping");

  xmlNodePtr channelNode = getOrNewChildWithIntAttr(murNode, "channel", "id", module);

  char buffer[1000];

  xmlNodePtr outputNode = getOrNewChildNode(channelNode, "output");
  setIntAttr(outputNode, "fibre", tx);

  xmlNodePtr stream0 = getOrNewChildNode(channelNode, "stream0");
  setIntAttr(stream0, "fibre", rx0);

  xmlNodePtr stream1 = getOrNewChildNode(channelNode, "stream1");
  setIntAttr(stream1, "fibre", rx1);
}

BOCChannelConfig ConfigurationXMLImpl::getBOCConfig(unsigned int partition, unsigned int crate, unsigned int rod,
                                                    unsigned int channel) {
  log << "Get BOC configuration\n";
  
  BOCChannelConfig result;

  char expression[1000];

  int order = channel/6;
  int chan = (channel % 6) + 1;

  sprintf(expression,
          "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]/MUR[@order=%d]/channel[@id=%d]",
          partition, crate, rod, order, chan);

  xmlNodePtr c = getQueryNode(expression, "BOC Channel Configuration");

  xmlNodePtr node = getChildNode(c, "output");

  result.current = getAttrAsInt(node, "current");
  result.delay = getAttrAsInt(node, "delay");
  result.markSpace = getAttrAsInt(node, "markSpace");
  try {
    result.txFibre = getAttrAsInt(node, "fibre");
  } catch(ConfigurationException &e) {
    result.txFibre = channel;
  }
  node = getChildNode(c, "stream0");

  result.threshold0 = getAttrAsInt(node, "threshold");
  result.delay0 = getAttrAsInt(node, "delay");
  try {
    result.rxFibre0 = getAttrAsInt(node, "fibre");
  } catch(ConfigurationException &e) {
    result.rxFibre0 = channel*2;
  }

  node = getChildNode(c, "stream1");

  result.threshold1 = getAttrAsInt(node, "threshold");
  result.delay1 = getAttrAsInt(node, "delay");
  try {
    result.rxFibre1 = getAttrAsInt(node, "fibre");
  } catch(ConfigurationException &e) {
    result.rxFibre1 = channel*2+1;
  }

  return result;
}

void ConfigurationXMLImpl::mapBarrelMUR(int MUR, int barrel, int row, int position) {
  char expression[1000];

  // Check if MUR already in map
  sprintf(expression,
          "/configuration/geography/barrel/row/MUR[@id=%d]",
          MUR);

  try {
    xmlNodePtr node = getQueryNode(expression, "Check MUR for remap");

    xmlUnlinkNode(node);
    xmlFreeNode(node);
  } catch (ConfigurationException &e) {
    // This is OK, we don't have to remove it
  }

  // Get barrel and row
  sprintf(expression,
          "/configuration/geography/barrel[@id=%d]/row[@id=%d]",
          barrel, row);

  xmlNodePtr rowNode = 0;
  try {
    rowNode = getQueryNode(expression, "Find BOC Channel Configs (Set)");

    // If MUR with this position there then delete it
    for(xmlNodePtr murNode = rowNode->children;
        murNode;
        murNode = murNode->next) {

      if(murNode->type != XML_ELEMENT_NODE) {
      } else if(getAttrAsInt(murNode, "position") == position) {
        xmlUnlinkNode(murNode);
        xmlFreeNode(murNode);
      }
    }
  } catch(ConfigurationException &e) {
    xmlNodePtr root = xmlDocGetRootElement(document);

    xmlNodePtr geog = getOrNewChildNode(root, "geography");

    xmlNodePtr barrelNode = 0;

    barrelNode = getNodeWithIntAttr(geog, "barrel", "id", barrel);

    if(!barrelNode) {
      barrelNode = getOrNewChildNode(geog, "barrel");

      char buffer[5];
      snprintf(buffer, 5, "%d", barrel);
      xmlSetProp(barrelNode, (xmlChar *)"id", (xmlChar *)buffer);
    }

    rowNode = getNodeWithIntAttr(barrelNode, "row", "id", row);

    if(!rowNode) {
      rowNode = xmlNewChild(barrelNode, NULL, (xmlChar*)"row", NULL);

      char buffer[5];
      snprintf(buffer, 5, "%d", row);
      xmlSetProp(rowNode, (xmlChar *)"id", (xmlChar *)buffer);
    }
  }

  if(rowNode) {
    // MUR insert
    xmlNodePtr newMUR = xmlNewChild(rowNode, NULL, (xmlChar *)"MUR", NULL);

    char buffer[5];
    snprintf(buffer, 5, "%d", MUR);
    xmlSetProp(newMUR, (xmlChar *)"id", (xmlChar *)buffer);
    snprintf(buffer, 5, "%d", position);
    xmlSetProp(newMUR, (xmlChar *)"position", (xmlChar *)buffer);
  }
}

void ConfigurationXMLImpl::mapEndcapMUR(int MUR, int disk, int quadrant, int position) {
  char expression[1000];

  // Check if MUR already in map
  sprintf(expression,
          "/configuration/geography/endcap/quadrant/MUR[@id=%d]",
          MUR);

  try {
    xmlNodePtr node = getQueryNode(expression, "Check Endcap MUR for remap");

    xmlUnlinkNode(node);
    xmlFreeNode(node);
  } catch (ConfigurationException &e) {
    // This is OK, we don't have to remove it
  }

  // Get barrel and row
  sprintf(expression,
          "/configuration/geography/endcap[@id=%d]/quadrant[@id=%d]",
          disk, quadrant);

  xmlNodePtr quadrantNode = 0;
  try {
    quadrantNode = getQueryNode(expression, "Find BOC Channel Configs (Set)");

    // If MUR with this position there then delete it
    for(xmlNodePtr murNode = quadrantNode->children;
        murNode;
        murNode = murNode->next) {

      if(murNode->type != XML_ELEMENT_NODE) {
      } else if(getAttrAsInt(murNode, "position") == position) {
        xmlUnlinkNode(murNode);
        xmlFreeNode(murNode);
      }
    }
  } catch(ConfigurationException &e) {
    xmlNodePtr root = xmlDocGetRootElement(document);

    xmlNodePtr geog = getOrNewChildNode(root, "geography");

    xmlNodePtr endcapNode = 0;

    endcapNode = getNodeWithIntAttr(geog, "endcap", "id", disk);

    if(!endcapNode) {
      endcapNode = getOrNewChildNode(geog, "endcap");

      char buffer[5];
      snprintf(buffer, 5, "%d", disk);
      xmlSetProp(endcapNode, (xmlChar *)"id", (xmlChar *)buffer);
    }

    quadrantNode = getNodeWithIntAttr(endcapNode, "quadrant", "id", quadrant);

    if(!quadrantNode) {
      quadrantNode = xmlNewChild(endcapNode, NULL, (xmlChar*)"quadrant", NULL);

      char buffer[5];
      snprintf(buffer, 5, "%d", quadrant);
      xmlSetProp(quadrantNode, (xmlChar *)"id", (xmlChar *)buffer);
    }
  }

  if(quadrantNode) {
    // MUR insert
    xmlNodePtr newMUR = xmlNewChild(quadrantNode, NULL, (xmlChar *)"MUR", NULL);

    char buffer[5];
    snprintf(buffer, 5, "%d", MUR);
    xmlSetProp(newMUR, (xmlChar *)"id", (xmlChar *)buffer);
    snprintf(buffer, 5, "%d", position);
    xmlSetProp(newMUR, (xmlChar *)"position", (xmlChar *)buffer);
  }
}

void ConfigurationXMLImpl::swapMURNames(int MUR1, int MUR2) {
  // Find everything to do with MUR1
  std::list<xmlNodePtr> idPointers1;

  {
    char expression[1000];
    sprintf(expression, "//MUR[@id=%d]", MUR1);
  
    xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

    if(queryResult) {
      if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          idPointers1.push_back(curr);
        }
      } else {
        log << "Bad query result data\n";
      }

      xmlXPathFreeObject(queryResult);
    } else {
      log << "No id pointers to MUR1\n";
    }
  }

  // Power mappings have channel elements with MUR attribute!
  std::list<xmlNodePtr> channelPointers1;

  {
    char expression[1000];
    sprintf(expression, "//channel[@MUR=%d]", MUR1);
  
    xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

    if(queryResult) {
      if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          channelPointers1.push_back(curr);
        }
      } else {
        log << "Bad query result data\n";
      }

      xmlXPathFreeObject(queryResult);
    } else {
      log << "No id pointers to MUR1\n";
    }
  }

  // Find everything to do with MUR2
  std::list<xmlNodePtr> idPointers2;

  {
    char expression[1000];
    sprintf(expression, "//MUR[@id=%d]", MUR2);
  
    xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

    if(queryResult) {
      if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          idPointers2.push_back(curr);
        }
      } else {
        log << "Bad query result data\n";
      }

      xmlXPathFreeObject(queryResult);
    } else {
      log << "No id pointers to MUR1\n";
    }
  }

  // Power mappings have channel elements with MUR attribute!
  std::list<xmlNodePtr> channelPointers2;

  {
    char expression[1000];
    sprintf(expression, "//channel[@MUR=%d]", MUR2);
  
    xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)expression, xpathContext);

    if(queryResult) {
      if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
        for(int i=0; i<queryResult->nodesetval->nodeNr; i++) {
          xmlNodePtr curr = queryResult->nodesetval->nodeTab[i];

          channelPointers2.push_back(curr);
        }
      } else {
        log << "Bad query result data\n";
      }

      xmlXPathFreeObject(queryResult);
    } else {
      log << "No id pointers to MUR1\n";
    }
  }

  // Write over MUR1 with MUR2
  for(std::list<xmlNodePtr>::const_iterator pointer = idPointers1.begin();
      pointer != idPointers1.end();
      pointer ++) {
    setIntAttr(*pointer, "id", MUR2);
  }

  for(std::list<xmlNodePtr>::const_iterator pointer = channelPointers1.begin();
      pointer != channelPointers1.end();
      pointer ++) {
    setIntAttr(*pointer, "MUR", MUR2);
  }

  // Write over MUR2 with MUR1
  for(std::list<xmlNodePtr>::const_iterator pointer = idPointers2.begin();
      pointer != idPointers2.end();
      pointer ++) {
    setIntAttr(*pointer, "id", MUR1);
  }

  for(std::list<xmlNodePtr>::const_iterator pointer = channelPointers2.begin();
      pointer != channelPointers2.end();
      pointer ++) {
    setIntAttr(*pointer, "MUR", MUR1);
  }

}

void ConfigurationXMLImpl::mapPowerChannel(int MUR, int number, unsigned int partition, unsigned int crate, unsigned int channel) {
  char expression[1000];

  // Get crate to put channel in
  sprintf(expression, "/configuration/power/crate[@id=%d]/channel[@id=%d]", crate, channel);

  xmlNodePtr channelNode = 0;
  try {
    channelNode = getQueryNode(expression, "Map Power supply channel");

#warning "What happens to old channel mapping + params?"

//     xmlNodePtr channelNode = getNodeWithIntAttr(crateNode, "channel", "id", channel);

//    // If MUR with this position there then delete it
//     for(xmlNodePtr murNode = crateNode->children;
//         murNode;
//         murNode = murNode->next) {

//       if(murNode->type != XML_ELEMENT_NODE) {
//       } else if(getAttrAsInt(murNode, "MUR") == MUR && getAttrAsInt(murNode, "module") == number) {
//         xmlUnlinkNode(murNode);
//         xmlFreeNode(murNode);
//       }
//     }
  } catch(ConfigurationException &e) {
    xmlNodePtr root = xmlDocGetRootElement(document);

    xmlNodePtr power = getOrNewChildNode(root, "power");

    //    xmlNodePtr partitionNode = getOrNewChildWithIntAttr(power, "partition", "id", partition);
    xmlNodePtr crateNode = getOrNewChildWithIntAttr(power /* partitionNode */, "crate", "id", crate);
    channelNode = getOrNewChildWithIntAttr(crateNode, "channel", "id", channel);
  }

  setIntAttr(channelNode, "MUR", MUR);
  setIntAttr(channelNode, "module", number);
#warning "Untested!"
}

void ConfigurationXMLImpl::mapRODMUR(int partition, int crate, int rod, int order, int number) {
  createMUR(partition, crate, rod, order, number);

  return;
// }

#warning "Partially implemented! (what if already there...)"
  char expression[1000];

  // Check if MUR already in map
  sprintf(expression, "/configuration/partition/crate/rod/MUR[@id=%d]", number);

  xmlNodePtr node = 0;
  try {
    node = getQueryNode(expression, "Check MUR for remap");
//     xmlUnlinkNode(node);
//     xmlFreeNode(node);
  } catch (ConfigurationException &e) {
    // This is OK, we don't have to remove it
  }

  if(node) {
#warning "What if there's useful stuff in it!"
    throw ConfigurationException("There's already an MUR with that number");
  }

  // Get ROD 
  sprintf(expression, "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]", partition, crate, rod);

  xmlNodePtr rodNode = 0;
  try {
    rodNode = getQueryNode(expression, "Find ROD for MUR Map");

    // If MUR with this position there then delete it
#warning "What if there's useful stuff in it!"
    for(xmlNodePtr murNode = rodNode->children;
        murNode;
        murNode = murNode->next) {

      if(murNode->type != XML_ELEMENT_NODE) {
      } else if(getAttrAsInt(murNode, "order") == order) {
        xmlUnlinkNode(murNode);
        xmlFreeNode(murNode);
      }
    }
  } catch(ConfigurationException &e) {
    // XXX Should create the ROD
    throw;
  }

  if(rodNode) {
    // MUR insert
    xmlNodePtr newMUR = xmlNewChild(rodNode, NULL, (xmlChar *)"MUR", NULL);

    char buffer[5];
    snprintf(buffer, 5, "%d", number);
    xmlSetProp(newMUR, (xmlChar *)"id", (xmlChar *)buffer);
    snprintf(buffer, 5, "%d", order);
    xmlSetProp(newMUR, (xmlChar *)"order", (xmlChar *)buffer);
  }

}

void ConfigurationXMLImpl::printModuleConfig(const ABCDModule &conf) {
  log << "Print module configuration\n";

  std::cout << "Module global settings\n";
  std::cout << " Active " << (int)conf.active << " Select " << (int)conf.select << " Group " << (int)conf.groupId << std::endl;
  std::cout << " TX fibre " << (int)conf.pTTC << " rx fibre0 " << (int)conf.rx[0] << " rx fibre1 " << (int)conf.rx[1] << std::endl;

//   std::cout << " BOC configuration " << conf.bocConfig << std::endl;

  for(int c=0; c<N_SCT_CHIPS; c++) {
    std::cout << "Chip: " << c << (conf.chip[c].active?" active ":" inactive ") ; 
    if(conf.chip[c].active) {
      std::cout << std::hex << "0x" << (int)conf.chip[c].address << std::dec << std::endl;
      std::cout << "Config 0x" << std::hex << *(UINT16*)(&conf.chip[c].basic.config) << std::dec << std::endl;

      std::cout << conf.chip[c].basic.config.readoutMode << " ";
      std::cout << conf.chip[c].basic.config.calibMode << " ";
      std::cout << conf.chip[c].basic.config.trimRange << " ";
      std::cout << conf.chip[c].basic.config.edgeDetect << " ";
      std::cout << conf.chip[c].basic.config.mask << " ";
      std::cout << conf.chip[c].basic.config.accumulate << " ";
      std::cout << conf.chip[c].basic.config.inputBypass << " ";
      std::cout << conf.chip[c].basic.config.outputBypass << " ";
      std::cout << conf.chip[c].basic.config.master << " ";
      std::cout << conf.chip[c].basic.config.end << " ";
      std::cout << conf.chip[c].basic.config.feedThrough << " ";

      std::cout << std::endl;

      std::cout << "Mask " << std::hex
           << " 0x" << conf.chip[c].basic.mask[0]
           << " 0x" << conf.chip[c].basic.mask[1]
           << " 0x" << conf.chip[c].basic.mask[2]
           << " 0x" << conf.chip[c].basic.mask[3] << std::dec << std::endl;

      std::cout << "Rc function type " << (int)conf.chip[c].caldata.rc_function << " :";
      for(int r=0; r<3; r++) {
        std::cout << " " << conf.chip[c].caldata.rc_params[r];
      }
      std::cout << std::endl;
      std::cout << "Regs ";
      std::cout << " vthr " << (int)conf.chip[c].basic.vthr;
      std::cout << " vcal " << (int)conf.chip[c].basic.vcal;
      std::cout << " delay " << (int)conf.chip[c].basic.delay;
      std::cout << " preamp " << (int)conf.chip[c].basic.preamp;
      std::cout << " shaper " << (int)conf.chip[c].basic.shaper;
      std::cout << std::endl;

      std::cout << "Trim target = " << (int)conf.chip[c].target;
      for(int i=0; i<128; i++) {
        if(!(i%32)) std::cout << std::endl;
        std::cout << std::hex << (int)conf.chip[c].trim[i] << std::dec;
      }
      std::cout << std::endl;
    } else {
      std::cout << std::endl;
    }
  }
  std::cout << std::endl;
}


/*
  Configure a module using the configuration

  XXX throws exception if module not configured previously ?
*/
void ConfigurationXMLImpl::configureModuleFromStructure(std::string module, ABCDModule conf) {
  log << "Configure for module " << module << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/modules/module[sn=\"%s\"]", module.c_str());
  xmlNodePtr moduleNode;
  try {
     moduleNode = getQueryNode(expression, "Module update");
  } catch(ConfigurationException &c) {
//     std::cerr << "Recovered\n";
    xmlNodePtr modulesNode = 0;
    try {
      modulesNode = getQueryNode("/configuration/modules", "Create module (1)");
    } catch(ConfigurationException &c) {
      xmlNodePtr root = xmlDocGetRootElement(document);

      xmlNewChild(root, NULL, (xmlChar *)"modules", NULL);

      modulesNode = getQueryNode("/configuration/modules", "Create Module (2)");
    }

    moduleNode = xmlNewChild(modulesNode, NULL, (xmlChar *)"module", NULL);
  }

  return replaceModuleConfig(moduleNode, conf, module);
}

void ConfigurationXMLImpl::configureModuleFromFile(std::string filename) {
  log << "Configure module from file " << filename << std::endl;

  char expression[1000];
  sprintf(expression, "/configuration/modules");
  xmlNodePtr modulesNode;
  try {
     modulesNode = getQueryNode(expression, "Module configure file");
  } catch(ConfigurationException &c) {
    xmlNodePtr root = xmlDocGetRootElement(document);

    xmlNewChild(root, NULL, (xmlChar *)"modules", NULL);

    modulesNode = getQueryNode("/configuration/modules", "Create Module (2)");
  }

  // Put <xi::include href="filename" /> into configuration/modules

  xmlNsPtr xiNameSpace = xmlSearchNsByHref(document, modulesNode, (const xmlChar*)"http://www.w3.org/2001/XInclude");

  if(!xiNameSpace) {
    xiNameSpace = xmlNewNs(modulesNode, (xmlChar*)"http://www.w3.org/2001/XInclude", (xmlChar*)"xi");
  }

  xmlNodePtr moduleNode = xmlNewChild(modulesNode, xiNameSpace, (xmlChar *)"include", NULL);
  xmlSetProp(moduleNode, (xmlChar *)"href", (xmlChar *)filename.c_str());

  xmlXIncludeProcess(document);

  // Check the serial number against all the others and remove the other one if necessary
#warning "Incomplete"
}

/*
  Replace the configuration in the given node with the data
  stored in the given configuration
*/
void ConfigurationXMLImpl::replaceModuleConfig(xmlNodePtr node, ABCDModule conf, std::string name) {
  xmlNodePtr newNode = xmlNewNode(NULL, (xmlChar *)"module");

  const int BUF_SIZE = 200;

  char buffer[BUF_SIZE];

  xmlNewChild(newNode, NULL, (xmlChar *)"sn", (xmlChar *)name.c_str());
  xmlNewChild(newNode, NULL, (xmlChar *)"source", (xmlChar *)"Module configuration saved during running");
  xmlNewChild(newNode, NULL, (xmlChar *)"location", (xmlChar *)"RAL");
  snprintf(buffer, BUF_SIZE, "%d", conf.active);
  xmlNewChild(newNode, NULL, (xmlChar *)"active", (xmlChar *)buffer);
  snprintf(buffer, BUF_SIZE, "%d", conf.groupId);
  xmlNewChild(newNode, NULL, (xmlChar *)"group", (xmlChar *)buffer);
  snprintf(buffer, BUF_SIZE, "%d", conf.select);
  xmlNewChild(newNode, NULL, (xmlChar *)"select", (xmlChar *)buffer);

  for(int chipNum=0; chipNum<12; chipNum++) {
    ABCDChip &currChip = conf.chip[chipNum]; 

    xmlNodePtr chip = xmlNewChild(newNode, NULL, (xmlChar *)"chip", (xmlChar *)"\n");
    snprintf(buffer, BUF_SIZE, "%02d", chipNum);
    xmlSetProp(chip, (xmlChar *)"id", (xmlChar *)buffer);
    snprintf(buffer, BUF_SIZE, "0x%x", currChip.address);
    xmlSetProp(chip, (xmlChar *)"address", (xmlChar *)buffer);
    snprintf(buffer, BUF_SIZE, "%d", currChip.active);
    xmlSetProp(chip, (xmlChar *)"active", (xmlChar *)buffer);

    ABCDConfig &chipConf = currChip.basic.config;

    snprintf(buffer, BUF_SIZE, "%d %d %d %d %d %d %d %d %d %d %d",
             chipConf.readoutMode, chipConf.calibMode, chipConf.trimRange,
             chipConf.edgeDetect, chipConf.mask, chipConf.accumulate,
             chipConf.inputBypass, chipConf.outputBypass, chipConf.master,
             chipConf.end, chipConf.feedThrough);
    xmlNewChild(chip, NULL, (xmlChar *)"config", (xmlChar *)buffer);

    snprintf(buffer, BUF_SIZE, "0x%08x 0x%08x 0x%08x 0x%08x",
             currChip.basic.mask[0], currChip.basic.mask[1],
             currChip.basic.mask[2], currChip.basic.mask[3]);
    xmlNewChild(chip, NULL, (xmlChar *)"mask", (xmlChar *)buffer);

    snprintf(buffer, BUF_SIZE, "0x%02x", currChip.basic.vthr);
    xmlNodePtr reg = xmlNewChild(chip, NULL, (xmlChar *)"reg", (xmlChar *)buffer);
    xmlSetProp(reg, (xmlChar *)"name", (xmlChar *)"vthr");
    snprintf(buffer, BUF_SIZE, "0x%02x", currChip.basic.vcal);
    reg = xmlNewChild(chip, NULL, (xmlChar *)"reg", (xmlChar *)buffer);
    xmlSetProp(reg, (xmlChar *)"name", (xmlChar *)"vcal");
    snprintf(buffer, BUF_SIZE, "0x%02x", currChip.basic.delay);
    reg = xmlNewChild(chip, NULL, (xmlChar *)"reg", (xmlChar *)buffer);
    xmlSetProp(reg, (xmlChar *)"name", (xmlChar *)"delay");
    snprintf(buffer, BUF_SIZE, "0x%02x", currChip.basic.preamp);
    reg = xmlNewChild(chip, NULL, (xmlChar *)"reg", (xmlChar *)buffer);
    xmlSetProp(reg, (xmlChar *)"name", (xmlChar *)"preamp");
    snprintf(buffer, BUF_SIZE, "0x%02x", currChip.basic.shaper);
    reg = xmlNewChild(chip, NULL, (xmlChar *)"reg", (xmlChar *)buffer);
    xmlSetProp(reg, (xmlChar *)"name", (xmlChar *)"shaper");

    snprintf(buffer, BUF_SIZE, "p0 %e p1 %e p2 %e",
             currChip.caldata.rc_params[0], currChip.caldata.rc_params[1], currChip.caldata.rc_params[2]);
    reg = xmlNewChild(chip, NULL, (xmlChar *)"rc_function",
                      (xmlChar *)buffer);
    snprintf(buffer, BUF_SIZE, "%d", currChip.caldata.rc_function); 
    xmlSetProp(reg, (xmlChar *)"type", (xmlChar *)buffer);

    snprintf(buffer, BUF_SIZE, "%f", currChip.caldata.c_factor); 
    xmlNewChild(chip, NULL, (xmlChar *)"c_factor", (xmlChar *)buffer);

    snprintf(buffer, BUF_SIZE, "%f", currChip.target * 2.5); 
    xmlNewChild(chip, NULL, (xmlChar *)"target", (xmlChar *)buffer);

    for(int u=0; u<4; u++) {
      buffer[0] = '\0';
      for(int t=u*32; t<(u+1)*32; t+=8) {
        snprintf(buffer + strlen(buffer), BUF_SIZE, "0x%x%x%x%x%x%x%x%x ",
                 currChip.trim[t+0], currChip.trim[t+1], currChip.trim[t+2], currChip.trim[t+3],
                 currChip.trim[t+4], currChip.trim[t+5], currChip.trim[t+6], currChip.trim[t+7]);
      }
      reg = xmlNewChild(chip, NULL, (xmlChar *)"trim",
                        (xmlChar *)buffer);
      snprintf(buffer, BUF_SIZE, "0x%02x", u*32);
      xmlSetProp(reg, (xmlChar *)"start", (xmlChar *)buffer);
      snprintf(buffer, BUF_SIZE, "0x%02x", u*32+31);
      xmlSetProp(reg, (xmlChar *)"end", (xmlChar *)buffer);
    }
  }

  xmlReplaceNode(node, newNode);
  xmlFreeNode(node);
}

/*
  Replace the in memory configuration for the given ROD
  with the configuration provided.
*/
void ConfigurationXMLImpl::updateRodConfig(unsigned int partition, unsigned int crate, unsigned int rod, RodConfig conf) {
  log << "Update configuration for ROD\n";

  char expression[1000];

  sprintf(expression, "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]", partition, crate, rod);

  xmlNodePtr oldConf = getQueryNode(expression, "ROD update"); // queryResult->nodesetval->nodeTab[0];

  if(oldConf) {
    xmlNodePtr firstNode = 0;

    char buffer[200];

    xmlNodePtr child = oldConf->children;
    while(child) {
      char *name = (char *)child->name;

      xmlNodePtr next = child->next;
      if((strcmp("baseAddress", name) == 0)) {
        xmlUnlinkNode(child);
        xmlFreeNode(child);
      } else if(strcmp("mapSize", name) == 0) {
        xmlUnlinkNode(child);
        xmlFreeNode(child);
      } else if(strcmp("numSlaves", name) == 0) {
        xmlUnlinkNode(child);
        xmlFreeNode(child);
      } else if(strcmp("slave", name) == 0) {
        xmlUnlinkNode(child);
        xmlFreeNode(child);
      } else if(strcmp("MUR", name) == 0) {
        if(!firstNode) firstNode = child;
      }

      child = next;
    }

    xmlNodePtr newChild = 0;
    sprintf(buffer, "0x%lx", conf.baseAddress);
    newChild = xmlNewChild(oldConf, NULL, (xmlChar *)"baseAddress", (xmlChar *)buffer);

    sprintf(buffer, "0x%lx", conf.mapSize);
    newChild = xmlNewChild(oldConf, NULL, (xmlChar *)"mapSize", (xmlChar *)buffer);

    sprintf(buffer, "%ld", conf.numSlaves);
    newChild = xmlNewChild(oldConf, NULL, (xmlChar *)"numSlaves", (xmlChar *)buffer);

    for(int s=3; s>=0; s--) {
      xmlNodePtr slave = xmlNewChild(oldConf, NULL, (xmlChar *)"slave", (xmlChar *)"\n");
          
      sprintf(buffer, "%d", s);
      xmlSetProp(slave, (xmlChar *)"id", (xmlChar *)buffer);

      xmlNewChild(slave, NULL, (xmlChar *)"emifFile", (xmlChar *)conf.slaves[s].emifFile.c_str());
      xmlNewChild(slave, NULL, (xmlChar *)"ipramFile", (xmlChar *)conf.slaves[s].ipramFile.c_str());
      xmlNewChild(slave, NULL, (xmlChar *)"idramFile", (xmlChar *)conf.slaves[s].idramFile.c_str());
      xmlNewChild(slave, NULL, (xmlChar *)"extFile", (xmlChar *)conf.slaves[s].extFile.c_str());

      xmlAddPrevSibling(firstNode, slave);
      firstNode = slave;
    }
  }
}

/*
  Write configuration to disc.
  This creates a unique filename based on the date and time
  and saves the current configuration to it.
*/
void ConfigurationXMLImpl::saveConfiguration(std::string filename) {
  log << "Save configuration " << std::flush;

  const int useCompress = 1;

  if(filename == "") {
    const int BUFF_SIZE = 50;
    char buffer[BUFF_SIZE];

    time_t secs;

    secs = time(NULL);

    struct tm broke;
    broke = *(gmtime(&secs));
    // 20021112143712
    if(useCompress) {
      strftime(buffer, BUFF_SIZE, "Conf_dump_%Y%m%d%H%M%S.xml.gz", &broke);
    } else {
      strftime(buffer, BUFF_SIZE, "Conf_dump_%Y%m%d%H%M%S.xml", &broke);
    }

    filename = buffer;
  }

  log << " to " << filename << std::endl;

  if(useCompress)
    xmlSetCompressMode(5);
  else
    xmlSetCompressMode(0);
  xmlSaveFormatFile(filename.c_str(), document, 0);
//   std::cout << "Saved the configuration as " << filename.c_str() << std::endl;
  log << "Saved the configuration as " << filename.c_str() << std::endl;
}

void ConfigurationXMLImpl::saveModuleConfiguration(std::string sn, std::string filename) {
#warning "Untested"

  char expression[1000];

  sprintf(expression, "/configuration/modules/module[sn=\"%s\"]", sn.c_str());
  xmlNodePtr moduleNode = getQueryNode(expression, "Save Module");
  xmlNodePtr moduleCopy = xmlCopyNode(moduleNode, 1);

  xmlDocPtr moduleDocument = xmlNewDoc((const xmlChar *)"1.0");

  xmlDocSetRootElement(moduleDocument, moduleCopy);
  xmlSaveFile(filename.c_str(), moduleDocument);

  xmlFreeDoc(moduleDocument);
}

std::string ConfigurationXMLImpl::getModuleConfigurationString(const std::string modsn, const ABCDModule config)
{
#warning "Somewhat tested"
  xmlDocPtr moduleDocument = xmlNewDoc((const xmlChar *)"1.0");
  xmlNodePtr moduleRoot = xmlNewNode(NULL, (const xmlChar*)"old");
  xmlDocSetRootElement(moduleDocument, moduleRoot);

  // Needs something to replace it
  replaceModuleConfig(moduleRoot, config, modsn);

  xmlChar *buffer;
  int length;
  xmlDocDumpMemory(moduleDocument, &buffer, &length);

  xmlFreeDoc(moduleDocument);

  std::string result((const char *)buffer);

  xmlFree(buffer);

  return result;
}

const RodConfig ConfigurationXMLImpl::parseRodConfig(xmlNodePtr c) {
  RodConfig result;

//   result.bocPresent = true;

  if(c) {
    for(xmlNodePtr child = c->children;
        child;
        child = child->next) {
      if(child->type == XML_ELEMENT_NODE) {
        char * content = (char *)xmlNodeGetContent(child);

	log << "Node " << child->name << std::endl;

        if(strcmp((char *)child->name, "baseAddress") == 0) {
          result.baseAddress = strtol(content, NULL, 0);
        } else if(strcmp((char *)child->name, "mapSize") == 0) {
          result.mapSize = strtol(content, NULL, 0);
        } else if(strcmp((char *)child->name, "numSlaves") == 0) {
          result.numSlaves = strtol(content, NULL, 0);
        } else if(strcmp((char *)child->name, "bocNotPresent") == 0) {
	  log << "BOC not present obsolete\n";
//           result.bocPresent = false;
        } else if(strcmp((char *)child->name, "slave") == 0) {
          try {
            unsigned int slaveId = getAttrAsInt(child, "id");
            //                std::cout << "Found data for slave " << slaveId << std::endl;
            for(xmlNodePtr slaveChild = child->children;
                slaveChild;
                slaveChild = slaveChild->next) {
              char *content = 0;
              if(strcmp((char *)slaveChild->name, "emifFile") == 0) {
                content = (char*)xmlNodeGetContent(slaveChild);
                result.slaves[slaveId].emifFile = content;
              } else if(strcmp((char *)slaveChild->name, "ipramFile") == 0) {
                content = (char*)xmlNodeGetContent(slaveChild);
                result.slaves[slaveId].ipramFile = content;
              } else if(strcmp((char *)slaveChild->name, "idramFile") == 0) {
                content = (char*)xmlNodeGetContent(slaveChild);
                result.slaves[slaveId].idramFile = content;
              } else if(strcmp((char *)slaveChild->name, "extFile") == 0) {
                content = (char*)xmlNodeGetContent(slaveChild);
                result.slaves[slaveId].extFile = content;
              }
              free(content);
            }
          } catch(ConfigurationException &c) {
            log << "This slave has no id attribute\n";
          }
        } else {
          log << "Unknown tag in rod conf " << child->name << " \n";
        }

        free(content);
      } else {
        // log << "Node of a different type\n";
      }
    } // End while child
  //      result.push_back(atoi((char *)xmlGetProp(curr, (xmlChar *)"id")));
  }
  return result;
}

const TimConfig ConfigurationXMLImpl::parseTimConfig(xmlNodePtr c) {
  TimConfig result;

  result.baseAddress = 0;
  result.mapSize = 0;

  if(c) {
    for(xmlNodePtr child = c->children;
        child;
        child = child->next) {
      if(child->type == XML_ELEMENT_NODE) {
        char * content = (char *)xmlNodeGetContent(child);

        if(strcmp((char *)child->name, "baseAddress") == 0) {
          result.baseAddress = strtol(content, NULL, 0);
        } else if(strcmp((char *)child->name, "mapSize") == 0) {
          result.mapSize = strtol(content, NULL, 0);
        } else {
          log << "Unknown tag in tim conf " << child->name << " \n";
        }

        free(content);
      } else {
        // log << "Node of a different type\n";
      }
    } // End while child
  }
  return result;
}

// Very dangerous method!!!!
void ConfigurationXMLImpl::clearAll() {
  log << "CLEAR ALL CONFIGURATION!!!!!!\n";

  xmlNodePtr conf = xmlNewNode(NULL, (const xmlChar*)"configuration");

  xmlNodePtr oldRoot = xmlDocGetRootElement(document);

  xmlReplaceNode(oldRoot, conf);

//    xmlNewChild(conf, NULL, (xmlChar *)"modules", NULL);

  xmlFreeNode(oldRoot);
}

// Is this necessary?
void ConfigurationXMLImpl::namePartition(int partition, std::string name) {
  log << "Name partition\n";

  char buffer[100];

  xmlNodePtr root;

  sprintf(buffer, "/configuration/partition[@id=%d]", partition);
  try {
    root = getQueryNode(buffer, "Name partition (1)");
  } catch(ConfigurationException &e) {
//     std::cerr << "Recovered\n";
    createPartition(partition);
    root = getQueryNode(buffer, "Name partition (2)");
  }

  xmlSetProp(root, (xmlChar *)"name", (xmlChar *)name.c_str());
}

void ConfigurationXMLImpl::createPartition(int partition) {
  log << "Create partition\n";

  char buffer[100];
 
  sprintf(buffer, "/configuration");
  xmlNodePtr root;
  try {
    root = getQueryNode(buffer, "Create partition");
  } catch(ConfigurationException &e) {
//     std::cerr << "Recovered\n";
    xmlNodePtr conf = xmlNewNode(NULL, (const xmlChar*)"configuration");
    xmlNodePtr oldRoot = xmlDocGetRootElement(document);
    xmlReplaceNode(oldRoot, conf);

    root = getQueryNode(buffer, "Create partition (2)");
  }

  xmlNodePtr newNode
    = xmlNewChild(root, NULL, (xmlChar *)"partition", NULL);
//      xmlSetProp(newNode, (xmlChar *)"name", (xmlChar *)name.c_str());
  sprintf(buffer, "%d", partition);
  xmlSetProp(newNode, (xmlChar *)"id", (xmlChar *)buffer);
}

void ConfigurationXMLImpl::createCrate(int partition, int crate) {
  log << "Create crate\n";

  char buffer[1000];

  xmlNodePtr part;
  sprintf(buffer, "/configuration/partition[@id=%d]", partition);
  try {
    part = getQueryNode(buffer, "Create crate");
  } catch(ConfigurationException &e) {
//     std::cerr << "Recovered\n";
    createPartition(partition);
    part = getQueryNode(buffer, "Create crate (2)");
  }

  xmlNodePtr newNode
    = xmlNewChild(part, NULL, (xmlChar *)"crate", NULL);
  sprintf(buffer, "%d", crate);
  xmlSetProp(newNode, (xmlChar *)"id", (xmlChar *)buffer);
}

void ConfigurationXMLImpl::createROD(int partition, int crate, int rod) {
  log << "Create ROD\n";

  char buffer[1000];

  xmlNodePtr crateNode;
  sprintf(buffer, "/configuration/partition[@id=%d]/crate[@id=%d]", partition, crate);
  try {
    crateNode = getQueryNode(buffer, "Create ROD");
  } catch(ConfigurationException &e) {
//     std::cerr << "Recovered\n";
    createCrate(partition, crate);
    crateNode = getQueryNode(buffer, "Create ROD (2)");
  }

  xmlNodePtr newNode = xmlNewChild(crateNode, NULL, (xmlChar *)"rod", NULL);
  sprintf(buffer, "%d", rod);
  xmlSetProp(newNode, (xmlChar *)"id", (xmlChar *)buffer);
}

void ConfigurationXMLImpl::configureROD(unsigned int partition, unsigned int crate, unsigned int rod, RodConfig conf) {
  log << "Configure ROD " << partition << " " << crate << " " << rod << std::endl;

  char buffer[1000];

//   std::cout << "Configure a ROD!\n";

  sprintf(buffer, "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]", partition, crate, rod);
  xmlNodePtr node;
  try {
    node = getQueryNode(buffer, "Configure ROD");
  } catch(ConfigurationException &e) {
//     std::cerr << "Recovered\n";
    createROD(partition, crate, rod);
    node = getQueryNode(buffer, "Configure ROD (2)");
  }

  updateRodConfig(partition, crate, rod, conf);
}

void ConfigurationXMLImpl::configureTIM(unsigned int partition, unsigned int crate, TimConfig conf) {
  log << "Configure TIM " << partition << " " << crate << std::endl;

  char buffer[1000];

  sprintf(buffer, "/configuration/partition[@id=%d]/crate[@id=%d]/tim", partition, crate);
  xmlNodePtr node;
  try {
    node = getQueryNode(buffer, "Configure TIM");
  } catch(ConfigurationException &e) {
    // Create TIM node 
    xmlNodePtr root = xmlDocGetRootElement(document);
    xmlNodePtr config = getOrNewChildNode(root, "configuration");
    xmlNodePtr partNode = getOrNewChildWithIntAttr(config, "partition", "id", partition);
    xmlNodePtr crateNode = getOrNewChildWithIntAttr(partNode, "crate", "id", crate);
    node = getOrNewChildNode(crateNode, "tim");
  }

  sprintf(buffer, "0x%lx", conf.baseAddress);
  setOrCreateChildNodeWithContent(node, "baseAddress", buffer);
  sprintf(buffer, "0x%lx", conf.mapSize);
  setOrCreateChildNodeWithContent(node, "mapSize", buffer);
}

void ConfigurationXMLImpl::createMUR(int partition, int crate, int rod, int order, int number) {
  log << "Create MUR\n";

  char buffer[1000];

  sprintf(buffer, "/configuration/partition[@id=%d]/crate[@id=%d]/rod[@id=%d]", partition, crate, rod);
  xmlNodePtr rodNode;
  try {
    rodNode = getQueryNode(buffer, "Create MUR");
  } catch(ConfigurationException &e) {
//     std::cerr << "Recovered\n";
    // Need ROD config
    createROD(partition, crate, rod);
    rodNode = getQueryNode(buffer, "Create MUR (2)");
  }

  xmlNodePtr newNode
    = xmlNewChild(rodNode, NULL, (xmlChar *)"MUR", NULL);
  sprintf(buffer, "%d", number);
  xmlSetProp(newNode, (xmlChar *)"id", (xmlChar *)buffer);
  sprintf(buffer, "%d", order);
  xmlSetProp(newNode, (xmlChar *)"order", (xmlChar *)buffer);
}

void ConfigurationXMLImpl::mapModuleMUR(int MUR, int order, int RMUR, int rorder, std::string number) {
  log << "Map module onto MUR\n";

  char buffer[1000];

  try {
    sprintf(buffer, "/configuration/partition/crate/rod/MUR[@id=%d]", MUR);
    xmlNodePtr node = getQueryNode(buffer, "Assign module to MUR");

    xmlNodePtr newNode
      = xmlNewChild(node, NULL, (xmlChar *)"module", (xmlChar *)number.c_str());

    sprintf(buffer, "%d", order);
    xmlSetProp(newNode, (xmlChar *)"id", (xmlChar *)buffer);
  } catch(ConfigurationException &e) {
    log << "No MUR " << MUR << " for module " << number << std::endl;
    throw;
  }

  try {
    sprintf(buffer, "/configuration/partition/crate/rod/MUR[@id=%d]", RMUR);
    xmlNodePtr node = getQueryNode(buffer, "Assign module to RMUR");

    xmlNodePtr newNode
      = xmlNewChild(node, NULL, (xmlChar *)"rmodule", (xmlChar *)number.c_str());

    sprintf(buffer, "%d", rorder);
    xmlSetProp(newNode, (xmlChar *)"id", (xmlChar *)buffer);
  } catch(ConfigurationException &e) {
    log << "No MUR " << RMUR << " for rmodule " << number << std::endl;
  }
}

void ConfigurationXMLImpl::configureBOCChannel(int MUR, int order, const BOCChannelConfig &bConf) {
  log << "Create BOC configuration\n";

  char buffer[1000];

  sprintf(buffer, "/configuration/partition/crate/rod/MUR[@id=%d]", MUR);
  xmlNodePtr node = getQueryNode(buffer, "Set BOC Channel Configs");

  xmlNodePtr ourChannelNode = 0;

  for(xmlNodePtr channelNode = node->children;
        channelNode;
        channelNode = channelNode->next) {

    if(channelNode->type != XML_ELEMENT_NODE) {
    } else {
      if(strcmp((char*)channelNode->name, "channel") == 0 &&  getAttrAsInt(channelNode, "id") == order) {
        ourChannelNode = channelNode;
        break;
      }
    }
  }

  if(!ourChannelNode) {
    ourChannelNode = xmlNewChild(node, NULL, (xmlChar *)"channel", NULL);
  }

  sprintf(buffer, "%d", order);
  xmlSetProp(ourChannelNode, (xmlChar *)"id", (xmlChar *)buffer);

  xmlNodePtr newNode = getOrNewChildNode(ourChannelNode, "output");
  sprintf(buffer, "0x%x", bConf.current);
  xmlSetProp(newNode, (xmlChar *)"current", (xmlChar *)buffer);
  sprintf(buffer, "0x%x", bConf.delay);
  xmlSetProp(newNode, (xmlChar *)"delay", (xmlChar *)buffer);
  sprintf(buffer, "0x%x", bConf.markSpace);
  xmlSetProp(newNode, (xmlChar *)"markSpace", (xmlChar *)buffer);

  newNode = getOrNewChildNode(ourChannelNode, "stream0");
  sprintf(buffer, "0x%x", bConf.threshold0);
  xmlSetProp(newNode, (xmlChar *)"threshold", (xmlChar *)buffer);
  sprintf(buffer, "0x%x", bConf.delay0);
  xmlSetProp(newNode, (xmlChar *)"delay", (xmlChar *)buffer);

  newNode = getOrNewChildNode(ourChannelNode, "stream1");
  sprintf(buffer, "0x%x", bConf.threshold1);
  xmlSetProp(newNode, (xmlChar *)"threshold", (xmlChar *)buffer);
  sprintf(buffer, "0x%x", bConf.delay1);
  xmlSetProp(newNode, (xmlChar *)"delay", (xmlChar *)buffer);
}

void ConfigurationXMLImpl::writePowerSupplyConfiguration(std::string directory) {
  ofstream parFile((directory + "/Cnf_Para.txt").c_str(), std::ios::out);

  const char *headers[] = {"ID", "Module", "Parameter", "EoP", 
                           "FL_Off", "WL_Off", "Nr_Off", "WH_Off", "FH_Off", "EoO",
                           "FL_Stb", "WL_Stb", "Nr_Stb", "WH_Stb", "FH_Stb", "EoS",
                           "FL_On", "WL_On", "Nr_On", "WH_On", "FH_On", "Comment"};

//                            "FatlL_Off", "WarnrL_Off", "Norm_Off", "WarnH_Off", "FatlH_Off", "EoO",
//                            "FatlL_Stb", "WarnrL_Stb", "Norm_Stb", "WarnH_Stb", "FatlH_Stb", "EoS",
//                            "FatlL_On", "WarnrL_On", "Norm_On", "WarnH_On", "FatlH_On", "Comment"};

  const char *params[] = {
    "LVchStat", "LVch_Vcc", "LVps_Vcc", "LVretVcc", "LVch_Icc", "LVch_Vdd", "LVps_Vdd", "LVretVdd", "LVch_Idd", "LVchVCSV", 
    "LVchVCSI", "LVchPINV", "LVchPINI", "MOch_Tm0", "MOch_Tm1", "LVchDTrp", "LVchCTrp", "LVchTLim", "LVchTTrp", "LVchCLKS", 
    "LVchCLKR", "LVchVOut", "HVchStat", "HVchVolt", "HVchCurr", "HVchITrp", "HVchRamp"};

  const float offValues[] = {
    /**/     0,        0.0,        0.0,         -1,         -1,        0.0,        0.0,         -1,         -1,        0.0,
    /**/    -1,        0.0,         -1,         -1,         -1,         50,         50,         10,         15,          0, 
    /**/     0,          0,          0,          0,         -1,         10,          4};

  const float stbValues[] = {
    /**/     0,        0.0,        3.5,         -1,         -1,        4.0,        4.0,         -1,         -1,        4.0,
    /**/    -1,        6.0,         -1,         -1,         -1,         50,         50,          5,         10,          0, 
    /**/     0,          1,          0,         50,         -1,         10,          4};

  const float onValues[] = {
    /**/     0,        3.5,        3.5,         -1,         -1,        4.0,        4.0,         -1,         -1,        4.0,
    /**/    -1,        6.0,         -1,         -1,         -1,         50,         50,         10,         15,          0, 
    /**/     0,          1,          0,        150,         -1,        100,          4};

  std::cout << "Size of headers is " << sizeof(headers) << std::endl;

  for(unsigned int i=0; i<sizeof(headers)/4; i++) {
    parFile << "\"" << headers[i] << "\"\t";
  }

  parFile << std::endl;

  int id=1;

  for(int module = 1; module < 4; module++) {
    for(unsigned int param=0; param<sizeof(params)/4; param++) {
      parFile << id++ << "\t" << module << "\t";

      parFile << "\"" << params[param] << "\"\t";

      parFile << "\"!\"\t";

      if(offValues[param] == -1) {
        parFile << "\"" << 1 << "\"\t";
        parFile << "\"" << 2 << "\"\t";
        parFile << "\"" << 3 << "\"\t";
        parFile << "\"" << 4 << "\"\t";
        parFile << "\"" << 5 << "\"\t";
      } else {
        float val = offValues[param];
        parFile << "\"" << (val * 0.8) << "\"\t";
        parFile << "\"" << (val * 0.9) << "\"\t";
        parFile << "\"" << (val * 1.0) << "\"\t";
        parFile << "\"" << (val * 1.1) << "\"\t";
        parFile << "\"" << (val * 1.2) << "\"\t";
      }

      parFile << "\"!\"\t";

      if(stbValues[param] == -1) {
        parFile << "\"" << 2 << "\"\t";
        parFile << "\"" << 3 << "\"\t";
        parFile << "\"" << 4 << "\"\t";
        parFile << "\"" << 5 << "\"\t";
        parFile << "\"" << 6 << "\"\t";
      } else {
        float val = stbValues[param];
        parFile << "\"" << (val * 0.8) << "\"\t";
        parFile << "\"" << (val * 0.9) << "\"\t";
        parFile << "\"" << (val * 1.0) << "\"\t";
        parFile << "\"" << (val * 1.1) << "\"\t";
        parFile << "\"" << (val * 1.2) << "\"\t";
      }

      parFile << "\"!\"\t";

      if(onValues[param] == -1) {
        parFile << "\"" << 3 << "\"\t";
        parFile << "\"" << 4 << "\"\t";
        parFile << "\"" << 5 << "\"\t";
        parFile << "\"" << 6 << "\"\t";
        parFile << "\"" << 7 << "\"\t";
      } else {
        float val = onValues[param];
        parFile << "\"" << (val * 0.8) << "\"\t";
        parFile << "\"" << (val * 0.9) << "\"\t";
        parFile << "\"" << (val * 1.0) << "\"\t";
        parFile << "\"" << (val * 1.1) << "\"\t";
        parFile << "\"" << (val * 1.2) << "\"\t";
      }

      parFile << "\"No Comment\"";

      parFile << std::endl;
    }
  }

  // Print sentinel at the end

  parFile << id++ << "\t99999\t";
  parFile << "\"Noparam\"\t";
  parFile << "\"!\"\t";

  for(int i=0; i<5; i++) {
    parFile << "\"" << 0 << "\"\t";
  }

  parFile << "\"!\"\t";

  for(int i=0; i<5; i++) {
    parFile << "\"" << 0 << "\"\t";
  }

  parFile << "\"!\"\t";

  for(int i=0; i<5; i++) {
    parFile << "\"" << 0 << "\"\t";
  }

  parFile << "\"No Comment\"";
  parFile << std::endl;


  // Write more files for the DCS system
//   ofstream parFile((directory + "Cnf_Para.txt").c_str(), std::ios::out);
}

xmlNodePtr ConfigurationXMLImpl::getQueryNode(std::string query, std::string task) {
  xmlXPathObjectPtr queryResult = xmlXPathEval((xmlChar *)(query.c_str()), xpathContext);

  xmlNodePtr nodeResult = 0;

  try {
    if(queryResult) {
      if(queryResult->type == XPATH_NODESET && queryResult->nodesetval) {
        if(queryResult->nodesetval->nodeNr > 1) {
          log << "Too many matches (" << queryResult->nodesetval->nodeNr << ") doing " << task << ": " << query << std::endl;
          throw ConfigurationException("Query: too many matches: " + query);
        } else if(queryResult->nodesetval->nodeNr == 0) {
          log << "Too few matches doing " << task << ": " << query << std::endl;
          throw ConfigurationException("Query: too few matches: " + query);
        } else {
          nodeResult = queryResult->nodesetval->nodeTab[0];
        }
      } else {
        log << "Bad result type (" << task << "): " << query << std::endl;
        throw ConfigurationException("Query: Bad result type: " + query);
      }

//       xmlXPathFreeNodeSetList(queryResult);
    } else {
      log << "No result (" << task << "): " << query << std::endl;
      throw ConfigurationException("Query: No result");
    }
  } catch(...) {
    log << "Unexpected exception in getQueryNode" << std::endl;
    if(queryResult)
      xmlXPathFreeObject(queryResult);
    throw;
  }

  return nodeResult;
}

xmlNodePtr ConfigurationXMLImpl::getChildNode(xmlNodePtr node, const char *name) {
  xmlNodePtr curr = node->children;

  while(curr != NULL) {
    if(curr->type == XML_ELEMENT_NODE) {
      if(strcmp((const char *)curr->name, name) == 0) {
        return curr;
      }
    }

    curr = curr->next;
  }

  log << "No child " << name << " of parent " << node->name << std::endl;
  throw ConfigurationException("Child node not found");
}

xmlNodePtr ConfigurationXMLImpl::getOrNewChildNode(xmlNodePtr node, const char *name) {
  xmlNodePtr curr = node->children;

  while(curr != NULL) {
    if(curr->type == XML_ELEMENT_NODE) {
      if(strcmp((const char *)curr->name, name) == 0) {
        return curr;
      }
    }

    curr = curr->next;
  }

  xmlNodePtr newNode = xmlNewChild(node, NULL, (xmlChar *)name, NULL);

  return newNode;
}

void ConfigurationXMLImpl::setOrCreateChildNodeWithContent(xmlNodePtr node, const char *name, const char *content) {
  xmlNodePtr curr = node->children;

  while(curr != NULL) {
    if(curr->type == XML_ELEMENT_NODE) {
      if(strcmp((const char *)curr->name, name) == 0) {
        // Set content 
        xmlNodeSetContent(curr, (xmlChar *)content);
        return;
      }
    }

    curr = curr->next;
  }

  xmlNewChild(node, NULL, (xmlChar *)name, (xmlChar *)content);
}

xmlNodePtr ConfigurationXMLImpl::getOrNewChildWithIntAttr(xmlNodePtr parent, const char *name, const char *attr, int integer) {
  xmlNodePtr result = getNodeWithIntAttr(parent, name, attr, integer);

  if(!result) {
    result = xmlNewChild(parent, NULL, (xmlChar*)name, NULL);

    setIntAttr(result, attr, integer);
  }

  return result;
}

void ConfigurationXMLImpl::setIntAttr(xmlNodePtr node, const char *attr, int integer) {
  char buffer[10];
  snprintf(buffer, 10, "%d", integer);

  xmlSetProp(node, (xmlChar*)attr, (xmlChar*)buffer);
}

int ConfigurationXMLImpl::getAttrAsInt(xmlNodePtr node, const char *attr, int base) {
  const char *str;

  str = (const char *)xmlGetProp(node, (xmlChar *)attr);

  if(str) {
    int result = strtol(str, NULL, base);
    xmlFree((void *)str);
    return result;
  } else {
    log << "Can't find attribute " << std::string(attr) << std::endl;
    throw ConfigurationException("Can't find attribute " + std::string(attr));
  }
}

std::string ConfigurationXMLImpl::getAttrAsString(xmlNodePtr node, const char *attr) {
  const char *str;

  str = (const char *)xmlGetProp(node, (xmlChar *)attr);

  if(str) {
    std::string result = str;
    xmlFree((void *)str);
    return result;
  } else {
    log << "Can't find attribute " << std::string(attr) << std::endl;
    throw ConfigurationException("Can't find attribute" + std::string(attr));
  }
}

xmlNodePtr ConfigurationXMLImpl::getNodeWithIntAttr(xmlNodePtr node, const char *name, const char *attr, int integer) {
  for(xmlNodePtr iter = node->children;
        iter;
        iter = iter->next) {

    if(iter->type != XML_ELEMENT_NODE) {
    } else {
      if(strcmp((char*)iter->name, name) == 0 &&  getAttrAsInt(iter, attr) == integer) {
        return iter;
      }
    }
  }

  return 0;
}
