#ifndef SCTAPI_CONFIGURATION
#define SCTAPI_CONFIGURATION

#include <list>
#include <string>
#include <stdexcept>

// For ABCDModule
#include <CommonWithDsp/sctStructure.h>

/*  struct xmlDoc; */
/*  struct xmlXPathContext; */

/*  typedef struct xmlDoc *xmlDocPtr; */
/*  typedef struct xmlXPathContext *xmlXPathContextPtr; */

namespace SctConfiguration {

/** 
    The configuration files needed to initialise a ROD slave DSP
 */
struct SlaveConfig {
  std::string emifFile;
  std::string ipramFile;
  std::string idramFile;
  std::string extFile;
};

/**
   Configuration needed for ROD initialisation
 */
struct RodConfig {
/*   /// Is there a BOC? (Shouldn't be necessary) */
/*   bool bocPresent; */
  /// VME base address
  unsigned long baseAddress;
  /// VME map size
  unsigned long mapSize;
  /// Number of slaves
  long numSlaves;
  /// Slave file configuration
  SlaveConfig slaves[4];
};

/**
   Configuration needed for TIM initialisation
 */
struct TimConfig {
  /// VME base address
  unsigned long baseAddress;
  /// VME map size
  unsigned long mapSize;
};

/**
   Configuration of a "channel" on a BOC

   This is the information associated with one module
   ie. one output channel for primary data and
   two input channels for streams 0 and 1
 */
struct BOCChannelConfig {
  /// Laser current
  int current;
  /// BPM delay 
  int delay;
  /// BPM markspace
  int markSpace;
  /// Input threshold (stream 0)
  int threshold0;
  /// Strobe delay (stream 0)
  int delay0;
  /// Input threshold (stream 1)
  int threshold1;
  /// Strobe delay (stream 1)
  int delay1;

  /// Fibre used for tranimission
  int txFibre;
  /// Fibre used for reception of link 0
  int rxFibre0;
  /// Fibre used for reception of link 1
  int rxFibre1;
};

enum MURType {BARREL, ENDCAP, UNMAPPED, UNKNOWN};

/**
   Exception thrown by public configuration methods
 */
class ConfigurationException : public std::exception {
  std::string desc;
 public:
  /// Provide a description on construction
  ConfigurationException(const std::string &str) throw() : desc(str) {}

  ~ConfigurationException() throw() {}

  /// Override std::exception method
  virtual const char* what () const throw() { return desc.c_str(); }
};

/**
   Interface to XML configuration file.
   This will be replaced by an interface to the DAQ confDB system.

   The primary function of this class is the readout of configuration
   data from the database.

   Translation functions:

   Barrel
   barrel, row,     number
   3 to 6  0 to 55  -1 to -6 and 1 to 6

   Internals:
   position on row given as -1 for -ve numbers and 1 for +ve numbers

   Endcap
   disc     quadrant number 
   0 to 17  0 to 3   0 to 32

   Internals:
   position of MUR in quadrant
   number is left to right top to bottom 
      0000001222222
        4444455555
        1111133333
*/
class Configuration {
public:
  /**
     Free resources
  */
  virtual ~Configuration() {};

  /************* Get physical configuration ****************/

  /**
     List partition ids
     Lookup the partitions defined in the configuration and return a list of integer ID's
  */
  virtual std::list<unsigned int> listPartitions() = 0;

  /**
     List crate ids with reference to the given partition
     Lookup the crates defined in the specified partition and return a list of integer ID's
  */
  virtual std::list<unsigned int> listCratesInPartition(unsigned int partition) = 0;

  /**
     List the rods present in a crate
     Lookup the RODs defined in the specified crate and return a list of integer ID's
  */
  virtual std::list<unsigned int> listRodsInCrate(unsigned int partition, unsigned int crate) = 0;

  /**
     List the MURs associated with a ROD
     Return integer id's of the MURs in the specified rod.

     FIXME:  Should be in numbered order
  */
  virtual std::list<unsigned int> listMURSInRod(unsigned int partition, unsigned int crate, unsigned int rod) = 0;

  /**
     List the modules in the given MUR
     Return a list of serial numbers associated with the specified MUR

     FIXME:  Should be in numbered order
  */
  virtual std::list<std::string> listModulesInMUR(unsigned int partition, unsigned int MUR) = 0;

  /**
     List the modules in the given MUR
     Return a list of serial numbers associated with the redundancy links of the specified MUR

     FIXME:  Should be in numbered order (array?)
  */
  virtual std::list<std::string> listRModulesInMUR(unsigned int partition, unsigned int MUR) = 0;

  /**
     Return a list of serial numbers of all modules that have configuration

     This includes those that are not attached to a ROD or MUR
  */
  virtual std::list<std::string> listAllModules() = 0;

  /**
     Return a list of serial numbers of modules that have configuration but are not attached to an MUR.
  */
  virtual std::list<std::string> listUnusedModules() = 0;

  /************** Get configurations *****************/

  /**
     Return the configuration for the module

     Deprecated: Should take MUR argument
  */
  virtual ABCDModule getModuleConfig(const std::string module) = 0;

  /**
     Return the Rod configuration
     Find the configuration associated with the specified ROD.
     Return a RodConfig structure.
  */
  virtual RodConfig getRodConfig(unsigned int partition, unsigned int crate, unsigned int rod) = 0;

  /**
     Get the BOC configuration of one of 48 channels 

     Input channels are paired as one.
  */
  virtual BOCChannelConfig getBOCConfig(unsigned int partition, unsigned int crate, unsigned int rod, 
                                        unsigned int channel) = 0;

  /**
     Get the BOC fibre mappings

     An array of 48*3 chars (out, in0, in1)
  */
  virtual char *getFibreMappings(unsigned int partition, unsigned int crate, unsigned int rod) = 0;

  /**
     Return the TIM configuration
     Find the configuration associated with the specified TIM.
     Return a TimConfig structure.
  */
  virtual TimConfig getTimConfig(unsigned int partition, unsigned int crate) = 0;

  /**
     Set fibre mappings for one channel

     Any value over 48/96 leaves current value unchanged
   */
  virtual void setFibreMapping(unsigned int partition, unsigned int crate, unsigned int rod, 
                               unsigned int channel, unsigned int tx, unsigned int rx0, unsigned int rx1) = 0;

  /**
     Print module configuration to cout
  */
  virtual void printModuleConfig(const ABCDModule &conf) = 0;

  virtual MURType getMURType(unsigned int MUR) = 0;

  /************** Change indexing schemes ****************/

  /**
     Translate from MUR / module(1-6) number to p/k/r/c
  */
  virtual void translateToROD(unsigned int MUR, unsigned int module,
                              unsigned int &partition, unsigned int &crate, 
                              unsigned int &rod, unsigned int &channel) = 0;
 
  /**
     Translate from MUR / redundant module(1-6) number to p/k/r/c

     This returns the channel to which the module referred to by MUR/module 
     is connected in its redundant configuration
  */
  virtual void translateToRROD(unsigned int MUR, unsigned int module,
                               unsigned int &partition, unsigned int &crate, 
                               unsigned int &rod, unsigned int &channel) = 0;
 
  /**
     Translate from MUR / module(1-6) number to serial number
  */
  virtual void translateToSN(unsigned int MUR, unsigned int module,
                             std::string &sn) = 0;

  /**
     Translate from MUR / module(1-6) number to geographical barrel system
  */
  virtual void translateToBarrel(unsigned int MUR, unsigned int module,
                                 unsigned int &barrel, unsigned int &row, int &number) = 0;

  /**
     Translate from MUR / module(1-6) number to geographical endcap system
   */
  virtual void translateToEndcap(unsigned int MUR, unsigned int module,
                                 unsigned int &disk, unsigned int &ring, unsigned int &number) = 0;

  /**
     Translate from MUR / module(1-6) number to power supply channel address
   */
  virtual void translateToPowerSupply(unsigned int MUR, unsigned int module,
                                      unsigned int &partition, 
                                      unsigned int &crate, unsigned int &channel) = 0;

  /**
     Translate to MUR / module(1-6) number from p/k/r/c
  */
  virtual void translateFromROD(unsigned int partition, unsigned int crate, 
                                unsigned int rod, unsigned int channel,
                                unsigned int &MUR, unsigned int &module) = 0;

  /**
     Translate to MUR / module(1-6) number from address of redundant module

     This returns the MUR/module of the module connected using its
     redundant link to the p/k/r/c address
  */
  virtual void translateFromRROD(unsigned int partition, unsigned int crate,
                                 unsigned int rod, unsigned int channel,
                                 unsigned int &MUR, unsigned int &module) = 0;

  /**
     Translate to MUR / module(1-6) number from serial number
  */
  virtual void translateFromSN(const std::string sn,
                               unsigned int &MUR, unsigned int &module) = 0;

  /**
     Translate to MUR / module(1-6) number from geographical barrel system
  */
  virtual void translateFromBarrel(unsigned int barrel, unsigned int row, int number,
                                   unsigned int &MUR, unsigned int &module) = 0;

  /**
     Translate to MUR / module(1-6) number from geographical endcap system
   */
  virtual void translateFromEndcap(unsigned int disk, unsigned int ring, unsigned int number,
                                   unsigned int &MUR, unsigned int &module) = 0;

  /**
     Translate from power supply channel address to MUR / module(1-6) number 
   */
  virtual void translateFromPowerSupply(unsigned int partition, 
                                        unsigned int crate, unsigned int channel, 
                                        unsigned int &MUR, unsigned int &module) = 0;


  /************** Change configuration ******************/

  /**
     Update or add the configuration of a module stored in memory
     Replace the configuration for the given module with the data
     stored in the given configuration
  */
  virtual void configureModuleFromStructure(const std::string module, const ABCDModule conf) = 0;

  /**
     Update or add the configuration of a module stored in memory
     Replace the configuration for the given module with the data
     stored in the given file (the serial number is read from the file)
  */
  virtual void configureModuleFromFile(const std::string filename) = 0;

  /**
     Configure a ROD

     Replace the in memory configuration for the given ROD
     with the configuration provided.

     If no configuration then construct it
   */
  virtual void configureROD(unsigned int partition, unsigned int crate, unsigned int rod, RodConfig conf) = 0;

  /**
     Configure a TIM

     Replace the in memory configuration for the given TIM
     with the configuration provided.

     If no configuration then construct it
   */
  virtual void configureTIM(unsigned int partition, unsigned int crate, TimConfig conf) = 0;

  /**
     Remove all configuration.
     Very dangerous!!
   */
  virtual void clearAll() = 0;

  /**
     Configuration creation:

     Create module (write configuration)

     Change translations

     Change MUR translations
   */

  /** 
     Name a partition (for information, any use?)
   */
  virtual void namePartition(int partition, const std::string name) = 0;

  /**
     Set a BOC configuration 
   */
/*virtual    void setBOCConfig(int partition, int crate, int rod, int channel, const BOCChannelConfig &conf) = 0; */
  
  /**
     Set a BOC configuration
  */
  virtual void configureBOCChannel(int MUR, int position, const BOCChannelConfig &conf) = 0;

  /**
     Map a module (by serial number) onto a position in an MUR
  */
  virtual void mapModuleMUR(int MUR, int order, int RMUR, int rorder, std::string number) = 0;

  /** 
     Map an MUR
  */
  virtual void mapRODMUR(int partition, int crate, int rod, int order, int number) = 0;

  /**
     Map Barrel MUR
  */
  virtual void mapBarrelMUR(int MUR, int barrel, int row, int position) = 0;

  /**
     Swap two MUR names around

     This means in regards to everything about one MUR you can use the other name
     If either doesn't exist then just one MUR is renamed
  */
  virtual void swapMURNames(int MUR1, int MUR2) = 0;

/* #warning "Need to think more about endcap mappings" */

  /**
     Map Endcap MUR
  */
  virtual void mapEndcapMUR(int MUR, int disk, int quadrant, int position) = 0;

  /**
     Map Power channel
  */
  virtual void mapPowerChannel(int MUR, int number, unsigned int partition, unsigned int crate, unsigned int channel) = 0;

  /**
     Set power supply parameter
  */
  virtual void modifyPowerParam(int MUR, int number, std::string name, float value) = 0;

  /**
     Set default power supply parameter
  */
  virtual void modifyDefaultPowerParam(std::string name, float value) = 0;

  /**
     Get power supply parameter
  */
  virtual float getPowerParam(int MUR, int number, std::string name) = 0;

  /** 
     Load configuration from named file
   */
  virtual void loadConfiguration(const std::string &filename) = 0;

  /************** Save configuration *********************/

  /**
     Save the current configuration

     Write configuration to disc.
     If no filename is provided a unique filename based on
     the date and time is created.

     Filename eg Conf_dump_20021210142311.xml.gz (in current directory)
  */
  virtual void saveConfiguration(const std::string filename = "") = 0;

  /**
     Save the current configuration for a module

     Write the configuration for the module modsn to disc.
  */
  virtual void saveModuleConfiguration(const std::string modsn, const std::string filename) = 0;

  /**
     Construct the XML for a module configuration and return as a string
  */
  virtual std::string getModuleConfigurationString(const std::string modsn, const ABCDModule config) = 0;

  /**
     Dump power supply configuration to a file
  */
  virtual void writePowerSupplyConfiguration(std::string filename) = 0;
 protected:
  /** Protected methods for creating internal structures */

  /**
     Create a partition 
  */
/*   virtual void createPartition(int partition) = 0; */

  /**
     Create a crate 
  */
/*   virtual void createCrate(int partition, int crate) = 0; */

  /**
     Create a ROD
   */
/*   virtual void createROD(int partition, int crate, int rod) = 0; */

  /** 
     Create an MUR
  */
/*   virtual void createMUR(int partition, int crate, int rod, int order, int number) = 0; */

  /**
     Create a module
  */
/*virtual    void createModule(int MUR, int order, int RMUR, int rorder, string number, const ABCDModule &mConf) = 0; */

  /**
     Create a BOC configuration
  */
/*   virtual void createBOCConfig(int MUR, int order, const BOCChannelConfig &bConf) = 0; */

};

}

#endif

