#ifndef SCTAPI_OKS_IMPL_CONFIGURATION
#define SCTAPI_OKS_IMPL_CONFIGURATION

#include <list>
#include <string>
#include <stdexcept>

#include <oks/class.h>
#include <oks/object.h>
#include <oks/kernel.h>

#include "RodDaq/CommonWithDsp/sctStructure.h"

#include "../SctApi/log.h"

#include "sctConf/configuration.h"

/** 
    The configuration files needed to initialise a ROD slave DSP
 */
/*  struct SlaveConfig { */
/*    std::string emifFile; */
/*    std::string ipramFile; */
/*    std::string idramFile; */
/*    std::string extFile; */
/*  }; */

/**
   Configuration needed for ROD initialisation
 */
/*  struct RodConfig { */
/*    /// Is there a BOC? (Shouldn't be necessary) */
/*    bool bocPresent; */
/*    /// VME base address */
/*    unsigned long baseAddress; */
/*    /// VME map size */
/*    unsigned long mapSize; */
/*    /// Number of slaves */
/*    long numSlaves; */
/*    /// Slave file configuration */
/*    SlaveConfig slaves[4]; */
/*  }; */

/**
   Configuration needed for TIM initialisation
 */
/*  struct TimConfig { */
/*    /// VME base address */
/*    unsigned long baseAddress; */
/*    /// VME map size */
/*    unsigned long mapSize; */
/*  }; */

/**
   Configuration of a "channel" on a BOC

   This is the information associated with one module
   ie. one output channel for primary data and
   two input channels for streams 0 and 1
 */
/*  struct BOCChannelConfig { */
/*    /// Laser current */
/*    int current; */
/*    /// BPM delay  */
/*    int delay; */
/*    /// BPM markspace */
/*    int markSpace; */
/*    /// Input threshold (stream 0) */
/*    int threshold0; */
/*    /// Strobe delay (stream 0) */
/*    int delay0; */
/*    /// Input threshold (stream 1) */
/*    int threshold1; */
/*    /// Strobe delay (stream 1) */
/*    int delay1; */

/*    /// Fibre used for tranimission */
/*    int txFibre; */
/*    /// Fibre used for reception of link 0 */
/*    int rxFibre0; */
/*    /// Fibre used for reception of link 1 */
/*    int rxFibre1; */
/*  }; */

/**
   Exception thrown by public configuration methods
 */
/*  class ConfigurationException : std::exception { */
/*    std::string desc; */
/*   public: */
/*    /// Provide a description on construction */
/*    ConfigurationException(const std::string &str) : desc(str) {} */
/*    /// Override std::exception method */
/*    virtual const char* what () const { return desc.c_str(); } */
/*  }; */

/**
   Interface to XML configuration file.
   This will be replaced by an interface to the DAQ confDB system.

   The primary function of this class is the readout of configuration
   data from the database.

   Translation functions:

   Barrel
   barrel, row,     number
   3 to 6  0 to 55  -1 to -6 and 1 to 6

   Internals:
   position on row given as -1 for -ve numbers and 1 for +ve numbers

   Endcap
   disc     quadrant number 
   0 to 17  0 to 3   0 to 33

   Internals:
   position of MUR in quadrant
   number is left to right top to bottom 
      00000012222223
        4444455555
        1111133333
*/
class OksImpl : public Configuration {
  OksImpl(const OksImpl &);
  OksImpl &operator=(const OksImpl &);
public:
  /**
     Initialise system
  */
  OksImpl(bool readOnly = true);
  /**
     Free resources
  */
  ~OksImpl();

  /************* Get physical configuration ****************/

  /**
     List partition ids
     Lookup the partitions defined in the configuration and return a list of integer ID's
  */
  std::list<unsigned int> listPartitions();

  /**
     List crate ids with reference to the given partition
     Lookup the crates defined in the specified partition and return a list of integer ID's
  */
  std::list<unsigned int> listCratesInPartition(unsigned int partition);

  /**
     List the rods present in a crate
     Lookup the RODs defined in the specified crate and return a list of integer ID's
  */
  std::list<unsigned int> listRodsInCrate(unsigned int partition, unsigned int crate);

  /**
     List the MURs associated with a ROD
     Return integer id's of the MURs in the specified rod.

     FIXME:  Should be in numbered order
  */
  std::list<unsigned int> listMURSInRod(unsigned int partition, unsigned int crate, unsigned int rod);

  /**
     List the modules in the given MUR
     Return a list of serial numbers associated with the specified MUR

     FIXME:  Should be in numbered order
  */
  std::list<std::string> listModulesInMUR(unsigned int partition, unsigned int MUR);

  /**
     List the modules in the given MUR
     Return a list of serial numbers associated with the redundancy links of the specified MUR

     FIXME:  Should be in numbered order (array?)
  */
  std::list<std::string> listRModulesInMUR(unsigned int partition, unsigned int MUR);

  /************** Get configurations *****************/

  /**
     Return the configuration for the module

     Deprecated: Should take MUR argument
  */
  ABCDModule getModuleConfig(std::string module);

  /**
     Return the Rod configuration
     Find the configuration associated with the specified ROD.
     Return a RodConfig structure.
  */
  RodConfig getRodConfig(unsigned int partition, unsigned int crate, unsigned int rod);

  /**
     Get the BOC configuration of one of 48 channels 

     Input channels are paired as one.
  */
  BOCChannelConfig getBOCConfig(unsigned int partition, unsigned int crate, unsigned int rod, 
                                unsigned int channel);

  /**
     Get the BOC fibre mappings

     An array of 48*3 chars (out, in0, in1)
  */
  char *getBOCMappings(unsigned int partition, unsigned int crate, unsigned int rod);

  /**
     Return the TIM configuration
     Find the configuration associated with the specified TIM.
     Return a TimConfig structure.
  */
  TimConfig getTimConfig(unsigned int partition, unsigned int crate);

  /**
     Set fibre mappings for one channel

     Any value over 48/96 leaves current value unchanged
   */
  void setBOCMapping(unsigned int partition, unsigned int crate, unsigned int rod, 
                     unsigned int channel, unsigned int tx, unsigned int rx0, unsigned int rx1);

  /**
     Print module configuration to cout
  */
  void printModuleConfig(const ABCDModule &conf);

  /************** Change indexing schemes ****************/

  /**
     Translate from MUR / module(1-6) number to p/k/r/c
  */
  void translateToROD(unsigned int MUR, unsigned int module,
		      unsigned int &partition, unsigned int &crate, 
		      unsigned int &rod, unsigned int &channel);
 
  /**
     Translate from MUR / redundant module(1-6) number to p/k/r/c
  */
  void translateToRROD(unsigned int MUR, unsigned int module,
		       unsigned int &partition, unsigned int &crate, 
		       unsigned int &rod, unsigned int &channel);
 
  /**
     Translate from MUR / module(1-6) number to serial number
  */
  void translateToSN(unsigned int MUR, unsigned int module,
                     std::string &sn);

  /**
     Translate from MUR / module(1-6) number to geographical barrel system
  */
  void translateToBarrel(unsigned int MUR, unsigned int module,
                         unsigned int &barrel, unsigned int &row, int &number);

  /**
     Translate from MUR / module(1-6) number to geographical endcap system
   */
  void translateToEndcap(unsigned int MUR, unsigned int module,
                         unsigned int &disk, unsigned int &ring, unsigned int &number);
  /**
     Translate from MUR / module(1-6) number to power supply channel address
   */
  void translateToPowerSupply(unsigned int MUR, unsigned int module,
                              unsigned int &partition, 
                              unsigned int &crate, unsigned int &channel);

  /**
     Translate to MUR / module(1-6) number from p/k/r/c
  */
  void translateFromROD(unsigned int partition, unsigned int crate, 
                        unsigned int rod, unsigned int channel,
                        unsigned int &MUR, unsigned int &module);

  /**
     Translate to MUR / module(1-6) number from serial number
  */
  void translateFromSN(std::string sn,
                       unsigned int &MUR, unsigned int &module);

  /**
     Translate to MUR / module(1-6) number from geographical barrel system
  */
  void translateFromBarrel(unsigned int barrel, unsigned int row, int number,
                         unsigned int &MUR, unsigned int &module);

  /**
     Translate to MUR / module(1-6) number from geographical endcap system
   */
  void translateFromEndcap(unsigned int disk, unsigned int ring, unsigned int number,
                           unsigned int &MUR, unsigned int &module);


  /************** Change configuration ******************/

  /**
     Update the configuration of a module stored in memory
     Replace the configuration for the given module with the data
     stored in the given configuration
  */
  void updateModuleConfig(std::string module, ABCDModule conf);

  /**
     Update the ROD configuration stored in memory

     Replace the in memory configuration for the given ROD
     with the configuration provided.
  */
  void updateRodConfig(unsigned int partition, unsigned int crate, unsigned int rod, RodConfig conf);

  /**
     Remove all configuration.
     Very dangerous!!
   */
  void clearAll();

  /**
     Configuration creation:

     Create module (write configuration)

     Change translations

     Change MUR translations
   */

  /** 
     Name a partition (for information, any use?)
   */
  void namePartition(int partition, string name);

  /**
     Create a partition 
  */
  void createPartition(int partition);

  /**
     Create a crate 
  */
  void createCrate(int partition, int crate);

  /**
     Create a ROD
   */
  void createROD(int partition, int crate, int rod, RodConfig conf);

  /** 
     Create an MUR
  */
  void createMUR(int partition, int crate, int rod, int order, int number);

  /**
     Create a module
  */
  void createModule(int MUR, int order, int RMUR, int rorder, string number, const ABCDModule &mConf);

  /**
     Create a BOC configuration
  */
  void createBOCConfig(int MUR, int order, const BOCChannelConfig &bConf);

  /**
     Set a BOC configuration 
   */
  void setBOCConfig(int partition, int crate, int rod, int channel, const BOCChannelConfig &conf);

  /** 
     Load configuration from named file
   */
  void loadConfiguration(const std::string &filename);

  /************** Save configuration *********************/

  /**
     Save the current configuration

     Write configuration to disc.
     This creates a unique filename based on the date and time
     and saves the current configuration to it.

     Filename eg Conf_dump_20021210142311.xml.gz (in current directory)
  */
  void saveConfiguration(std::string filename = "");

 private:
  /**
     Query the configuration with an XPath expression, task is for error reporting
  */
  xmlNodePtr getQueryNode(std::string query, std::string task);

  /**
     Return contents of named attribute parsed as an int.

     Throws: runtime_error if no attribute
   */
  int getAttrAsInt(xmlNodePtr node, const char *attr);

  /**
     Return the first element node called name that is a child of node

     If not present throw exception
  */
  xmlNodePtr getChildNode(xmlNodePtr node, const char *name);

  /**
     Return the first element node called name that is a child of node

     If not present return new one
  */
  xmlNodePtr getOrNewChildNode(xmlNodePtr node, const char *name);

  /**
     Parse the xml tree provided and return the module configuration
  */
  const ABCDModule parseModuleConfig(xmlNodePtr node);

  /**
     Parse the xml tree provided and return the ROD configuration
  */
  const RodConfig parseRodConfig(xmlNodePtr node);

  /**
     Parse the xml tree provided and return the TIM configuration
  */
  const TimConfig parseTimConfig(xmlNodePtr node);

  /**
     Replace the configuration in the given node with the data
     stored in the given configuration
  */
  void replaceModuleConfig(xmlNodePtr node, ABCDModule conf, std::string name);

 private:
  OksObject *getPartition(unsigned int p);
  OksObject *getCrate(unsigned int p, unsigned int c);
  OksObject *getRod(unsigned int p, unsigned int c, unsigned int r);
  OksObject *getMUR(unsigned int m);
  OksObject *getModule(unsigned short mur, unsigned char position);

  OksObject *getObjectFromRelationship(const OksObject *obj, const std::string rel);
  unsigned long getULongData(const OksObject *obj, const std::string attr);
  unsigned short getUShortData(const OksObject *obj, const std::string attr);
  unsigned char getUCharData(const OksObject *obj, const std::string attr);
  std::string getStringData(const OksObject *obj, const std::string attr);

  OksClass *partClass;
  OksClass *crateClass;
  OksClass *rodClass;
  OksClass *murClass;
  OksClass *moduleClass;

/*    ConfdbConfiguration configuration; */

  OksKernel kernel;
};
#endif
