// File: SctApi.cxx

#include <iostream>
#include <stdexcept>
#include <cmath>
#include <cstdio>  // For tempnam (and ScanResultWriter)
#include <unistd.h>

#include "SctApi.h"
#include "sctConf/configXMLImpl.h"
#include "SctApiImpl.h"
#include "crate.h"

#include "scan.h"

#include "ABCD/ABCDscans.h"
#include "ABCD/ABCDchip.h"

#include "CommonWithDsp/primParams.h"

#include "RodCrate/RodModule.h"

#ifdef USE_IS
#include "Sct/IoExceptions.h"
#include "ScanResultWriter/ScanResultWriter.h"
#include "SctApi/dataTypes.h"
#endif

#ifdef HAVE_PRIM_NAMES
#include "decodeTestStand/primNames.h"
const char *getName(int id);
const char *getTaskName(int id);
#endif

using namespace std;
using namespace SctPixelRod;
using namespace SctConfiguration;

// Some static methods
static void handle_unexpected(void);
static void decodeVme(VmeException &e);
static void getpcrc(UINT32 mid, unsigned int &partition, unsigned int &crate, unsigned int &rod, unsigned &channel);
static UINT32 packpcrc(unsigned int &partition, unsigned int &crate, unsigned int &rod, unsigned &channel);
static void printMemoryBlock(ostream &os, unsigned long *mem, unsigned long words, int wordsPerLine, int level = 0);

namespace SctApi {

const unsigned int SctApi::BAD_MODULE = 0xffffffff;

/*
  The default constructor.
  This sets the unexpected handler to interpret all sorts of exceptions
*/
SctApi::SctApi() 
  : mrs(0), m_isDict(0), debugLevel(0), scanNumber(0), runNumber(0) 
{
  cout << "Load configuration\n";
  config.reset(new ConfigurationXMLImpl());
  setup();
}

SctApi::SctApi(boost::shared_ptr<Configuration> newConf) 
  : mrs(0), m_isDict(0), debugLevel(0), scanNumber(0), runNumber(0) 
{
  config = newConf;
  setup();
}

SctApi::SctApi(int initDebug) 
  : mrs(0), m_isDict(0), debugLevel(initDebug), scanNumber(0), runNumber(0) 
{ 
  cout << "Load configuration\n";
  config.reset(new ConfigurationXMLImpl());

 setup();
}

/* The destructor. */
SctApi::~SctApi() {
  for(map<pair<unsigned int,unsigned int>, Crate*>::const_iterator i=crateMap.begin();i!=crateMap.end();i++){
    delete i->second;
  }

  if(debugPrimList) 
    delete debugPrimList;
}

void SctApi::setup() 
{
  debugPrimList = new RodPrimList;

  // Clear contents of file
  ofstream fout("PrimLists.bin", ios::binary);

  // Examine exceptions on the way to a crash...
  set_unexpected(handle_unexpected);
  cout << "Installed unexpected handler\n";

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Initial setup complete\n";
  }
}

void SctApi::setMrsStream(MRSStream *stream) 
{
  mrs = stream;
}

#if USE_IS
void SctApi::setIsDictionary(ISInfoDictionary *dict)
{
  m_isDict = dict;
}
#endif

// ROD Diagnostics

/*
  Dump contents of a block of DSP memory to cout.
  Use dspNumber -1 to refer to the master DSP.
*/
int SctApi::dspBlockDump(unsigned int partition, unsigned int crate, unsigned int rod,
                         long dspStart, long numWords, long dspNumber){
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "DspBlockDump (" << partition << " " << crate << " " << rod << ") "
        << dspStart << " " << numWords << " " << dspNumber << endl << flush;
  }
  Crate* myCrate = getCrate(partition, crate);

  if(!myCrate) {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return 1;
  }

  // create buffer
  unsigned long * buffer = new unsigned long[numWords];

  for(int i=0; i<numWords; i++) {
    buffer[i] = 0;
  }

  try {
    // read data
    if (dspNumber == -1) {
      if(debugLevel > 1)
        cout << "Reading from mdsp\n";
      myCrate->mdspBlockRead(rod, dspStart, buffer, numWords);
    } else {
      if(debugLevel > 1)
        cout << "Reading from dsp " << dspNumber << endl;
      myCrate->slvBlockRead(rod, dspStart, buffer, numWords, dspNumber);
    }
  } catch (PrimListException &p) {
    cerr << "Dsp Block Read:\n";
    cerr << "Exception sending primlist:\n";
    cerr << p.getDescriptor() << " " << p.getData1() << ", " << p.getData2() << endl;
  } catch (HpiException &h) {
    cerr << "Dsp Block Read:\n";
    cerr << "HpiException:\n";
    hex(cerr);
    cerr << h.getDescriptor() << '\n';
    cerr << "calcAddr: " << h.getCalcAddr() << ", readAddr: " <<
      h.getReadAddr() << '\n';
    dec(cerr);
  } catch (RodException &r) {
    cerr << "Dsp Block Read:\n";
    cerr << "Rod Exception\n";
    cerr << r.getDescriptor() <<", " << r.getData1() << ", " << r.getData2()
         << '\n';
  } catch (NoImageFile &f) {
    cerr << "Dsp Block Read:\n";
    cerr << "No image file exception " << f.getFileName() << endl;
  } catch (VmeException &e) {
    cerr << "Dsp Block Read:\n";
    decodeVme(e);
  }

  // output the data
  hex(cout);

  cout.fill('0');
  for (int i=0; i<numWords; i+=8) {
    cout.width(8);
    cout << buffer[i]   << ' ';
    cout.width(8);
    cout << buffer[i+1] << ' ';
    cout.width(8);
    cout << buffer[i+2] << ' ';
    cout.width(8);
    cout << buffer[i+3] << ' ';
    cout.width(8);
    cout << buffer[i+4] << ' ';
    cout.width(8);
    cout << buffer[i+5] << ' ';
    cout.width(8);
    cout << buffer[i+6] << ' ';
    cout.width(8);
    cout << buffer[i+7] << endl;
  }
  dec(cout);
  cout.fill(' ');

  char *name = tempnam("./", "textB");
  if(debugLevel > 1)
    cout << "Write output to " << name << endl;
  ofstream output(name);
  free(name);

  output.write((char *)buffer, numWords * 4);
 
  // delete the buffer
  delete [] buffer;

  return 0;
}

/*
  Dump contents of a block of DSP memory to a file.
  Use dspNumber -1 to refer to the master DSP.
*/
int SctApi::dspBlockDumpFile(unsigned int partition, unsigned int crate, unsigned int rod,
                             long dspStart, long numWords, long dspNumber, string filename){
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "DspBlockDumpFile (" << partition << " " << crate << " " << rod << ") "
        << dspStart << " " << numWords << " " << dspNumber << " to " << filename << endl << flush;
  }

  Crate* myCrate = getCrate(partition, crate);

  if(!myCrate) {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return 1;
  }

  // create buffer
  unsigned long * buffer = 0;
  try {
    // read data
    if (dspNumber == -1) {
      if(debugLevel > 1)
        cout << "Reading from mdsp\n";
      // create buffer
      buffer = new unsigned long[numWords];

      // Clear buffer
      for(int i=0; i<numWords; i++) {
        buffer[i] = 0;
      }

      myCrate->mdspBlockRead(rod, dspStart, buffer, numWords);
    } else {
      if(debugLevel > 1)
        cout << "Reading from dsp " << dspNumber << endl;
      buffer = readSlaveDsp(partition, crate, rod, dspNumber, dspStart, numWords);
    }
  } catch (BaseException &e) {
    cerr << "Dsp Block Read:\n";
    cerr << "Exception :\n";
    cerr << e << endl;
    return 1;
  }

  if(debugLevel > 1)
    cout << "Writing output to " << filename << endl;
  ofstream output(filename.c_str());

  output.write((char *)buffer, numWords * 4);

  // delete the buffer
  delete [] buffer;

  return 0;
}

/*
  Read contents of a block of DSP memory 

  Use dspNumber -1 to refer to the master DSP.
*/
unsigned long *SctApi::dspBlockRead(unsigned int partition, unsigned int crate, unsigned int rod,
                                    long dspStart, long numWords, long dspNumber, long &length){
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "DspBlockRead (" << partition << " " << crate << " " << rod << ") "
        << dspStart << " " << numWords << " " << dspNumber << endl << flush;
  }
  Crate* myCrate = getCrate(partition, crate);

  if(!myCrate) {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return 0;
  }

  // create buffer
  unsigned long * buffer = new unsigned long[numWords];

  for(int i=0; i<numWords; i++) {
    buffer[i] = 0;
  }

  try {
    // read data
    if (dspNumber == -1) {
      if(debugLevel > 1)
        cout << "Reading from mdsp\n";
      myCrate->mdspBlockRead(rod, dspStart, buffer, numWords);
    } else {
      if(debugLevel > 1)
        cout << "Reading from dsp " << dspNumber << endl;
      myCrate->slvBlockRead(rod, dspStart, buffer, numWords, dspNumber);
    }
  } catch (PrimListException &p) {
    cerr << "Dsp Block Read:\n";
    cerr << "Exception sending primlist:\n";
    cerr << p.getDescriptor() << " " << p.getData1() << ", " << p.getData2() << endl;
  } catch (HpiException &h) {
    cerr << "Dsp Block Read:\n";
    cerr << "HpiException:\n";
    hex(cerr);
    cerr << h.getDescriptor() << '\n';
    cerr << "calcAddr: " << h.getCalcAddr() << ", readAddr: " <<
      h.getReadAddr() << '\n';
    dec(cerr);
  } catch (RodException &r) {
    cerr << "Dsp Block Read:\n";
    cerr << "Rod Exception\n";
    cerr << r.getDescriptor() <<", " << r.getData1() << ", " << r.getData2()
         << '\n';
  } catch (NoImageFile &f) {
    cerr << "Dsp Block Read:\n";
    cerr << "No image file exception " << f.getFileName() << endl;
  } catch (VmeException &e) {
    cerr << "Dsp Block Read:\n";
    decodeVme(e);
  }

  length = numWords;

  // Don't delete the buffer!!!
  return buffer;
}

int SctApi::dspBlockWrite(unsigned int partition, unsigned int crate, unsigned int rod,
                          unsigned long *buffer, unsigned long dspAddress, long numWords, long dspNumber){
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "DspBlockWrite (" << partition << " " << crate << " " << rod << ") "
        << dspAddress << " " << numWords << " " << dspNumber << endl << flush;
  }

  Crate* myCrate = getCrate(partition, crate);

  if(!myCrate) {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return 0;
  }

  try {
    // Write data
    if (dspNumber == -1) {
      if(debugLevel > 1)
        cout << "Writing from mdsp\n";
      myCrate->mdspBlockWrite(rod, dspAddress, buffer, numWords);
    } else {
      if(debugLevel > 1)
        cout << "Writing from dsp " << dspNumber << endl;
      myCrate->slvBlockWrite(rod, dspAddress, buffer, numWords, dspNumber);
    }
  } catch (PrimListException &p) {
    cerr << "Dsp Block Read:\n";
    cerr << "Exception sending primlist:\n";
    cerr << p.getDescriptor() << " " << p.getData1() << ", " << p.getData2() << endl;
  } catch (HpiException &h) {
    cerr << "Dsp Block Read:\n";
    cerr << "HpiException:\n";
    hex(cerr);
    cerr << h.getDescriptor() << '\n';
    cerr << "calcAddr: " << h.getCalcAddr() << ", readAddr: " <<
      h.getReadAddr() << '\n';
    dec(cerr);
  } catch (RodException &r) {
    cerr << "Dsp Block Read:\n";
    cerr << "Rod Exception\n";
    cerr << r.getDescriptor() <<", " << r.getData1() << ", " << r.getData2()
         << '\n';
  } catch (NoImageFile &f) {
    cerr << "Dsp Block Read:\n";
    cerr << "No image file exception " << f.getFileName() << endl;
  } catch (VmeException &e) {
    cerr << "Dsp Block Read:\n";
    decodeVme(e);
  }

  return 0;
}

// Read one ROD Status Register via HPI
unsigned long SctApi::readRodStatusReg(unsigned int partition, unsigned int crate, unsigned int rod,
                                       long regNumber) {
  unsigned long result = 0;

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "readRodStatusReg " << regNumber << endl;
  }

  Crate *myCrate = getCrate(partition, crate);
  if(myCrate) {
    result = myCrate->readRodStatusReg(rod, regNumber);
  }

  return result;
}
  
// Read one ROD Command Register via HPI
unsigned long SctApi::readRodCommandReg(unsigned int partition, unsigned int crate, unsigned int rod,
                                        long regNumber) {
  unsigned long result = 0;

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "readRodCommandReg\n";
  }

  Crate *myCrate = getCrate(partition, crate);
  if(myCrate) {
    result = myCrate->readRodCommandReg(rod, regNumber);
  }

  return result;
}

// Read a single 32b word from MasterDSP SDRAM via HPI
unsigned long SctApi::dspSingleRead(unsigned int partition, unsigned int crate, unsigned int rod,
                                    const unsigned long dspAddr, long dspNumber) {
  unsigned long result = 0;

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "dspSingleRead 0x" << hex << dspAddr << dec << endl;
  }
  Crate *myCrate = getCrate(partition, crate);
  if(myCrate) {
    result = myCrate->dspSingleRead(rod, dspAddr, dspNumber);
  }

  return result;
}

// Write a single 32b word to MasterDSP SDRAM via HPI
void SctApi::dspSingleWrite(unsigned int partition, unsigned int crate, unsigned int rod,
                            unsigned long dspAddr, unsigned long value, long dspNumber) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "dspSingleWrite " << dspAddr << " " << value << endl;
  }
  Crate *myCrate = getCrate(partition, crate);
  if(myCrate) {
    myCrate->dspSingleWrite(rod, dspAddr, value, dspNumber);
  }
}

ABCDModule *SctApi::lookupConfig(UINT32 mid) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "lookupconfig " << mid << endl;
  }

  if(moduleMap.find(mid) == moduleMap.end()) {
    cacheModuleConfig(mid);
  }

  ABCDModule *result = 0;

  if(moduleMap.find(mid) != moduleMap.end()) {
    result = &(moduleMap.find(mid)->second);
  }

  return result;
}

void SctApi::cacheModuleConfig(UINT32 mid) {
  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  std::string moduleName;
  unsigned int mur, module;

  // First find out what this module is called
  try {
    if(debugLevel > 1)
      cout << "Looking for partition " << partition << " crate " << crate << " rod " << rod << " channel " << channel << endl;
    config->translateFromROD(partition, crate, rod, channel, 
                             mur, module);

    if(debugLevel > 2)
      cout << "Found MUR " << mur << " module number " << module << endl;
  } catch(ConfigurationException &c) {
    cout << "Can't find module in MUR map: \n";
    cout << c.what() << endl;

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "  (Not found)\n";
    }
    return;
  }

  try {
    config->translateToSN(mur, module, moduleName);

    if(debugLevel > 2)
      cout << "Found module name " << moduleName;
    //        config->getModuleRodPosition(moduleName, partition, crate, rod, channel);
  } catch(ConfigurationException &c) {
    cout << "Module name not found: \n";
    cout << c.what() << endl;

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "  (Not found)\n";
    }
    return;
  }

  bool haveRedundant = false;

  unsigned int rchannel;

  try {
    unsigned int rpartition, rcrate, rrod;
    config->translateToRROD(mur, module, 
                            rpartition, rcrate, rrod, rchannel);

    if(partition == rpartition && crate == rcrate && rod == rrod) {
      haveRedundant = true;
    }
  } catch(ConfigurationException &c) {
    cout << "Module redundancy not found: \n";
    cout << c.what() << endl;

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "  (Not found)\n";
    }
  }

  // Got the module serial number, now get the configuration
  ABCDModule configuration; 
  try {
    configuration = config->getModuleConfig(moduleName);

    char *mappings = config->getFibreMappings(partition, crate, rod);
    if(configuration.select) {
      for(int c=0; c<12; c++) {
        configuration.chip[c].address |= 0x10;
      }
    } 

    configuration.pTTC  = mappings[channel * 3 + 0];
    if(haveRedundant)
      configuration.rTTC = mappings[rchannel * 3 + 0];
    else 
      configuration.rTTC = OFF_ROD_TTC;
    configuration.rx[0] = mappings[channel * 3 + 1];
    configuration.rx[1] = mappings[channel * 3 + 2];

    cout << "pTTC = " << (int)configuration.pTTC << " rTTC = " << (int)configuration.rTTC 
         << " rx[0] = " << (int)configuration.rx[0] << " rx[1] = " << (int)configuration.rx[1] << endl;

    moduleMap.insert(make_pair(mid, configuration));

    if(mrs) {
      *mrs << "MODULE_LOADED" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<const char *>("sn", moduleName.c_str())
           << MRS_PARAM<int>("group", configuration.groupId)
           << MRS_PARAM<int>("pTTC", configuration.pTTC)
           << MRS_PARAM<int>("rTTC", configuration.rTTC)
           << MRS_PARAM<int>("rx0", configuration.rx[0])
           << MRS_PARAM<int>("rx1", configuration.rx[1])
           << MRS_TEXT("Module configuration loaded")
           << ENDM;
    }

    delete [] mappings;
  } catch(ConfigurationException &c) {
    cout << "Module not initialised: \n";
    cout << c.what() << endl;

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "  (Not found)\n";
    }
  }
}

ABCDModule *SctApi::retrieveModule(UINT32 mid) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "retrieveModule " << mid << endl;
  }
  return lookupConfig(mid);
}

// Create a prim list
void SctApi::createDebugPrimList() {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "createDebugPrimList\n";
  }
  debugPrimList->clear();
}

// Append a primitive to the prim list
void SctApi::addDebugPrimList(unsigned long length, long index, long id, long version,
                              unsigned long * body) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "addDebugPrimList ID: " << id << " length: " << length << endl;
  }
  debugPrimList->push_back(RodPrimitive((long)length, index, id, version, (long *)body));
}

// Send primlist
void SctApi::sendDebugPrimList(unsigned int partition, unsigned int crate, unsigned int rod) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "sendDebugPrimList " << partition << " " << crate << " " << rod << endl;
  }
  sendPrimList(partition, crate, rod, debugPrimList);
}

void SctApi::sendDebugSlavePrimList(unsigned int partition, unsigned int crate, unsigned int rod,
                                    unsigned int slave, bool await, bool response) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "sendDebugSlavePrimList \n";
  }
  sendSlavePrimList(partition, crate, rod, debugPrimList, slave, await, response);
}

/*
  Initialise a module:
  Load configuration from database
  Send configuration to ROD

  return 0 for success
*/
int SctApi::initialiseModule(string modulename) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "InitialiseModule " << modulename << endl << flush;
  }

  unsigned int partition, crate, rod, channel;

  // First find out where this module is
  try {
    unsigned int MUR, module;
    config->translateFromSN(modulename, MUR, module);
    config->translateToROD(MUR, module, partition, crate, rod, channel);
  } catch(ConfigurationException &c) {
    cout << "Module not initialised: \n";
    cout << c.what() << endl;
    return -1;
  }

  cout << "Initialise module " << modulename << " at " <<
    partition << ", " << crate << ", " << rod << ", " << channel << " \n";

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << " found at " <<
      partition << ", " << crate << ", " << rod << ", " << channel << " \n" << flush;
  }

  cacheModuleConfig(packpcrc(partition, crate, rod, channel));

  // Install configuration in ROD
  setABCDModule(packpcrc(partition, crate, rod, channel), 0);
  setABCDModule(packpcrc(partition, crate, rod, channel), 1);

  return 0;
}

/*
   Initialise a rod from the configuration
 */
int SctApi::initialiseRod(unsigned int partition, unsigned int crate, unsigned int rod) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "InitialiseRod\n";
  }
  Crate *myCrate = getCrate(partition, crate);
  if(!myCrate) {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return -1;
  } else {
    if(!(myCrate->initialiseRod(rod) == 0)) {
      return -2;
    }
  }

  return 0;
}

/*
   Initialise a TIM from the configuration
 */
int SctApi::initialiseTim(unsigned int partition, unsigned int crate) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "InitialiseTim\n";
  }
  Crate *myCrate = getCrate(partition, crate);
  if(!myCrate) {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return -1;
  } else {
    if(!(myCrate->initialiseTim() == 0)) {
      return -2;
    }
  }

  return 0;
}

/*
   Initialise a BOC from the configuration
 */
int SctApi::initialiseBOC(unsigned int partition, unsigned int crate, unsigned int rod) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Initialise BOC\n";
  }
  Crate *myCrate = getCrate(partition, crate);
  if(!myCrate) {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return -1;
  } else {
      if(!(myCrate->initialiseBOC(rod) == 0)) {
        return -2;
      }
  }

  return 0;
}

/*
   Initialise everything from the configuration.
   Uses initialiseRod and initialiseModule to do actual configuration
 */
void SctApi::initialiseAll(int run) {
  runNumber = run;
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "InitialiseAll " << runNumber << "\n";
  }
  unsigned int totalPartitions = 0;
  unsigned int totalCrates = 0;
  unsigned int totalRods = 0;
  unsigned int totalModules = 0;

  // Clear up from previous time if necessary
  for(map<pair<unsigned int,unsigned int>, Crate*>::const_iterator i=crateMap.begin();i!=crateMap.end();i++){
    delete i->second;
  }

  crateMap.clear();
  debugPrimList->clear();

  // Read in configuration and act accordingly
  list<unsigned int> partitions; 
  try {
    partitions = config->listPartitions();
  } catch(ConfigurationException &c) {
    cout << "No partitions to initialise\n";
    cout << c.what() << endl;
    return;
  }

  for(list<unsigned int>::const_iterator p=partitions.begin();
      p!=partitions.end(); 
      p++) {

    list<unsigned int> crates; 

    try {
      crates = config->listCratesInPartition(*p);
    } catch(ConfigurationException &c) {
      cout << "No crates in this partition\n";
      cout << c.what() << endl;
      continue;
    }

    for(list<unsigned int>::const_iterator c=crates.begin(); 
        c!=crates.end(); 
        c++) {
      try {
        crateMap.insert(make_pair(make_pair(*p, *c), new Crate(*p, *c, config)));

        if(mrs)
          getCrate(*p, *c)->setMrsStream(mrs);

        if(!initialiseTim(*p, *c)) {
          cout << "TIM Successful\n";
        }

        list<unsigned int> rods;
        try {
          rods = config->listRodsInCrate(*p, *c);
        } catch(ConfigurationException &c) {
          cout << "No RODs in this crate\n";
          cout << c.what() << endl;
          continue;
        }

        for(list<unsigned int>::const_iterator r=rods.begin(); 
            r!=rods.end(); 
            r++) {
          if(initialiseRod(*p, *c, *r)) {
            continue;
          } else {
            totalRods ++;

            initialiseBOC(*p, *c, *r);
            if(debugLevel > 1) 
              cout << "Should have initialised BOC now" << endl;
          }

          list<unsigned int> MURs;
          try {
            MURs = config->listMURSInRod(*p, *c, *r);
          } catch(ConfigurationException &c) {
            cout << "No MURs for this ROD\n";
            cout << c.what() << endl;
            continue;
          }

          for(list<unsigned int>::const_iterator mur=MURs.begin(); 
              mur!=MURs.end(); 
              mur++) {

            list<string> modules;
            try {
              modules = config->listModulesInMUR(*p, *mur);
            } catch(ConfigurationException &c) {
              cout << "No modules for this MUR\n";
              cout << c.what() << endl;
              continue;
            }

            for(list<string>::const_iterator m=modules.begin(); 
                m!=modules.end(); 
                m++) {
              if(!initialiseModule(*m)) {
                totalModules ++;
              }
            }
          }

          getCrate(*p, *c)->status();
        }
        totalCrates++;
      } catch(...) {
        // VME initialisation failed (crate already told MRS)
      }
    }
    totalPartitions ++;
  }

  // Load modules first so we know which channels to unmask
  cout << "Setup for calibration...\n";
  calib_init();
  cout << "... Done\n";

  cout << "Initialisation counts:\n";
  cout << "\ttotalPartitions: " << totalPartitions << endl;
  cout << "\ttotalCrates: " << totalCrates << endl;
  cout << "\ttotalRods: " << totalRods << endl;
  cout << "\ttotalModules: " << totalModules << endl;

  if(mrs) {
    *mrs << "SCTAPIINIT_COMPLETE" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
         << MRS_TEXT("SCTAPI Initialisation counts")
         << MRS_PARAM<int>("partitions", totalPartitions)
         << MRS_PARAM<int>("crates", totalCrates)
         << MRS_PARAM<int>("rods", totalRods)
         << MRS_PARAM<int>("modules", totalModules)
         << ENDM;
  }

  cout << "************** SCT API initialised ****************\n";
}

void SctApi::setRunNumber(UINT32 newRun) {
  if(newRun < runNumber) {
    if(mrs) {
      *mrs << "SCTAPI_RUNNUMBER" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
           << MRS_TEXT("SCTAPI runnumber decrease")
           << MRS_PARAM<int>("oldRunNumber", runNumber)
           << MRS_PARAM<int>("newRunNumber", newRun)
           << MRS_PARAM<int>("newScanNumber", 0)
           << ENDM;
    }
    cout << "****  Run number has been decreased!!\n";
    runNumber = newRun;
    scanNumber = 0;
  } else if(newRun == runNumber) {
    if(mrs) {
      *mrs << "SCTAPI_RUNNUMBER" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
           << MRS_TEXT("SCTAPI runnumber unchanged")
           << MRS_PARAM<int>("oldRunNumber", runNumber)
           << MRS_PARAM<int>("newRunNumber", newRun)
           << MRS_PARAM<int>("newScanNumber", scanNumber)
           << ENDM;
    }
    cout << "**** Run number is unchanged scan number not reset!!!\n";
  } else {
    if(mrs) {
      *mrs << "SCTAPI_RUNNUMBER" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
           << MRS_TEXT("SCTAPI runnumber changed")
           << MRS_PARAM<int>("oldRunNumber", runNumber)
           << MRS_PARAM<int>("newRunNumber", newRun)
           << MRS_PARAM<int>("newScanNumber", 0)
           << ENDM;
    }
    runNumber = newRun;
    scanNumber = 0;
  }
}

void SctApi::setScanNumber(UINT32 newScan) {
  if(mrs) {
    *mrs << "SCTAPI_SCANNUMBER" << MRS_INFORMATION << MRS_QUALIF("SCTAPI");
  }

  if(newScan < scanNumber) {
    cout << "**** Scan number has been decreased!!\n";
    if(mrs) {
      *mrs << MRS_TEXT("SCTAPI scannumber decreased");
    }
  } else {
    if(mrs) {
      *mrs << MRS_TEXT("SCTAPI scannumber changed");
    }
  }

  if(mrs) {
    *mrs << MRS_PARAM<int>("oldScanNumber", scanNumber)
         << MRS_PARAM<int>("newScanNumber", newScan)
         << ENDM;
  }

  scanNumber = newScan;
}

/*
   Send a PrimList to a ROD

   This function exposes RodPrimList so make it private.
   Therefore to be used internally to this class.
*/
void SctApi::sendPrimList(unsigned int partition, unsigned int crate, unsigned int rod,
                          RodPrimList *prim) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "SendPrimList (" << partition << " " << crate << " " << rod << ")\n" << flush;
  }

  prim->bufferBuild();

  long buffLength = prim->getBufferLength();
  unsigned long* bufferStart = prim->getBuffer();

  if(debugLevel > 4)
    printMemoryBlock(log, bufferStart, buffLength, 8);

  ofstream fout("PrimLists.bin", ios::binary | ios::app);
  fout.write((char*)&bufferStart[0], buffLength*4);

  if(debugLevel > 3)
    printOutList(bufferStart, buffLength, true);

  if(getCrate(partition, crate)) {
    getCrate(partition, crate)->sendPrimList(rod, prim);
  } else {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
  }
}

/*
   Send a PrimList to all RODs in a crate

   This function exposes RodPrimList so make it private.
   Therefore to be used internally to this class.
*/
void SctApi::sendPrimListAll(unsigned int partition, unsigned int crate,
                             RodPrimList *prim) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "SendPrimListAll (" << partition << " " << crate << ")\n" << flush;
  }

  prim->bufferBuild();

  long buffLength = prim->getBufferLength();
  unsigned long* bufferStart = prim->getBuffer();

  printMemoryBlock(log, bufferStart, buffLength, 8);

  ofstream fout("PrimLists.bin", ios::binary | ios::app);
  fout.write((char*)&bufferStart[0], buffLength*4);

  if(debugLevel > 3)
    printOutList(bufferStart, buffLength, true);

  if(getCrate(partition, crate)) {
    getCrate(partition, crate)->sendPrimListAll(prim);
  } else {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
  }
}

/*
   Shutdown anything that needs shutting down
   Send DCS to standby message
 */
void SctApi::shutdownAll() {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "SctApi told to shutdown\n";
  }
  // What does shutdown involve?
  //  lasersOff();   ????

  unsigned long bufLength = max(REPLY_BUFF_SIZE, PRIM_BUFF_SIZE);
  bufLength = max(bufLength, TEXT_BUFF_SIZE);

  // Only read out info from the first crate (XXX)
  Crate* myCrate = getCrate(0, 0);
  if(myCrate) {
    unsigned long *buffer = new unsigned long[REPLY_BUFF_SIZE];

    // This is what Doug requested for error diagnoses so put in the log at the end
    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "PRIMITIVE Buffer " << hex << PRIM_BUFF_BASE << " " << PRIM_BUFF_SIZE/4 << dec << "words\n";
    }
    myCrate->mdspBlockRead(0, PRIM_BUFF_BASE, buffer, PRIM_BUFF_SIZE/4);

    {
      boost::mutex::scoped_lock lock(logMutex);
      printMemoryBlock(log, buffer, PRIM_BUFF_SIZE/4, 8);
    }

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "REPLY buffer " << hex << REPLY_BUFF_BASE << " " << REPLY_BUFF_SIZE/4 << dec << "words\n";
    }
    myCrate->mdspBlockRead(0, REPLY_BUFF_BASE, buffer, REPLY_BUFF_SIZE/4);
    {
      boost::mutex::scoped_lock lock(logMutex);
      printMemoryBlock(log, buffer, REPLY_BUFF_SIZE/4, 8);
    }

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "ERROR text buffer " << hex << ERR_BUFF_BASE << " " << TEXT_BUFF_SIZE/4 << dec << "words\n";
    }
    myCrate->mdspBlockRead(0, ERR_BUFF_BASE, buffer, TEXT_BUFF_SIZE/4);
    {
      boost::mutex::scoped_lock lock(logMutex);
      printMemoryBlock(log, buffer, TEXT_BUFF_SIZE/4, 8);
    }

    delete [] buffer;
  }

  if(mrs) {
    *mrs << "SCTAPI_SHUTDOWN" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") << MRS_TEXT("First ROD shut down") << ENDM;
  }

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Shutdown sequence complete\n";
  }
}

/*
    Flash the LED associated with a Slave DSP.
    This creates its own primitive list and sends it to the ROD
*/
void SctApi::flashLED(unsigned int partition, unsigned int crate, unsigned int rod,
                      unsigned long slaveNumber, long period, long flashes) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "FlashLED\n" << flush;
  }

  RodPrimList primList(1);                      // Primitive List

  // Create a FLASH_LED primitive to be sent to a slave DSP
  long *ledData = new long[sizeof(FLASH_LED_IN)/4];

  FLASH_LED_IN &ledParam = *(FLASH_LED_IN *)ledData;

#if (R_FLASH_LED == 103) 
  ledParam.ledNum = YELLOW_LED;
  ledParam.period = period;
  ledParam.numTimes = flashes;
#else 
#error "FLASH_LED not compiled (revision change)"
#endif

  RodPrimitive ledPrim(4 + sizeof(FLASH_LED_IN)/4, 
                       0, FLASH_LED, R_FLASH_LED, ledData);
  primList.push_back(ledPrim);

  sendSlavePrimList(partition, crate, rod, &primList, slaveNumber, 1, 0);

  delete [] ledData;

  return;
}

/*
   Create a RodPrimList to send to the Master DSP which contains the RodPrimList which is passed as an argument
 */
void SctApi::sendSlavePrimList(unsigned int partition, unsigned int crate, unsigned int rod,
                               RodPrimList *prim, unsigned int slaveNumber, bool await, bool response) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "SendSlavePrimlist (" << partition << " " << crate << " " << rod << ")\n" << flush;
  }

  if(!getCrate(partition, crate)->slavePresent(rod, slaveNumber)) {
    cout << "***** The slave DSP was not initialised, please check the configuration and the Slave Image files\n";
  }

  RodPrimList container(1);

  buildSlavePrimList(container, prim, slaveNumber, await, response);

  try {
    container.bufferBuild();
  }
  catch (PrimListException &p) {
    cout << p.getDescriptor() << " ";
    cout << p.getData1() << ", " << p.getData2() << "\n";
  }

  try {
    sendPrimList(partition, crate, rod, &container); 
  }
  catch (HpiException &h) {
    hex(cout);
    cout << h.getDescriptor() << '\n';
    cout << "calcAddr: " << h.getCalcAddr() << ", readAddr: " <<
            h.getReadAddr() << '\n';
    dec(cout);
  }
  catch (PrimListException &p) {
    cout << p.getDescriptor() << " ";
    cout << p.getData1() << ", " << p.getData2() << "\n";
  }
}

void SctApi::buildSlavePrimList(RodPrimList &primList, RodPrimList *prim, unsigned int slaveNumber, bool await, bool response) {
  try {
    prim->bufferBuild();
  }
  catch (PrimListException &p) {
    cout << "Exception building prim list for slave " << slaveNumber << endl;
    cout << p.getDescriptor() << " ";
    cout << p.getData1() << ", " << p.getData2() << "\n";
    return;
  }

  // Get a copy of buffer and use SEND_SLAVE_LIST/START_SLAVE_LIST primitives
  // to have MDSP send it to the slave
  long buffLength = prim->getBufferLength();
  unsigned long* bufferStart = prim->getBuffer();

  // Create Slave List primitives
  int offset = sizeof(SEND_SLAVE_LIST_IN)/4;

  long *slaveData = new long[offset+buffLength];

  SEND_SLAVE_LIST_IN &slaveList = *(SEND_SLAVE_LIST_IN *)slaveData;

#if (R_SEND_SLAVE_LIST == 103)
  slaveList.slaveNumber = slaveNumber;
  slaveList.listLength = buffLength;
  slaveList.slavePrimList = (unsigned int*)DEFAULT;
  slaveList.slaveRepData = (unsigned int*)DEFAULT;
#else 
#error "SEND_SLAVE_LIST not compiled (new revision)"
#endif

  // Copy prim list as send data
  for (int i=0; i< buffLength; i++) {
    slaveData[offset + i] = bufferStart[i];
  }

  primList.push_back(RodPrimitive(4+offset+buffLength, 1, SEND_SLAVE_LIST,
                                  R_SEND_SLAVE_LIST, slaveData));

  long *startData = new long[sizeof(START_SLAVE_LIST_IN)/4];

  START_SLAVE_LIST_IN &startPrim = *(START_SLAVE_LIST_IN*)startData;
#if (R_START_SLAVE_LIST == 103)
  startPrim.slaveNumber = slaveNumber;
  startPrim.pauseMasterList = await?1:0;
  startPrim.getSlaveReply = response?1:0;
#else
#error "START_SLAVE_LIST not compiled (new revision)"
#endif

  primList.push_back(RodPrimitive(7, 2, START_SLAVE_LIST,
                                  R_START_SLAVE_LIST, startData));
#warning "Memory leak Can't delete slaveData and startData till primList is finished"
}

void SctApi::echo(unsigned int partition, unsigned int crate, unsigned int rod,
                  unsigned int length, long *data) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Echo primitive\n";
  }

  RodPrimList *primList = new RodPrimList(1);

  // Create and Send a simple ECHO primitive
#if (R_ECHO != 100) 
#error "ECHO revision changed!"
#endif
  primList->push_back(RodPrimitive(4+length, 1, ECHO, R_ECHO, data));

//    // Redundant
//    try {
//      primList->bufferBuild();
//    }
//    catch (PrimListException &p) {
//      cout << p.getDescriptor() << " ";
//      cout << p.getData1() << ", " << p.getData2() << "\n";
//    }

  try {
    sendPrimList(partition, crate, rod, primList);
  }
  catch (HpiException &h) {
    hex(cout);
    cout << h.getDescriptor() << '\n';
    cout << "calcAddr: " << h.getCalcAddr() << ", readAddr: " <<
            h.getReadAddr() << '\n';
    dec(cout);
  }
  catch (PrimListException &p) {
    cout << p.getDescriptor() << " ";
    cout << p.getData1() << ", " << p.getData2() << "\n";
  }

  if(debugLevel>1) {
    cout << "Echo primList sent\n";
  }

#warning "This should be here but means it won't work with the threading..."
#ifndef USE_THREADS
  delete primList;
#endif
}

/*
    Lookup the Crate object to send instructions to.
    Return 0 if not found.
 */
Crate *SctApi::getCrate(unsigned int partition, unsigned int crate) const {
  map<pair<unsigned int, unsigned int>, Crate* >::const_iterator found;

  found = crateMap.find(make_pair(partition, crate));
  if(found == crateMap.end()) return 0;
  else return found->second;
}

void SctApi::awaitResponse(unsigned int partition, unsigned int crate, unsigned int rod, int timeout) {
  int retVal = 0;
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "AwaitResponse\n";
  }

  responseCode = 0;
  if(getCrate(partition, crate)) {
    try {
      retVal = getCrate(partition, crate)->awaitResponse(rod, timeout);
    } catch(...) {
      cout << "Exception in await response\n";
    }

    if(retVal != 0) {
      cout << "Problem during await response\n";
      responseCode = retVal;
    }
  } else {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
  }
}

void SctApi::awaitResponseAll(unsigned int partition, unsigned int crate, int timeout) {
  int retVal = 0;
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "AwaitResponse\n";
  }

  responseCode = 0;
  if(getCrate(partition, crate)) {
    try {
      retVal = getCrate(partition, crate)->awaitResponseAll(timeout);
    } catch(...) {
      cout << "Exception in await response\n";
    }

    if(retVal != 0) {
      cout << "Problem during await response\n";
      responseCode = retVal;
    }
  } else {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
  }
}

unsigned long *SctApi::getResponse(unsigned int partition, unsigned int crate, unsigned int rod, 
                                   unsigned long &length) {
  unsigned long *result;

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "GetResponse\n";
  }
  if(getCrate(partition, crate)) {
    shared_ptr<RodOutList> output = getCrate(partition, crate)->getResponse(rod);

    if(!output) {
      cout << "No response from crate\n";
      return NULL;
    }

    length = (unsigned long) output->getLength();

    if(debugLevel > 1)
      cout << "Got response from crate length = " << length << endl;

    unsigned long *body = output->getBody();

    result = new unsigned long[length];

    for(unsigned int i=0; i<length; i++) {
      result[i] = body[i];
    }

    if(debugLevel > 3)
      printOutList(result, length);
  } else {
    cout << "Request for non-existent crate " << partition << " " << crate << endl;
    return NULL;
  }

  return result;
}

void SctApi::printOutList(unsigned long *result, unsigned long length, bool in, int level) {
  unsigned long outLength = result[0];
  unsigned long outIndex = result[1];
  unsigned long outNumPrims = result[2];
  unsigned long outPrimVersion = result[3];

  ostream *tempStream;
  if(debugLevel>3) {
    tempStream = &log;
  } else {
    tempStream = &cout;
  }
  ostream &stream(*tempStream);

  if(!in) {
    stream << string("        ", 0, level);
    stream << "Outlist decode: \n";
    stream << string("        ", 0, level);
    stream << "outLength = " << outLength << ", outIndex = " << outIndex << 
      ", outNumPrims = " << outNumPrims << ", outPrimVersion = " << 
      outPrimVersion <<'\n';
  } else {
    stream << string("        ", 0, level);
    stream << "Inlist decode: \n";
    stream << string("        ", 0, level);
    stream << "inLength = " << outLength << ", inIndex = " << outIndex << 
      ", inNumPrims = " << outNumPrims << ", inPrimVersion = " << 
      outPrimVersion <<'\n';
  }
  unsigned long *outPtr = result + 4;
  for (unsigned int j=0; j<outNumPrims; j++) {
    unsigned long primLength = *(outPtr++);
    unsigned long primIndex = *outPtr++;
    unsigned long primId = *outPtr++;
    unsigned long primVersion = *outPtr++;
    stream << string("        ", 0, level);
    stream << "primLength = " << primLength << ", primIndex = " << primIndex << 
      ", primId = " << primId << ", primVersion = " << primVersion;
#ifdef HAVE_PRIM_NAMES
    stream << " " << getName(primId);
#endif
    stream << '\n';
    if(in) { // Input list
      if(primId == SEND_SLAVE_LIST) {
        stream << string("        ", 0, level);
        stream << " Send list to slave DSP " << outPtr[0] << " length " << outPtr[1] << ":\n";
        printOutList(outPtr + 4, outPtr[1], in, level+2);
      } else if(primId == START_SLAVE_LIST) {
        stream << string("        ", 0, level);
        stream << " Start slave list on " << outPtr[0] << endl;
      } else if(primId == START_TASK) {
        START_TASK_IN &prim =  *(START_TASK_IN *)outPtr;

        stream << string("        ", 0, level);
        int taskId = outPtr[0];

#ifdef HAVE_PRIM_NAMES
        stream << "Start task " << getTaskName(outPtr[0]) << " (" << outPtr[0] << ")" << endl;
#else
        stream << "Start task " << " (" << outPtr[0] << ")" << endl;
#endif
        unsigned long *taskPtr = outPtr + 4;

        int tLevel = level + 1;

#if (R_HISTOGRAM_CTRL_TASK == 107)
        if(taskId == HISTOGRAM_CTRL_TASK) {
          HISTOGRAM_CTRL_TASK_IN &task = *(HISTOGRAM_CTRL_TASK_IN*)taskPtr;
          stream << "Histogramming\n";
          stream << string("        ", 0, tLevel);
          stream << "SlaveBits 0x" << hex << (int)task.slvBits << dec << " Port " << (int)task.port 
                 << " cfgRegs " << (int)task.configRegister[0] << " " << (int)task.configRegister[1] << endl;
          stream << string("        ", 0, tLevel);
          stream << "ConfigSet " << (int)task.configSctSet << " DataSet " << (int)task.dataSet 
                 << " GroupRangeMaps 0x" << hex << (int)task.groupRangeMap[0] 
                 << " 0x" << (int)task.groupRangeMap[1] << dec << endl;
          stream << string("        ", 0, tLevel);
          stream << " GroupDspMaps 0x" << hex << (int)task.groupDSPMap[0] << " 0x" << (int)task.groupDSPMap[1] 
                 << " 0x" << (int)task.groupDSPMap[2] << " 0x" << (int)task.groupDSPMap[3] << dec;
          stream << " Group SP Maps 0x" << hex << (int)task.groupSPMap[0] 
                 << " 0x" << (int)task.groupSPMap[1] << endl;
          stream << string("        ", 0, tLevel);
          stream << "Histogram address " << hex << task.histoBase << dec << endl;
          stream << string("        ", 0, tLevel);
          stream << "arrangement " << (int)task.arrangement << " dataFormat " << (int)task.dataFormat 
                 << " Binsize " << (int)task.binSize << " extSetup " << (int)task.extSetup 
                 << " dataPath " << (int)task.dataPath << " capture " << (int)task.capture 
                 << " useRangeList " << (int)task.useRangeList << endl;
          stream << string("        ", 0, tLevel);
          stream << "Reps " << task.repetitions << " NBins " << task.nBins << " Bin0 " << task.bin0 << endl;
          stream << string("        ", 0, tLevel);
          stream << "Data pointers " << task.dataPtr[0] << " " << task.dataPtr[1] << endl;
          for(int t=0; t<2; t++) {
            stream << "Trigger " << t << endl;
            for(int i=0; i<N_CMD_LIST_CMDS; i++) {
              stream << task.triggerSequence[t].cmd[i] << ": " << task.triggerSequence[t].data[i] << "  ";
            }
            stream << endl;
          }
          stream << string("        ", 0, tLevel);
          stream << "incCmds " << (int)task.incCmd[0] << " " << (int)task.incCmd[1] 
                 << " calLineLoop " << (int)task.calLineLoop << " DistToggle " << (int)task.distributionToggle 
                 << " incData " << (int)task.incData[0] << " " << (int)task.incData[1] 
                 << endl;
          stream << string("        ", 0, tLevel);
          stream << "Gendata: ";
          for(int i=0; i<10; i++) stream << " " << task.genData[i];
          stream << endl;
          FLOAT32 *dataPtr = (FLOAT32*)(outPtr + sizeof(START_TASK_IN)/4); // HISTOGRAM_SETUP_IN));
          stream << string("        ", 0, tLevel+2);
          for(int b=0; b<2*task.nBins; b++) {
            if(b == task.nBins) {
              stream << endl;
              stream << string("        ", 0, tLevel+2);
            }
            stream << dataPtr[b] << " ";
          }
          stream << endl;
        } else 
#else
#warning "Debug output for HISTOGRAM_CTRL_TASK not compiled!"
#endif
        if(taskId == HISTOGRAM_TASK) {
          HISTOGRAM_TASK_IN &task = *(HISTOGRAM_TASK_IN*)taskPtr;
          stream << string("        ", 0, tLevel);
          stream << "NEvents " << task.nEvents << " controlFlag = " << task.controlFlag << endl;
        } else { // Unknown task
          if(debugLevel > 4)
            printMemoryBlock(stream, outPtr+4, primLength-8, 8, level+1);
        }
      } else if(primId == RW_SLAVE_MEMORY) {
        stream << string("        ", 0, level);
        stream << (outPtr[1]?"READ":"WRITE") << " " << outPtr[4] << " words to slave " << outPtr[0] << endl;
        if(debugLevel > 5)
          printMemoryBlock(stream, outPtr+5, primLength-9, 8, level+1);
#if R_EVENT_TRAP_SETUP == 103
      } else if(primId == EVENT_TRAP_SETUP) {
        stream << string("        ", 0, level);
        stream << " Slaves " << outPtr[0] << " Events " << outPtr[1] << " Timeout: " << outPtr[2] << endl;

        stream << string("        ", 0, level);
        stream << " Ext: " << outPtr[3] << " Dist: " << outPtr[4] << " Release: " << outPtr[5] << " Back: " << outPtr[6] << endl;
        stream << string("        ", 0, level);
        stream << " Mode: " << outPtr[7] << " sLink: " << outPtr[8] << " Format: " << outPtr[9] << " Strays: " << outPtr[10] << " IterLimit: " << outPtr[11] << endl;
        stream << string("        ", 0, level);
        stream << " 0: Config: " << outPtr[12] << " Xclude: " << outPtr[14] << " Function: " << outPtr[16];
        stream << " Match: " << outPtr[18] << " Modulus: " << outPtr[20] << " Remainder: " << outPtr[22] << endl;
        stream << string("        ", 0, level);
        stream << " 1: Config: " << outPtr[13] << " Xclude: " << outPtr[15] << " Function: " << outPtr[17];
        stream << " Match: " << outPtr[19] << " Modulus: " << outPtr[21] << " Remainder: " << outPtr[23] << endl;
#else
#warning "Debug output for EVENT_TRAP_SETUP not compiled!"
#endif
#if R_HISTOGRAM_SETUP == 105
      } else if(primId == HISTOGRAM_SETUP) {
        HISTOGRAM_SETUP_IN &prim =  *(HISTOGRAM_SETUP_IN *)outPtr;
        stream << string("        ", 0, level);
        stream << " HistoBase = 0x" << hex << prim.base << dec << " nBins: " << prim.nBins << endl;

//         unsigned char *charPtr = (unsigned char *)prim.padding[3];
        stream << string("        ", 0, level);
        stream << " Padding: " << (int)prim.padding[0] << " " << (int)prim.padding[1] 
               << " DataTypes: " << (int)prim.dataType[0] << " " << (int)prim.dataType[1] << endl;
        stream << string("        ", 0, level);
        stream << " Arrangement: " << (int)prim.opt[0] << " DataFormat: " << (int)prim.opt[1] 
               << " BinSize: " << (int)prim.binSize << " (Ignored: " << (int)prim.opt[2] << ")" << endl;

        stream << string("        ", 0, level);
        stream << " Module Maps: 0x" << hex << prim.validModules[0] << " 0x" << prim.validModules[1] << dec
               << " RangeMaps: 0x" << hex << prim.moduleRangeMap[0][0] << " 0x" << prim.moduleRangeMap[0][1] 
               << " 0x" << prim.moduleRangeMap[1][0] << " 0x" << prim.moduleRangeMap[1][1] << dec << endl;
        stream << string("        ", 0, level);
        stream << " Data Pointers " << hex << prim.xPtr[0] << " " << prim.xPtr[1] << dec << "\n";
#elif R_HISTOGRAM_SETUP == 104
      } else if(primId == HISTOGRAM_SETUP) {
        HISTOGRAM_SETUP_IN &prim =  *(HISTOGRAM_SETUP_IN *)outPtr;
        stream << string("        ", 0, level);
        stream << " HistoBase = 0x" << hex << outPtr[0] << dec << " RodType " << outPtr[1] << " nBins: " << outPtr[2] << endl;

        unsigned char *charPtr = (unsigned char *)&outPtr[3];
        stream << string("        ", 0, level);
        stream << " Padding: " << (int)charPtr[0] << " " << (int)charPtr[1] << " DataTypes: " << (int)charPtr[2] << " " << (int)charPtr[3] << endl;
        stream << string("        ", 0, level);
        stream << " Arrangement: " << (int)charPtr[4] << " DataFormat: " << (int)charPtr[5] << " BinSize: " << (int)charPtr[6] << endl;

        stream << string("        ", 0, level);
        stream << " Module Maps: 0x" << hex << outPtr[5] << " 0x" << outPtr[6] << dec
               << " RangeMaps: 0x" << hex << outPtr[7] << " 0x" << outPtr[8] 
               << " 0x" << outPtr[9] << " 0x" << outPtr[10] << dec << endl;
        stream << string("        ", 0, level);
        stream << " Data Pointers " << hex << prim.xPtr[0] << " " << prim.xPtr[1] << dec << "\n";
#else
#warning "Debug output for HISTOGRAM_SETUP not compiled!"
#endif
      } else if(primId == RW_MODULE_DATA) {
        stream << string("        ", 0, level + 1) << (outPtr[0]?"Read":"Write") 
               << " Struct: " << outPtr[1] << " Module: " << outPtr[2] << endl;
        stream << string("        ", 0, level + 1) << "Skip module data\n";
      } else { // Unknown primitive
        if(debugLevel > 2)
          printMemoryBlock(stream, outPtr, primLength-4, 8, level+1);
      }
    } else { // Out list
      if(primId == ECHO) {
        stream << string("        ", 0, level);
        stream << "ECHO Primitive response:\n";
        hex(stream);
        for (unsigned int i=0; i<primLength-4; i++) {
          stream.width(8);
          stream << outPtr[i] <<" ";
          if (0 == (i+1)%8) {
            stream << endl; 
            stream << string("        ", 0, level);
          }
        }
        if (0 != (primLength-4)%8) stream << endl;
        dec(stream);
      } else if(primId == SEND_SLAVE_LIST) {
        stream << string("        ", 0, level);
        stream << "Response from slave DSP:\n";
        printOutList(outPtr, outPtr[-4]-6, in, level+1);
      } else { // Unknown primitive
        if(debugLevel > 4)
          printMemoryBlock(stream, outPtr, primLength-4, 8, level+1);
      }
    }

    outPtr += primLength-4;
  }
  unsigned long trailLength = *outPtr++;
  unsigned long trailCheck = *outPtr++;
  stream << string("        ", 0, level);
  if(in) 
    stream << "Inlist ";
  else 
    stream << "Outlist ";
  stream << "Trailer: length = " << trailLength  << " check = " << hex << trailCheck << dec << endl;
  stream << "End of response\n";
}

/* Get a message from a ROD text buffer
   Returns whether the action was completed
*/
bool SctApi::getRodMessage(unsigned int partition, unsigned int crate, unsigned int rod,
                           char *buffer, int &length) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Get message\n";
  }
  if(getCrate(partition, crate)) {
    return getCrate(partition, crate)->getRodMessage(rod, buffer, length);
  } else {
    buffer[0] = 0;
    length = 0;
    return false;
  }
}

void SctApi::getABCDModule(UINT32 bank) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "getABCDModule (all) " << bank << endl;
  }
  for(map<UINT32, ABCDModule> ::const_iterator iter = moduleMap.begin();
      iter != moduleMap.end();
      iter ++) {
    getABCDModule(iter->first, bank);
  }
}

void SctApi::getABCDModule(UINT32 mid, UINT32 bank) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "getABCDModule " << mid << " " << bank << endl;
  }

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  // Find module channel in ROD

  long *primBuff = new long[sizeof(RW_MODULE_DATA_IN)/4];
  RW_MODULE_DATA_IN &primData = *(RW_MODULE_DATA_IN*)primBuff;

  primData.readNotWrite = 1;
#if (R_RW_MODULE_DATA == 101)
  primData.structId = bank;
#elif (R_RW_MODULE_DATA == 100)
  primData.structID = bank;
#else
#error "RW_MODULE_DATA not compiled (primitive change)"
#endif
  primData.moduleNum = channel;

  // Ignore configdata as this is a read

  RodPrimList primList(1);

  //  const int RW_MODULE_DATA = 8204;
  //  const int R_RW_MODULE_DATA = 101;

  RodPrimitive readModule(4+3, 1, RW_MODULE_DATA, R_RW_MODULE_DATA, primBuff);

  primList.push_back(readModule);
  sendPrimList(partition, crate, rod, &primList);

  delete [] primBuff;

  awaitResponse(partition, crate, rod, 2);

  unsigned long length;
  unsigned long *outBody = 0;
  if(responseCode == 0) {
    outBody = getResponse(partition, crate, rod, length);
  }

  if(outBody) {
//      unsigned long outLength = outBody[0];
//      unsigned long outIndex = outBody[1];
    unsigned long outNumPrims = outBody[2];
//      unsigned long outPrimVersion = outBody[3];

    if(outNumPrims == 1) {
      unsigned long primLength = outBody[4];
//        unsigned long primIndex = outBody[5];
      unsigned long primID = outBody[6];
//        unsigned long primVersion = outBody[7];

      if(primID == RW_MODULE_DATA) {
        if(primLength != (sizeof(ABCDModule)/sizeof(UINT32) + 4)) { // Prim length includes header
          cout << "Output primitive was " << primLength << " not " << sizeof(ABCDModule)/sizeof(UINT32) << endl;
        }

        // Data is at outBody[8]
        ABCDModule result;

        // Copy data
        for(unsigned int i=0; i<sizeof(ABCDModule)/sizeof(UINT32); i++) {
          ((unsigned long *)&result)[i] = outBody[8 + i];
        }

        ABCDModule *previous = lookupConfig(mid);

        if(previous) {
          cout << "Replacing previous module config\n";

          *previous = result;
        } else {
          cout << "New module config inserted\n";
          moduleMap.insert(make_pair(mid, result));
        }
      } else {
        cout << "Returned primitive was not RW_MODULE_DATA\n";
      }
    } else {
      cout << "Too many primitives in output list\n";
    }

    delete [] outBody;
  } else {
    cout << "No response to primitive list!\n";
  }
}

/* Store config in ROD */
void SctApi::setABCDModule(UINT32 mid, UINT32 bank) {
  if(mid==0xffffffff){
    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "setABCDModule called for all modules" << endl;
    }
    for(map<UINT32, ABCDModule>::const_iterator iter = moduleMap.begin();
        iter != moduleMap.end();
        iter ++) {
      cout << "Set Config " << iter->first << endl;
      setABCDModule(iter->first, bank);
    }
    return;
  }

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "setABCDModule " << mid << " " << bank << endl;
  }

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  std::string moduleName;
  ABCDModule *moduleConfig;

  moduleConfig = lookupConfig(mid);

  if(!moduleConfig) {
    cout << "NO module configuration for mid = " << mid << endl;
    return;
  }

  cout << "\nModule config\n";
  cout << " primary channel " << (int)moduleConfig->pTTC 
       << " red channel " << (int)moduleConfig->rTTC 
       << " group " << (int)moduleConfig->groupId
       << " select " << (int)moduleConfig->select << endl;
  cout << " target (chip 0) " << (int)moduleConfig->chip[0].target << endl;

  const unsigned int longsInConfig = sizeof(ABCDModule) / 4;

  long *configData = new long[4+longsInConfig];

  RW_MODULE_DATA_IN &params = *(RW_MODULE_DATA_IN*)configData;

#if (R_RW_MODULE_DATA == 101)
  params.readNotWrite = 0;
  params.structId = bank;
  params.moduleNum = channel;
  params.configData = (ABCDModule *)0xffffffff;
#else
#error "RW_MODULE_DATA not compiled (primitive change)"
#endif

  for(unsigned int i=0; i<longsInConfig; i++) {
    // Overwrite params.configData with the first word of the config
    configData[i+3] = ((long *)moduleConfig)[i];
  }

  RodPrimList primList(1);

  RodPrimitive writeConfig(4+3+longsInConfig, 1, RW_MODULE_DATA, R_RW_MODULE_DATA, configData);
  primList.push_back(writeConfig);

  sendPrimList(partition, rod, crate, &primList);

  delete [] configData;

  awaitResponse(partition, rod, crate, 10);

  if(responseCode!=0) {
    cout << "Set module unsuccessful!\n";
  }
}

void SctApi::modifyABCDMask(UINT32 mid, UINT32* mask) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "modifyABCDMask on " << mid << "\n";
  }

  ABCDModule *module = lookupConfig(mid);

  if(module) {
    for(int c=0; c<12; c++) {
      for(int m=0; m<4; m++) {
        module->chip[c].basic.mask[m] = mask[c*4+m];
      }
    }
  } 
}

void SctApi::modifyABCDRC(UINT32 mid, UINT32 chip, UINT16 index, FLOAT32 p0, FLOAT32 p1, FLOAT32 p2){
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "modifyABCDRC on " << mid << " " << chip << "\n";
  }
  ABCDModule *module = lookupConfig(mid);

  if(module) {
    module->chip[chip].caldata.rc_function = index;
    module->chip[chip].caldata.rc_params[0]=p0;
    module->chip[chip].caldata.rc_params[1]=p1;
    module->chip[chip].caldata.rc_params[2]=p2;
  }
}

void SctApi::modifyABCDTrims(UINT32 mid, UINT8* trims) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "modifyABCDTrims on " << mid << "\n";
  }

  ABCDModule *module = lookupConfig(mid);

  if(module) {
    for(int c=0; c<12; c++) {
      for(int t=0; t<128; t++) {
        module->chip[c].trim[t] = trims[c*128+t];
      }
    }
  }
}

void SctApi::modifyABCDVar(UINT32 typ, FLOAT32 var) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "modifyABCDVar (all)\n";
  }

  cout << "Modify var on all modules: setting " << typ << " to " << var << endl;

  for(map<UINT32, ABCDModule>::const_iterator iter = moduleMap.begin();
      iter != moduleMap.end();
      iter ++) {
    modifyABCDVar(iter->first, typ, var);
  }
}

#define fToInt(val) ((int)((val)+0.5))

void SctApi::modifyABCDVar(UINT32 mid, UINT32 typ, FLOAT32 var) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "modifyABCDVar (module " << mid << ") " << typ << " " << var << "\n";
  }

  switch(typ) {
    // First some module parameters (BOC)
  case ST_RX_DELAY:
  case ST_RX_DELAY0:
  case ST_RX_DELAY1:
  case ST_RX_THRESHOLD:
  case ST_RX_THRESHOLD0:
  case ST_RX_THRESHOLD1:
  case ST_TX_CURRENT:
  case ST_TX_MARKSPACE:
  case ST_TX_DELAY:
  case ST_TX_COARSE:
  case ST_TX_FINE:

#ifdef ST_BOC_BPM_PH
  case ST_BOC_BPM_PH:
  case ST_BOC_BREG_PH:
  case ST_BOC_V0_PH:
  case ST_BOC_V1_PH:
  case ST_BOC_VRN_FINE:
#else
#warning  "No BOC wide parameter modification"
#endif
    {
      unsigned int partition, crate, rod, channel;
      getpcrc(mid, partition, crate, rod, channel);

      cout << "Calling BOCCard method for module " << mid << " " << typ << " " << var << endl;

      getCrate(partition, crate)->modifyBOCParam(rod, channel, typ, fToInt(var));
      break;
    }

    // The rest are chip parameters
  default:
    {
      for(int c=0; c<12; c++) {
        modifyABCDVar(mid, c, typ, var);
      }
    }
    break;
  }
}

#warning "Should use Doug's primitive instead and make this modifyABCDVarLocal"
void SctApi::modifyABCDVar(UINT32 mid, UINT32 c, UINT32 typ, FLOAT32 val) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "modifyABCDVar (module " << mid << ": chip " << c  << ") " << typ << " " << val << "\n";
  }

  ABCDModule *mConf = lookupConfig(mid);

  if(!mConf) {
    cout << "No module configuration to change\n";
    return;
  }

  ABCDModule &module = *mConf;

  ABCDChip &chip = module.chip[c];

  switch(typ) {
  case ST_VTHR: 
    chip.basic.vthr = (char) fToInt(val/2.5);
    break;
  case ST_VCAL:
    chip.basic.vcal = (char) fToInt(val/0.625);
    break;
  case ST_STROBE_DELAY:
    chip.basic.delay = (char) fToInt(val);
    break;
  case ST_PREAMP:
    chip.basic.preamp = (char) fToInt(val/9.2);
    break;
  case ST_SHAPER:
    chip.basic.shaper = (char) fToInt(val/1.2);
    break;
  case ST_TRIM:
    for(int i=0; i<128; i++) {
      chip.trim[i] = (char) fToInt(val);
    }
    break;
  case ST_MASK:
    {
      UINT32 myval = (UINT32) fToInt(val);
      UINT32 mask = myval;
      for(int i=0; i<16; i++) {
        mask = (mask << 2) + myval;
      }

      for(int i=0; i<4; i++) {
        chip.basic.mask[i] = mask;
      }
    }
    break;
  case ST_ROLE:
    switch(fToInt(val)) {
    case END:
      chip.basic.config.master = 1;  // active low
      chip.basic.config.end = 1;     // active high
      chip.basic.config.feedThrough = 1; // active low
      break;
    case MASTER:
      chip.basic.config.master = 0;  // active low
      chip.basic.config.end = 0;     // active high
      chip.basic.config.feedThrough = 1;
      break;
    case SLAVE:
      chip.basic.config.master = 1;  // active low
      chip.basic.config.end = 0;     // active high
      chip.basic.config.feedThrough = 1;
      break;
    case LONELY:
      chip.basic.config.master = 0;  // active low
      chip.basic.config.end = 1;     // active high
      chip.basic.config.feedThrough = 1;
      break;
    case PARANOID:
      chip.basic.config.master = 0;  // active low
      chip.basic.config.end = 1;     // active high
      chip.basic.config.feedThrough = 1;
    case DEAD:
    default:
      module.chip[(c+11)%12].basic.config.outputBypass = 1;
      module.chip[(c+1)%12].basic.config.inputBypass = 1;
    case MISSING:
      break;
    } // End switch role

  case ST_NMASK:
    {
      int channels = fToInt(val);
      if(channels > 128) channels = 128;

      for(int i=0; i<4; i++) {
        // How many channels in this group
        int chan = channels - i*32;
        if(chan >= 32) {
          chip.basic.mask[i] = 0xffffffff;
        } else {
          int value = 0;
          for(int b=0; b<chan; b++) {
            value ++;
            value <<= 1;
          }
          chip.basic.mask[i] = value;
        }
      }
    }
    break;
  case ST_CAL_MODE:
    chip.basic.config.calibMode = fToInt(val);
    break;
  case ST_COMPRESSION:
    chip.basic.config.readoutMode = fToInt(val);
    break;
  case ST_TRIM_RANGE:
    chip.basic.config.trimRange = fToInt(val);
    break;
  case ST_EDGE_DETECT:
    chip.basic.config.edgeDetect = fToInt(val);
    break;
  case ST_SEND_MASK:
    chip.basic.config.mask = fToInt(val)?1:0;
    break;
  case ST_ACCUMULATE:
    chip.basic.config.accumulate = fToInt(val);
    break;
  case ST_ACTIVE:
    chip.active = fToInt(val);
    break;

//    case ST_BYPASS:
//      {
//        /*
//         * A method used to select certain predetermined bypass configurations.
//         * Made obsolete by ST_TOKEN (below) but included here for reasons of
//         * backward compatibility. Recoded to use setToken function.
//         */
//        UINT8 int_val = (UINT8)((val+0.5)/1.0);
//        switch(int_val){
//        case 0: /* ME---- ME---- */
//          setToken(module, 0x03 +(0x03<<6),1,1);
//          break;
//        case 1: /* MSE--- MSE--- */
//          setToken(module, 0x07 +(0x07<<6),1,1);
//          break;
//        case 2: /* MSSE-- MSSE-- */
//          setToken(module, 0x0F +(0x0F<<6),1,1);
//          break;
//        case 3: /* MSSSE- MSSSE- */
//          setToken(module, 0x1F +(0x1F<<6),1,1);
//          break;
//          /* case 4 is the default configuration */
//        case 5: /* MDE--- MDE--- */
//          setToken(module, 0x05 +(0x05<<6),1,1);
//          break;
//        case 6: /* MSDE-- MSDE-- */
//          setToken(module, 0x0b +(0x0b<<6),1,1);
//          break;
//        case 7: /* MSSDE- MSSDE- */
//          setToken(module, 0x17 +(0x17<<6),1,1);
//          break;
//        case 8: /* MSSSDE MSSSDE */
//          setToken(module, 0x2f +(0x2f<<6),1,1);
//          break;
//        case 9: /* MSSSSS DSSSSE */
//          setToken(module, 0x3f +(0x3e<<6),2,0);
//          break;
//        case 10: /* DSSSSE MSSSSS */
//          setToken(module, 0x3e +(0x3F<<6),0,2);
//          break;
//        case 11: /* MSSSSS SSSSSE */
//          setToken(module, 0x3f +(0x3f<<6),2,0);
//          break;
//        case 12: /* SSSSSE MSSSSS */
//          setToken(module, 0x3f +(0x3F<<6),0,2);
//          break;
//        default: /* MSSSSE MSSSSE */
//          setToken(module, 0x3F +(0x3F<<6),1,1);
//          break;
//        }
//        break;
//      }

//    case ST_TOKEN:
//      {
//        /*
//         * This variable iterates over a set of token/data passing
//         * schemes to determine the functionality of each token link.
//         * The sequence sets all the combinations to read-out
//         * n chips by first testing the bypass token (chip n-2 to n) 
//         * for all combinations of (n-2) chips. Secondly trying the 
//         * direct token (n-1 to n) for all combinations of (n-1) chips.
//         *
//         * For configurations 0-19, both datalinks are exercised at once.
//         * For the remaining configurations, only one datalink is
//         * used at once, as outlined below:
//         *
//         * o Even Configurations 20-34 read out chip S9 via datalink 0
//         *   using the bypass link between E5 and S9.
//         * 
//         * o Odd  Configurations 21-35 read out chip S1 via datalink 1
//         *   using the bypass link between E13 and S1.
//         *
//         * o Even configurations 36-60 read out chip M8 via datalink 0
//         *   using first the bypass link between S4 and M8 and then
//         *   the direct link between E5 and M8.
//         *
//         * o Odd  configurations 37-61 read out chip M0 via datalink 1
//         *   using first the bypass link between S12 and M0 and then
//         *   the direct link between E13 and M0.
//         *
//         * Only cases 0-35 have been implememted in the design of the
//         * barrel hybrid. The logical inconsistency in the order between 
//         * cases 20-35 and 36-61 is done such that the subset of the 
//         * schemes used by the barrel design form a consecutive series.
//         *
//         * Orignal code by Lars Eklund, March 2001
//         */

//         if(val<20) {              /* use both datalinks */
//           setToken(module, (tokens[(int)val] + (tokens[(int)val] << 6)),1,1);

//         } else if(val<36) {       /* use datalink 0 */
//           if(!((int)val%2)) {
//             setToken(module, (tokens[(int)(val-20)/2+12] + (0x1 << 7)),2,0);
//           } else {                /* use datalink 1 */
//             setToken(module, (0x2 + (tokens[((int)val-21)/2+12] << 6)),0,2);
//           }
//         } else { // only possible for forward modules
//          if(!((int)val%2)) {
//            setToken(module, (tokens[(int)((val-36)/2+7)] + (1 << 6)),2,0);
//          } else {
//            setToken(module, (0x1 + (tokens[(int)((val-37)/2+7)] << 6)),0,2);
//          }
//        }
//        break;
//    }

  case ST_QTHR: 
    {
      ABCDCaldata &calPtr = chip.caldata;
      float v, vv;

      switch(calPtr.rc_function){
      case 1: /* polynomial */
        v = calPtr.rc_params[0] + 
          val * (calPtr.rc_params[1] + 
                 (val * calPtr.rc_params[2]) );
        break; 
      case 2: /* grillo */
        if((val==0)|| (calPtr.rc_params[1]==0))
          vv = 0;
        else
          vv = calPtr.rc_params[2] / 
            (calPtr.rc_params[1] *val);
        v  = calPtr.rc_params[0] + 
          calPtr.rc_params[2] / (float) sqrt(1+(vv*vv)); 
        break; 
      case 3: /* exp */
        if(calPtr.rc_params[1]==0)
          v = 0;
        else
          v = calPtr.rc_params[2] + 
            calPtr.rc_params[0]/(1+ (float)exp(-val/calPtr.rc_params[1]));
        break;  
      case 4: /* linear fit */
        v = calPtr.rc_params[0] + 
          (val * calPtr.rc_params[1]);
        break; 
      default:
        v = 0;
        break;
      }

      UINT8 int_val = ((UINT8)((v+1.25)/(float)2.5));  /* 2.5mV / DAC bit */
      chip.basic.vthr = int_val;
      break;
    }

  case ST_QCAL:
    /*
     * calculate the actual vcal step size taking into
     * account the capacitor correction factor:
     * step size = (c_factor * 0.0625) fC / DAC bit 
     */
    {
      FLOAT32 step = (float) 0.0625 * chip.caldata.c_factor;
      UINT8 int_val = (UINT8)((val+(step/2))/step);
      chip.basic.vcal = int_val;
      break;
    }
  case ST_TARGET:
    {
      /*
       * record the target to which the chip has been trimmed
       * added PWP 07.01.03
       */
      UINT8 int_val = ((UINT8)((val+1.25)/(float)2.5));  /* 2.5mV / DAC bit */

      /* int_val must be in range if declared UINT8
         if(int_val<0x00) int_val = 0x00;
         if(int_val>0xff) int_val = 0xff;
      */
      chip.target = int_val;
      break;
    }

  case ST_TTHR:
    {
      /*
       * set the threshold in mV wrt the trim target
       * added PWP 07.01.03
       */
      UINT8 int_val = ((UINT8)((val+1.25)/(float)2.5));  /* 2.5mV / DAC bit */
      int_val += chip.target;

      /*
        if(int_val<0x00) int_val = 0x00;
        if(int_val>0xff) int_val = 0xff;
      */
      chip.basic.vthr = int_val;
      break;
    }

  case ST_RX_DELAY:
  case ST_RX_DELAY0:
  case ST_RX_DELAY1:
  case ST_RX_THRESHOLD:
  case ST_RX_THRESHOLD0:
  case ST_RX_THRESHOLD1:
  case ST_TX_CURRENT:
  case ST_TX_MARKSPACE:
  case ST_TX_DELAY:
  case ST_TX_COARSE:
  case ST_TX_FINE:
    {
      unsigned int partition, crate, rod, channel;
      getpcrc(mid, partition, crate, rod, channel);

      cout << "Calling BOCCard method for channel " << channel << " " << typ << " " << val << endl;

      getCrate(partition, crate)->modifyBOCParam(rod, channel, typ, fToInt(val));
      break;
    }

  default:
    cout << "Unknown variable type " << typ << "\n";
    break;
  } // End switch var
}

// Get ID number of module
UINT32 SctApi::findModule(string sn) { 
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "findModule " << sn << endl;
  }

  unsigned int partition, crate, rod, channel;

  try {
    unsigned int mur, module;
    config->translateFromSN(sn, mur, module);
    config->translateToROD(mur, module, partition, crate, rod, channel);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";
    return BAD_MODULE;
  }

  return packpcrc(partition, crate, rod, channel);
}

string SctApi::convertToString(UINT32 mid) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "convertToString " << mid << endl;
  }

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  string sn;

  try {
    unsigned int mur, module;
    config->translateFromROD(partition, crate, rod, channel, mur, module);
    config->translateToSN(mur, module, sn);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";
    return "BAD_MODULE";
  }

  return sn;
}

UINT32 SctApi::findModule(INT32 mur, INT32 module) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "findModule " << mur << " " << module << endl;
  }

  unsigned int partition, crate, rod, channel;

  try {
    config->translateToROD(mur, module,
                           partition, crate, rod, channel);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";
    return BAD_MODULE;
  }

  return packpcrc(partition, crate, rod, channel);
}

pair<INT32, INT32> SctApi::convertToMUR(UINT32 mid) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "convertToMUR " << mid << endl;
  }

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  UINT32 mur;
  UINT32 module;

  try {
    config->translateFromROD(partition, crate, rod, channel, mur, module);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";
    return make_pair(BAD_MODULE, 0);
  }

  return make_pair(mur, module);
}

UINT32 SctApi::findBarrelModule(INT32 barrel, INT32 row, INT32 number) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "findBarrelModule " << barrel << " " << row << " " << number << endl;
  }

  unsigned int partition, crate, rod, channel;

  try {
    unsigned int mur, module;
    config->translateFromBarrel(barrel, row, number, mur, module);
    config->translateToROD(mur, module,
                           partition, crate, rod, channel);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";
    return BAD_MODULE;
  }

  return packpcrc(partition, crate, rod, channel); 
}

void SctApi::convertToBarrelModule(UINT32 mid, UINT32 &barrel, UINT32 &row, int &number) { 
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "convertToBarrelModule " << mid << endl;
  }

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  try {
    unsigned int mur, module;
    config->translateFromROD(partition, crate, rod, channel, mur, module);
    config->translateToBarrel(mur, module, barrel, row, number);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";

    barrel = BAD_MODULE;
    return;
  }
}

UINT32 SctApi::findEndcapModule(INT32 disk, INT32 ring, INT32 number) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "findEndcapModule " << disk << " " << ring << " " << number << endl;
  }
  unsigned int partition, crate, rod, channel;

  try {
    unsigned int mur, module;
    config->translateFromEndcap(disk, ring, number, mur, module);
    config->translateToROD(mur, module,
                           partition, crate, rod, channel);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";
    return BAD_MODULE;
  }

  return packpcrc(partition, crate, rod, channel);
}

void SctApi::convertToEndcapModule(UINT32 mid, UINT32 &disk, UINT32 &ring, UINT32 &number) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "convertToEndcapModule " << mid << endl;
  }

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  try {
    unsigned int mur, module;
    config->translateFromROD(partition, crate, rod, channel, mur, module);
    config->translateToEndcap(mur, module, disk, ring, number);
  } catch(ConfigurationException &c) {
    cout << "Module not found\n";
    disk = BAD_MODULE;
    return;
  }
}

void SctApi::sendAllABCDModules(UINT32 bank, UINT32 type) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "sendABCDModule called for all modules" << endl;
  }

#if R_SEND_CONFIG == 104
#warning "Send modules in all RODs"
  unsigned int partition, crate, rod;
  partition = crate = rod = 0;

  if(!getCrate(partition, crate)->checkBOCLasersOn(rod)) {
    cout << "Trying to send module configuration using BOC that has its lasers cut out\n";
    if(mrs) {
      *mrs << "BOC_INTERLOCKED" << MRS_ERROR << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<int>("crate", crate) 
           << MRS_PARAM<int>("rod", rod)
           << MRS_TEXT("Can't send all module config (BOC interlocked)") << ENDM;
    }

    return;
  }

  RodPrimList primList(1);

  // Write "Disable trigger decoder" to primList
  writeRODRegister(0x1c7, 18, 1, 0, primList);

#warning "Type (ie trim, basic, mask) ignored"

  long *primBuff = new long[sizeof(SEND_CONFIG_IN) /4];

  SEND_CONFIG_IN &primData = *(SEND_CONFIG_IN*)primBuff;

  primData.port = SP_BOTH;
  primData.captureSerOn = 0;
  primData.moduleNum[0] = ALL_MODULES;
  primData.moduleNum[1] = ALL_MODULES;
  primData.chipNum = ALL_CHIPS;
  primData.setLinks = 1;               // Set links 
  primData.restore = 1;                // Restore links 
  primData.structId = bank;
  primData.groupId = MODULE_GROUP_ALL;
  primData.dataType = CONFIG_MODULE_ALL;
  primData.activeOnly = 0;
  primData.enableDataTaking = 1;

  primList.push_back(RodPrimitive(4+sizeof(primData)/4, 1, SEND_CONFIG, R_SEND_CONFIG, primBuff));

  // Write "Re-enable trigger decoder" to primList
  writeRODRegister(0x1c7, 18, 1, 1, primList);

  sendPrimList(partition, crate, rod, &primList);

  delete [] primBuff;

  awaitResponse(partition, crate, rod, 2);

  if(responseCode != 0) {
    cout << "Send modules unsuccessful\n";
  }
#else
#warning "Slow module sending"
  for(map<UINT32, ABCDModule>::const_iterator iter = moduleMap.begin();
      iter != moduleMap.end();
      iter ++) {
    cout << "SendConfig " << iter->first << endl;
    sendABCDModule(iter->first, bank, type);
  }
#endif

#warning "Reset only ROD 0..."
  // Reset everything so its not confused by the clk/2
  cout << "Resetting front end interface\n";
  writeRODRegister(0, 0, 0, 0x1c6, 0, 8, 0xff);    // Reset everything
  writeRODRegister(0, 0, 0, 0x1c6, 0, 8, 0x78);
  return;
}

void SctApi::sendABCDModule(UINT32 mid, UINT32 bank, UINT32 type) {
  if(mid==0xffffffff){
    sendAllABCDModules(bank, type);
    return;
  }
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "sendABCDModule " << mid << " " << bank << " " << type << endl;
  }

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  if(!getCrate(partition, crate)->checkBOCLasersOn(rod)) {
    cout << "Trying to send module configuration using BOC that has its lasers cut out\n";
    if(mrs) {
      *mrs << "BOC_INTERLOCKED" << MRS_ERROR << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<int>("module", mid) << MRS_PARAM<int>("crate", crate) 
           << MRS_PARAM<int>("rod", rod)
           << MRS_TEXT("Can't send module config (BOC interlocked)") << ENDM;
    }

    return;
  }

  RodPrimList primList(1);

  // Write "Disable trigger decoder" to primList
  writeRODRegister(0x1c7, 18, 1, 0, primList);

#warning "Type (ie trim, basic, mask) ignored"

  long *primBuff = new long[sizeof(SEND_CONFIG_IN) /4];

  SEND_CONFIG_IN &primData = *(SEND_CONFIG_IN*)primBuff;

#if R_SEND_CONFIG == 104
  primData.port = SP0;                   // Only one module at a time, so use SP0
  primData.captureSerOn = 0;
  primData.moduleNum[0] = channel;
  primData.moduleNum[1] = NO_MODULE;
  primData.chipNum = ALL_CHIPS;
  primData.setLinks = 1;
  primData.restore = 1;                  // Restore links
  primData.structId = bank;
  primData.groupId = MODULE_GROUP_ALL;
  primData.dataType = CONFIG_MODULE_ALL;
  primData.activeOnly = 0;
  primData.enableDataTaking = 1;
#elif R_SEND_CONFIG == 102
  primData.cmdBuff = CMD_BUFFER_0;            // Only one module
  primData.captureSerOn = 0;
  primData.moduleNum[0] = channel;
  primData.moduleNum[1] = NO_MODULE;                // No module
  primData.loop = 0;
  primData.setLinks = 1;
  primData.restore = 0;
  primData.structId = bank;
  primData.groupId = MODULE_GROUP_ALL;
  primData.dataType = CONFIG_MODULE_ALL;
  primData.activeOnly = 0;
  primData.enableDataTaking = 1;
#elif R_SEND_CONFIG == 101
  primData.cmdBuff = CMD_BUFFER_0;  // ???
  primData.captureSerOn = 0;    // Capture to where?
  primData.moduleNum[0] = channel;
  primData.moduleNum[1] = 0;
  primData.structId = bank;
  primData.dataType = CONFIG_MODULE_ALL;
#else
  // R_SEND_CONFIG
#error "Send config not compiled! (new prim version)" 
#endif

  primList.push_back(RodPrimitive(4+sizeof(primData)/4, 1, SEND_CONFIG, R_SEND_CONFIG, primBuff));

  // Write "Re-enable trigger decoder" to primList
  writeRODRegister(0x1c7, 18, 1, 1, primList);

  sendPrimList(partition, crate, rod, &primList);

  delete [] primBuff;

  awaitResponse(partition, crate, rod, 2);

  if(responseCode != 0) {
    cout << "Send module unsuccessful\n";
  }
}

void SctApi::loadConfiguration() {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "loadConfiguration (default)\n";
  }

  cout << "Load configuration\n";
#warning "Using feature of ConfigurationXmlImpl"
  config->loadConfiguration("");

  cout << "Remove module configuration cache\n";
  moduleMap.clear();
}

void SctApi::loadConfiguration(string filename) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "loadConfiguration (" << filename << ")\n";
  }

  cout << "Load configuration\n";
  config->loadConfiguration(filename); 

  cout << "Remove module configuration cache\n";
  moduleMap.clear();
}

void SctApi::storeModuleConfigurations() {
  for(std::map<UINT32, ABCDModule>::const_iterator i = moduleMap.begin(); 
      i != moduleMap.end(); 
      i++) {
    const ABCDModule &moduleConf = i->second;
    std::string sn = convertToString(i->first);
    config->configureModuleFromStructure(sn, moduleConf);
  }
}

void SctApi::configureBOC(unsigned int partition, unsigned int crate, unsigned int rod) {
  getCrate(partition, crate)->configureBOC(rod);
}

void SctApi::status() {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "print all status\n";
  }

  cout << "SCTAPI status:\n";
  cout << "Run " << runNumber << " Scan " << scanNumber << endl;

#if USE_THREADS
  cout << "Threaded version\n";
#else
  cout << "Unthreaded version (dangerous in IPC)\n";
#endif
  if(mrs) {
    *mrs << "SCTAPI_STATUS" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
         << MRS_PARAM<int>("run", runNumber) << MRS_PARAM<int>("scan", scanNumber) 
         << MRS_TEXT("Run numbers") << ENDM;
  }

  for(map<pair<unsigned int, unsigned int>, Crate* >::const_iterator crate = crateMap.begin();
      crate != crateMap.end();
      crate++) {

    cout << "Crate " << crate->first.first << " " << 
      crate->first.second << endl;
    crate->second->status();
  }

  cout << "Module configuration for:\n";
  for(map<UINT32, ABCDModule>::const_iterator mi = moduleMap.begin();
      mi!=moduleMap.end();
      mi ++) {
    UINT32 mid = mi->first;

    string sn = convertToString(mid);
    int group = lookupConfig(mid)->groupId;

    cout.width(8);
    cout << mid << " " << sn << " Group: " << group << endl;

    if(mrs) {
      *mrs << "MODULE_INFO" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<const char *>("sn",sn.c_str()) << MRS_PARAM<int>("group",group) 
           << MRS_TEXT("Module status ")
           << ENDM;
    }
  }

  if(mrs) {
    *mrs << "SCTAPI_STATUS" << MRS_INFORMATION << MRS_QUALIF("SCTAPI") 
         << MRS_PARAM<int>("run", runNumber) << MRS_PARAM<int>("scan", scanNumber) 
         << MRS_TEXT("Run numbers") << ENDM;
  }
}

void SctApi::writeSlaveBlock(unsigned int partition, unsigned int crate, unsigned int rod, int s, long sAdd, int words, long *block) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Write slave block\n";
  }

  long *buffer = new long[5+words];

#if (R_RW_SLAVE_MEMORY == 100)
  buffer[0] = s;
  buffer[1] = 0;             // Write
  buffer[2] = sAdd;
  buffer[3] = 0xffffffff;
  buffer[4] = words;
#else 
#error "RW_SLAVE_MEMORY revision changed"
#endif

  for(int i=0; i<words; i++) {
    buffer[i+5] = block[i];
  }

  RodPrimList primList(1);

  primList.push_back(RodPrimitive(4 + 5 + words, 1, RW_SLAVE_MEMORY, R_RW_SLAVE_MEMORY, buffer));
  sendPrimList(partition, crate, rod, &primList);

  delete [] buffer;
  
  awaitResponse(partition, crate, rod, 2);

  if(responseCode == 0) {
    cout << "Slave data sent\n";
  } else {
    cout << "Slave data failed during send\n";
  }
}

void SctApi::setSlaveBlock(unsigned int partition, unsigned int crate, unsigned int rod, int s, long sAdd, int words, long value) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "SetSlaveBlock to " << value << endl;
  }

  long *buffer = new long[5+words];

#if (R_RW_SLAVE_MEMORY == 100)
  buffer[0] = s;
  buffer[1] = 0;             // Write
  buffer[2] = sAdd;
  buffer[3] = 0xffffffff;
  buffer[4] = words;
#else 
#error "RW_SLAVE_MEMORY revision changed"
#endif

  for(int i=0; i<words; i++) {
    buffer[i+5] = value;
  }

  RodPrimList primList(1);

  primList.push_back(RodPrimitive(4 + 5 + words, 1, RW_SLAVE_MEMORY, R_RW_SLAVE_MEMORY, buffer));
  sendPrimList(partition, crate, rod, &primList);

  delete [] buffer;

  awaitResponse(partition, crate, rod, 2);
  if(responseCode == 0) {
    cout << "Slave " << s << " block set to " << value << "\n";
  } else {
    cout << "Slave set failed\n";
  }
}

// ****  Histogramming  ****
void SctApi::defaultScan(int type) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Send default scan number " << type << endl;
  }
  ScanDefImpl scan;
  ScanEx extra;

  long *evSetup;
  long *histSetup;
  long *histStart;

  RodPrimList primList(1);

  int histStartLength;

  switch(type) {
  default:
  case 0:
    {
      // Event Trap Setup
      long setup[24] = {0x1,   0, 0x2000, 0, 
                        0x3, 0x1,      0, 0,
                          0,   0,      0, 0,
                        0x3,   0,      0, 1,               // Config, exclusion
                        0x1,   0,      0, 1,               // function, match
                          1,   0,      0, 0};              // modulus, remainder

      evSetup = new long[24];
      for(int i=0; i<24; i++) evSetup[i] = setup[i];

      // Setup histogram
      long setupH[] = {
        0x20d2000, 0, 0x60, 0x01010000, 0x20200101, 1, 0, 1, 
        0,         0,    0, 0x03f00000, 0x30f01000};

      histSetup = new long[13];
      for(int i=0; i<13; i++) histSetup[i] = setupH[i];

      // Start histogram
      long startHist[4 + 63] = {
        16, 107, 1, 1,

        // Task params
        // 
        0x01010001, 0x000f0001, 0x08040201,        0xf, 0x020d2000, 0x00200101, 0x0100000f,       1000, 
                50,        0x0, 0x40a00000, 0x40a00000, 0x41200000, 0x40a00000, 0x42c80000, 0x40200000, 
        0x43160000, 0x40a00000, 0x43480000, 0x40a00000, 0x40a00000, 0x40a00000, 0x41200000, 0x40a00000,

        0x42c80000, 0x40200000, 0x43160000, 0x40a00000, 0x43480000, 0x40a00000,        0x0,        0x0,
        0x0070006a, 0x00640065, 0x00640064, 0x00000000, 0x81,              0x0,        0x0,        0x0,
        0x0,      // End of trigger sequence (1)
        0x00640064, 0x00640064, 0x00640064, 0, 0, 0, 0, 
        0, 0,      // End of trigger sequence (2)
        0,         // incCmd (2 bytes) etc 
        0, 0,      // incData
        0, 0, 0,
        0, 0, 0, 0, 0,      // End of control data (rest is up to max task structure size)
      };

      histStart = new long[4+63];
      for(int i=0; i<4+63; i++) histStart[i] = startHist[i];

      histStartLength = 67;
    }
    break;
  case 1:
    // A short nmask scan on module 0
    {
      // Event Trap Setup
      long setup[24] = {
        0x00000001, 0x00000000, 0x00002000, 0x00000000, 0x00000003, 0x00000000, 0x00000001, 0x00000000,
        0x00000000, 0x00000000, 0x00000001, 0x00000002, 0x00000003, 0x00000000, 0x00000000, 0x00000001, 
        0x00000001, 0x00000000, 0x00000000, 0x00000000, 0x00000001, 0x00000000, 0x00000000, 0x00000000};

      evSetup = new long[24];
      for(int i=0; i<24; i++) evSetup[i] = setup[i];

      // Setup histogram
      long setupH[] = {
        0x020d2000, 0x00000000, 0x00000008, 0x18090000, 0x00200101, 0x00000001, 0x00000000, 0x00000001, 
        0x00000000, 0x00000000, 0x00000000, 0x03f00000, 0x03f01000};

      histSetup = new long[13];
      for(int i=0; i<13; i++) histSetup[i] = setupH[i];

      // Start histogram
      long startHist[79] = {
        0x18090001, 0x000f0001, 
        0x000000ff, 0x0000f00f, 0x020d2000, 0x00200101, 0x0000000f, 0x0000040a, 0x00000008, 0x00000000, 
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xffffffff, 0xffffffff, 0x0070006a, 0x00640065, 
        0x00640064, 0x00000000, 0x00000082, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00640064, 
        0x00640064, 0x00640064, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
        0x00000000, 0x00000000, 0x00000000, 0x0000000f, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 
        0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x41800000, 0x42000000, 
        0x42400000, 0x42800000, 0x42a00000, 0x42c00000, 0x42e00000, 0x00000000, 0x41800000, 0x42000000, 
        0x42400000, 0x42800000, 0x42a00000, 0x42c00000, 0x42e00000 
      };

      histStart = new long[79];
      for(int i=0; i<79; i++) histStart[i] = startHist[i];

      histStartLength = 79;


      /* 000000ff 00000001 00000006 0000006f 0000001c 00000000 00000003 00000067 

         00000001 00000000 00002000 00000000 00000003 00000000 00000001 00000000 
         00000000 00000000 00000001 00000002 00000003 00000000 00000000 00000001 
         00000001 00000000 00000000 00000000 00000001 00000000 00000000 00000000 

         00000012 00000001 00002007 00000067 00000000 0000000a ffffffff ffffffff 
         0000000a 00000001 00000001 0000006f 00000004 00000000 00001000 00000065 
         0000000a 0000100e 00000007 00000002 00002008 00000067 00000000 00000001 
         00000001 00000066 00000001 00002007 00000067 00000000 0000005e ffffffff 
         ffffffff 0000005e 00000004 00000002 0000006f 00000011 00000002 00001002 
         00000068 020d2000 00000000 00000008 18090000 00200101 00000001 00000000 
         00000001 00000000 00000000 00000000 03f00000 03f01000 

         00000047 00000003 
         0000000c 00000065 00000020 00000064 00000001 00000001 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 0000005e 1a242170 00000007 
         00000002 00002008 00000067 00000000 00000001 00000001 00000057 00000000 
         0000000c 00000065 00000010 0000006b 00000001 00000001 18090001 000f0001 
         000000ff 0000f00f 020d2000 00200101 0000000f 0000040a 00000008 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 ffffffff ffffffff 0070006a 00640065 
         00640064 00000000 00000082 00000000 00000000 00000000 00000000 00640064 
         00640064 00640064 00000000 00000000 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 0000000f 00000000 00000000 00000000 00000000 
         00000000 00000000 00000000 00000000 00000000 00000000 41800000 42000000 
         42400000 42800000 42a00000 42c00000 42e00000 00000000 41800000 42000000 
         42400000 42800000 42a00000 42c00000 42e00000 000000ff 1a3ff509 
      */
      break;
    }
  }

  if(!getCrate(0, 0)->checkBOCLasersOn(0)) {
    cout << "Trying to send default scan using BOC that has its lasers cut out (continuing)\n";
    if(mrs) {
      *mrs << "BOC_INTERLOCKED" << MRS_WARNING << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<int>("crate", 0) 
           << MRS_PARAM<int>("rod", 0)
           << MRS_TEXT("Default scan continuing even though BOC interlocked") << ENDM;
    }
  }

  primList.push_back(RodPrimitive(4+24, 1, 3, 103, evSetup));

  // Start event trap
  long trap[0] = {};
  RodPrimList sPrimList(2);
  sPrimList.push_back(RodPrimitive(4, 0, 4096, 101, trap));
  buildSlavePrimList(primList, &sPrimList, 0, 1, 1);

  // Can clear slave list because its fully built!
  sPrimList.clear();
  sPrimList.push_back(RodPrimitive(4 + 13 /* + 2*3 */ , 3, 4098, 104, histSetup));
  buildSlavePrimList(primList, &sPrimList, 0, 1, 1);
  
  long task[] = {32, 100, 1, 1, 0, 0};
  sPrimList.clear();
  sPrimList.push_back(RodPrimitive(4 + 6, 4, 12, 101, task));
  buildSlavePrimList(primList, &sPrimList, 0, 1, 1);

  primList.push_back(RodPrimitive(histStartLength + 4, 3, 12, 101, histStart));

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Send default scan list\n";
  }
  sendPrimList(0, 0, 0, &primList);
  awaitResponse(0, 0, 0, 2);


  // Get result
  unsigned long length;

  unsigned long *result = getResponse(0, 0, 0, length);

  if(result) {
    cout << "Got response from default scan setup\n";
    printOutList(result, length);

    delete [] result;
  } else {
    cout << "No response from default scan setup\n";
  }

  // Poll histogram
//    pollHistogramming(extra, 0, scan.getScanPoints1().size(), scan.getNTrigs(), 5);

//    // Read histogram
//  #ifdef USE_IS
//    readHistogramsToIS(scan, extra);
//  #endif
//    readHistogramsToFile(scan, extra);
}


void SctApi::doScan(Scan &scan) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "doScan\n";
  }

  ScanEx extra;

  if(scan.getOption(Scan::DISTSLAVE) == 0) {
    extra.slaves = 1;
    extra.bitFieldDSP = 0x1;
  } else if(scan.getOption(Scan::DISTSLAVE) == 1) {
    extra.slaves = 0;
    extra.bitFieldDSP = 0x0; // Fill in which slaves are used later
  } else if(scan.getOption(Scan::DISTSLAVE) == 2) {
    extra.slaves = 4;
    extra.bitFieldDSP = 0xf;
  }

  extra.trimScan = 0;
  if(scan.getScanVariable1() == ST_TRIM) {
    // We need to download the trim configuration
    extra.trimScan = 1;
  }

  cout << "Do scan " << scanNumber << ":\n";
  scan.setRunNumber(runNumber);
  scan.setScanNumber(scanNumber);

  if(mrs) {
    *mrs << "SCAN_REQUESTED" << MRS_INFORMATION
         << MRS_PARAM<int>("run", runNumber) << MRS_PARAM<int>("scan", scanNumber) << MRS_TEXT("Scan requested") << ENDM;
  }

  scan.print();

  if(moduleMap.begin() == moduleMap.end()) {
    cout << "No modules to scan!\n";
    if(mrs) {
      *mrs << "SCAN_NO_MODULES" << MRS_ERROR 
           << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
           << MRS_TEXT("No modules to scan")
           << ENDM;
    }
    return;
  }

#warning "Scan only first ROD!"
  // Initialise with first module
  UINT32 mid = moduleMap.begin()->first;
  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);


  extra.partition = partition;
  extra.crate = crate;
  extra.rod = rod;
  extra.channels[0] = 0;
  extra.channels[1] = 0;

  for(int i=0; i<4; i++) {
    extra.dspChannels[i][0] = 0;
    extra.dspChannels[i][1] = 0;
  }

  // Do checks on scan
  if(scan.getScanPoints2().size() != 0 && scan.getScanPoints2().size() != scan.getScanPoints1().size()) {
    cout << "Can't have two different sized scan points lists!\n";
    if(mrs) {
      *mrs << "SCAN_MISMATCH" << MRS_ERROR 
           << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
           << MRS_TEXT("doScan supplied with two different sized scan points lists!")
           << ENDM;
    }
    return;
  }

  std::vector<std::list<std::string> > groupLists;

  // Do checks on module list
  for(map<UINT32, ABCDModule>::const_iterator mi = moduleMap.begin();
      mi!=moduleMap.end();
      mi ++) {
    UINT32 currMid = mi->first;

    unsigned int newPartition, newCrate, newRod, newChannel;

    getpcrc(currMid, newPartition, newCrate, newRod, newChannel);

    if(newPartition != partition) {
      cout << "Can't do scans on modules in different partitions (ever?)\n";
      if(mrs) {
        *mrs << "SCAN_MISMATCH" << MRS_ERROR 
             << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
             << MRS_TEXT("Mismatch partitions. Can't do scans on modules in different partitions (ever?)")
             << ENDM;
      }
      return;
    }

    if(newCrate != crate) {
      cout << "Can't do scans on modules in different crates (yet)\n";
      if(mrs) {
        *mrs << "SCAN_MISMATCH" << MRS_ERROR 
             << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
             << MRS_TEXT("Crate mismatch, can't do scans on modules in different crates (yet)")
             << ENDM;
      }
      return;
    }

    if(newRod != rod) {
      cout << "Can't do scans on modules in different rods (yet)\n";
      if(mrs) {
        *mrs << "SCAN_MISMATCH" << MRS_ERROR 
             << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
             << MRS_TEXT("Can't do scans on modules in different rods (yet)")
             << ENDM;
      }
      return;
    }

    ABCDModule *config = lookupConfig(currMid);

    if(!config) {
      cout << "Trying to do a scan on a module with no configuration FAILED!\n";
      if(mrs) {
        *mrs << "SCAN_NOCONFIG" << MRS_ERROR 
             << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
             << MRS_TEXT("Trying to do a scan on a module with no configuration FAILED!")
             << ENDM;
      }
      return;
    }

#warning "If on different ROD then deal with differently"
//     // Check if redundancy in use
//     if(config->select) {
//       // does clk/cmd go from same ROD
//       cout << "Can't do redundant stuff at the moment, SCAN CANCELLED\n";
//       return;
//     }

    if(newChannel<32) {
      if(extra.channels[0] & 1<<newChannel) {
        cout << "Module already using this channel in this group\n";
      }
      extra.channels[0] |= 1<<newChannel;

      if(scan.getOption(Scan::DISTSLAVE) == 1) {
        extra.dspChannels[config->groupId][0] |= 1<<newChannel;
        if(!(extra.bitFieldDSP & (1<<config->groupId))) extra.slaves++; 
        extra.bitFieldDSP |= 1<<config->groupId;
      } else if(scan.getOption(Scan::DISTSLAVE) == 0) {
        extra.dspChannels[0][0] |= 1<<newChannel;
      }
    } else {
      if(extra.channels[1] & 1<<(newChannel-32)) {
        cout << "Module already using this channel in this group\n";
      }
      extra.channels[1] |= 1<<(newChannel-32);

      if(scan.getOption(Scan::DISTSLAVE) == 1) {
        extra.dspChannels[config->groupId][1] |= 1<<newChannel;
        if(!(extra.bitFieldDSP & (1<<config->groupId))) extra.slaves++; 
        extra.bitFieldDSP |= 1<<config->groupId;
      } else if(scan.getOption(Scan::DISTSLAVE) == 0) {
        extra.dspChannels[0][1] |= 1<<newChannel;
      }
    }

    int group = config->groupId;

    if(group+1 > groupLists.size()) {
      groupLists.resize(group + 1);
    }
    groupLists[group].push_back(convertToString(currMid));
  }

  for(int i = 0;
      i<groupLists.size();
      i++) {
    scan.setModuleList(i, groupLists[i]);
  }

  scan.setNGroups(groupLists.size());


  cout << hex << "Channels in use: 0x" << extra.channels[0] << " 0x" << extra.channels[1] << dec << endl;

  for(int i=0; i<4; i++) {
    cout << hex << "Slave channels: 0x" << extra.dspChannels[i][0] << " 0x" << extra.dspChannels[i][1] << dec << endl;
  }

  cout << "Slave DSP bit - field: " << hex << (int)extra.bitFieldDSP << dec << " nSlaves: " << extra.slaves << endl;

  // Check that the appropriate slaves were initialised 
  for(int i=0; i<4; i++) {
    if(extra.bitFieldDSP & (1<<i) && getCrate(partition, crate) && !getCrate(partition, crate)->slavePresent(rod, i)) {
      cout << "***** A slave DSP necessary for the requested scan was not initialised\n";
      cout << "*****  please check the configuration and the Slave Image files\n";
      if(mrs) {
        *mrs << MRS_ERROR 
             << "SCAN_NODSP" 
             << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
             << MRS_PARAM<int>("slavesNeeded", extra.bitFieldDSP)
             << MRS_PARAM<int>("slaveNotStarted", i)
             << MRS_TEXT("Required DSPs not initialised for the requested scan")
             << ENDM;
      }
      return;
    }
  }

  if(extra.bitFieldDSP & 0xf == 0) {
    cout << "No slaves have been deemed involved in histogramming aborting\n";
    if(mrs) {
      *mrs << MRS_ERROR 
           << "SCAN_NODSP" 
           << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
           << MRS_TEXT("No DSP matched to modules for scan")
           << ENDM;
    }
    return;
  }

  for(map<UINT32, ABCDModule>::const_iterator mi = moduleMap.begin();
      mi!=moduleMap.end();
      mi ++) {
    UINT32 currMid = mi->first;

    /* Make sure module configuration is in place (or reset it after previous scan) */
    setABCDModule(currMid, 0);
    setABCDModule(currMid, 1);
    sendABCDModule(currMid, 1);

    cout << "Configuration for module " << currMid << " " << convertToString(currMid) << " uploaded\n";
  }

#warning "Need to check H/T limit error and reset ROD, ROD gets confused by clk/2!" // Reset is done by sendABCDModule(0xffffffff)...

  cout << "Module Configurations for scan uploaded to ROD\n";

  cout << "ROD configuration (masks etc)\n"; 
  calib_init();
  cout << "... Done\n";

  // Clear event memory
  // 8192 words from 0x80008000
  for(int slaveNumber=0; slaveNumber<4; slaveNumber++) {
    if((extra.bitFieldDSP & (1<<slaveNumber)) && getCrate(partition, crate)->slavePresent(rod, slaveNumber))
      setSlaveBlock(partition, crate, rod, slaveNumber, 0x80008000, 8192, 0);
  }

  RodPrimList primList(1);

  setupEventTrapping(scan, extra, primList);
  startEventTrapping(scan, extra, primList);

  cout << "Histogramming setup...\n";

  setupHistogramming(scan, extra, primList);

  // Start triggers when necessary

//    // Wait for things to settle before starting histogramming
//    sendPrimList(partition, crate, rod, &primList);
//    awaitResponse(partition, crate, rod, 5);
//    primList.clear();

  if(scan.getOption(Scan::DEBUG)) {
    if(scan.getOption(Scan::DEBUG) > 1) {
      cout << "** Debug mode setting REPORTS & step mode\n"; 
      cout << "**  Use \"tapi.dspSingleWrite(0, 0, 0, 0x80000010, 0x" << hex << ((1<<6) + (1<<7) + (1<<23)) << dec << ", -1);\" to step\n";

      dspSingleWrite(partition, crate, rod, 0x80000010, (1<<6) + (1<<23), -1);
    } else {
      cout << "** Debug mode setting AUTO_STALL & REPORTS\n"; 
      dspSingleWrite(partition, crate, rod, 0x80000010, (1<<21) + (1<<23), -1);
    }
  }

  if(mrs) {
    *mrs << "SCAN_STARTED" << MRS_INFORMATION 
         << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
         << MRS_TEXT("Scan started") 
         << MRS_PARAM<int>("ScanNumber", scanNumber)
         << MRS_PARAM<int>("ScanType1", scan.getScanVariable1())
         << MRS_PARAM<int>("ScanType2", scan.getScanVariable2())
         << ENDM;
  }

  time_t scanStart = time(0);

  if(!getCrate(extra.partition, extra.crate)->checkBOCLasersOn(extra.rod)) {
    cout << "Trying to do scan using BOC that has its lasers cut out (aborting)\n";
    if(mrs) {
      *mrs << "BOC_INTERLOCKED" << MRS_ERROR << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<int>("crate", crate) 
           << MRS_PARAM<int>("rod", rod)
           << MRS_TEXT("Scan aborting (BOC interlocked)") << ENDM;
    }

    return;
  }

  startHistogramTask(scan, extra, primList);

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Send Scan compound primitive list\n";
  }
  sendPrimList(partition, crate, rod, &primList);
  awaitResponse(partition, crate, rod, 5);

  cout << "Histogramming started\n";
  // Let it get going so the bin counter gets reset
  sleep(1);

  if(!scan.getOption(Scan::DEBUG)) {
    if(responseCode == 0) {
      cout << "Polling histogramming task\n";

      int slave;
      for(slave = 0; slave<4; slave++) {
        if(extra.bitFieldDSP & (1 << slave)) {
          break;
        }
      }
      pollHistogramming(extra, slave, scan.getScanPoints1().size(), scan.getNTrigs(), 5);

      time_t scanEnd = time(0);

      if(mrs) {
        *mrs << "SCAN_FINISHED" << MRS_INFORMATION 
             << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
             << MRS_TEXT("Scan finished") 
             << MRS_PARAM<int>("ScanNumber", scanNumber)
             << MRS_PARAM<int>("ScanType1", scan.getScanVariable1())
             << MRS_PARAM<int>("ScanType2", scan.getScanVariable2())
             << MRS_PARAM<int>("ScanTime", scanEnd - scanStart)
             << ENDM;
      }

      cout << "Histogramming complete, " << (scanEnd - scanStart) << " seconds, tidying up...\n";
      {
        boost::mutex::scoped_lock lock(logMutex);
        log << "Histogramming complete, " << (scanEnd - scanStart) << " seconds\n";
      }

      cout << "Reading histogram...\n";

      readHistogramsToFile(scan, extra);
#ifdef USE_IS
      readHistogramsToIS(scan, extra);
#endif
      cout << "Histogram read\n";
    } else {
      cout << "Histogram initialisation failed\n";
    }

    stopHistogramming(extra);

    // Only restore module config if not debug mode
    setABCDModule(0xffffffff, 0);
    sendAllABCDModules(0);
  } else {
    if(mrs) {
      *mrs << "SCAN_DEBUG" << MRS_INFORMATION 
           << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doScan") 
           << MRS_TEXT("Scan in debug mode, probably best to be at the command line!") 
           << ENDM;
    }

    cout << "** In debug mode, you're responsible for terminating histogram and for readout\n";

    cout << "Trying polling with small timeout\n";
    int slave;
    for(slave = 0; slave<4; slave++) {
      if(extra.bitFieldDSP & (1 << slave)) {
        break;
      }
    }
    pollHistogramming(extra, slave, scan.getScanPoints1().size(), scan.getNTrigs(), 1);

    // Leave module config alone
  }

  scanNumber ++;

  // This should now have been updated with the modules in the scan
  scan.print();
}

void SctApi::setupEventTrapping(const Scan &scan, const ScanEx &ex, RodPrimList &primList) {
  // Set up each slave individually
  for(int slaveNumber=0; slaveNumber<4; slaveNumber++) {
    if((ex.bitFieldDSP & (1<<slaveNumber)) && getCrate(ex.partition, ex.crate)->slavePresent(ex.rod, slaveNumber)) {
      long *evData = new long[sizeof(EVENT_TRAP_SETUP_IN)/4];

      EVENT_TRAP_SETUP_IN &primData = *(EVENT_TRAP_SETUP_IN*)evData;

#if R_EVENT_TRAP_SETUP == 103
      primData.slvBits = 0x1 << slaveNumber;
      primData.numberOfEvents = COLLECT_FOREVER;
      primData.timeoutInUsec = 0x2000;

      primData.extRouterSetup = 0;

      // This tells the setup code to change the trapRemainder
      //  Only matters if setting up more than one slave
      primData.distribute = TRAP_DISTRIB_NEITHER;

      primData.releaseFrames = 0;
      primData.permitBackPressure = 1;
      primData.dataMode = 0;                        // Data or Event mode
      primData.sLink = 0;                           // Where to pick events (dsp or router)

      primData.format = TRAP_FMT_NORMAL;            // Slink format
      primData.trapStray = 1;                       // Trap stray events
      primData.iterLimit = 2;                       // Don't keep them long

      // Trap events from ROD triggers
      primData.trapConfig[0] = TRAP_CFG_ROD_EVT;
      primData.trapExclusionFlag[0] = 0;            // Don't exclude anything
      primData.trapFunction[0] = TRAP_FXN_HISTOGRAM;

      if(scan.getOption(Scan::DISTSLAVE) == 2) {
        primData.trapMatch[0] = 0;                  // = slave number??
        primData.trapModulus[0] = ex.slaves;        // Trap if (count % mod) == rem
        primData.trapRemainder[0] = slaveNumber;
      } else {
        primData.trapMatch[0] = slaveNumber;        // = slave number??
        primData.trapModulus[0] = 1;                // Trap if (count % mod) == rem
        primData.trapRemainder[0] = 0;
      }

      // Trap one is idle
      primData.trapConfig[1] = TRAP_CFG_IDLE;
      primData.trapExclusionFlag[1] = 0;
      primData.trapFunction[1] = 0;

      primData.trapMatch[1] = 0; 
      primData.trapModulus[1] = 0;
      primData.trapRemainder[1] = 0;

#else 
#error "Event trapping not compiled (new Primitive definition)!"
#endif
      primList.push_back(RodPrimitive(sizeof(primData)/sizeof(UINT32) + 4, 
                                      0, EVENT_TRAP_SETUP, R_EVENT_TRAP_SETUP, evData));
    }
  }

#warning "Mem leak doesn't delete evData (* numSlaves)"
}

void SctApi::startEventTrapping(const Scan &scan, const ScanEx &ex, RodPrimList &primList) {
#warning "Is a simple loop for event trap right?"
  for(int slaveNumber=0; slaveNumber<4; slaveNumber++) {
    if((ex.bitFieldDSP & (1<<slaveNumber)) && getCrate(ex.partition, ex.crate)->slavePresent(ex.rod, slaveNumber)) {
      long *trap = new long[0];

      RodPrimList startList(1);
      startList.push_back(RodPrimitive(4, 0, START_EVENT_TRAPPING, R_START_EVENT_TRAPPING, trap));

      buildSlavePrimList(primList, &startList, slaveNumber, 1, 1);

      delete [] trap;
    }
  }
}

void  SctApi::setupHistogramming(const Scan &scan, const ScanEx &ex, RodPrimList &primList) {
#warning "Is a simple loop like this right?"
  for(int slaveNumber = 0; slaveNumber < 4; slaveNumber ++) {
    if((ex.bitFieldDSP & (1<<slaveNumber)) && getCrate(ex.partition, ex.crate)->slavePresent(ex.rod, slaveNumber)) {
      // Primitive
      long *histoData = new long[sizeof(HISTOGRAM_SETUP_IN)/4];

      HISTOGRAM_SETUP_IN &primData = *(HISTOGRAM_SETUP_IN*)histoData;

#if R_HISTOGRAM_SETUP == 105
      primData.base = (UINT32 *)HISTOGRAM_DEFAULT_BASE;    // 0x20d2000
      primData.nBins = scan.getScanPoints1().size();

      primData.padding[0] = 0;
      primData.padding[1] = 0;

      primData.dataType[0] = scan.getScanVariable1();
      primData.dataType[1] = scan.getScanVariable2();

      primData.binSize = scan.getOption(Scan::BITS32)?HISTOGRAM_32BIT:HISTOGRAM_16BIT;

      // arrangement
      primData.opt[0] = scan.getOption(Scan::FORMAT)?HISTOGRAM_SLICE:HISTOGRAM_BLOCK;
      // dataFormat
      primData.opt[1] = scan.getOption(Scan::FULL)?HISTOGRAM_FULL:HISTOGRAM_CONDENSED;          // Get all three data sets
      // unused
      primData.opt[2] = 0;

      if(scan.getOption(Scan::DISTSLAVE) == 1) {
        primData.validModules[0] = ex.dspChannels[slaveNumber][0];          // Modules to this slave
        primData.validModules[1] = ex.dspChannels[slaveNumber][1];          //    32-47 in low 16 bits
        primData.moduleRangeMap[0][0] = ex.dspChannels[slaveNumber][0];     // All modules use rangemap 0
        primData.moduleRangeMap[0][1] = ex.dspChannels[slaveNumber][1];
      } else {
        primData.validModules[0] = ex.channels[0];                          // Modules to this slave
        primData.validModules[1] = ex.channels[1];                          //    32-47 in low 16 bits
        primData.moduleRangeMap[0][0] = ex.channels[0];                     // All modules use rangemap 0
        primData.moduleRangeMap[0][1] = ex.channels[1];
      }
      primData.moduleRangeMap[1][0] = 0;                                    // No modules for rangemap 1
      primData.moduleRangeMap[1][1] = 0;

      primData.xPtr[0] = (FLOAT32 *)0x3f00000;                              // Where the fit data will be stored
      primData.xPtr[1] = (FLOAT32 *)0x3f01000; 
#elif R_HISTOGRAM_SETUP == 104
      primData.base = (UINT32 *)HISTOGRAM_DEFAULT_BASE;    // 0x20d2000
      primData.rodType = ROD_TYPE_SCT;
      primData.nBins = scan.getScanPoints1().size();

      primData.padding[0] = 0;
      primData.padding[1] = 0;

      primData.dataType[0] = scan.getScanVariable1();
      primData.dataType[1] = scan.getScanVariable2();

      primData.arrangement = scan.getOption(Scan::FORMAT)?HISTOGRAM_SLICE:HISTOGRAM_BLOCK;
      primData.dataFormat = scan.getOption(Scan::FULL)?HISTOGRAM_FULL:HISTOGRAM_CONDENSED;          // Get all three data sets
      primData.binSize = scan.getOption(Scan::BITS32)?HISTOGRAM_32BIT:HISTOGRAM_16BIT;
      primData.unused = 0;

      if(scan.getOption(Scan::DISTSLAVE) == 1) {
        primData.validModules[0] = ex.dspChannels[slaveNumber][0];          // Modules to this slave
        primData.validModules[1] = ex.dspChannels[slaveNumber][1];          //    32-47 in low 16 bits
        primData.moduleRangeMap[0][0] = ex.dspChannels[slaveNumber][0];     // All modules use rangemap 0
        primData.moduleRangeMap[0][1] = ex.dspChannels[slaveNumber][1];
      } else {
        primData.validModules[0] = ex.channels[0];                          // Modules to this slave
        primData.validModules[1] = ex.channels[1];                          //    32-47 in low 16 bits
        primData.moduleRangeMap[0][0] = ex.channels[0];                     // All modules use rangemap 0
        primData.moduleRangeMap[0][1] = ex.channels[1];
      }
      primData.moduleRangeMap[1][0] = 0;                                    // No modules for rangemap 1
      primData.moduleRangeMap[1][1] = 0;

      primData.xPtr[0] = (FLOAT32 *)0x3f00000;                              // Where the fit data will be stored
      primData.xPtr[1] = (FLOAT32 *)0x3f01000; 
#else 
#error "Histogram setup not compiled (new Primitive definition)!"
#endif

      long *taskData = new long[sizeof(START_TASK_IN)/4];

      for(unsigned int i=0; i<sizeof(START_TASK_IN)/4; i++) {
        taskData[i] = 0;
      }

      START_TASK_IN &taskPrim = *(START_TASK_IN *)taskData;

      taskPrim.taskType = HISTOGRAM_TASK;
      taskPrim.taskRevision = R_HISTOGRAM_TASK;
      taskPrim.priority = 1; // Top???
      taskPrim.completionFlag = 1; 

      HISTOGRAM_TASK_IN &histoTaskData = taskPrim.taskStruct.histogramTaskIn;

      histoTaskData.nEvents = COLLECT_FOREVER;
      histoTaskData.controlFlag = MASTER_HREG; // ???

      RodPrimList myList(4);

      myList.push_back(RodPrimitive(4 + sizeof(HISTOGRAM_SETUP_IN)/sizeof(UINT32), 
                                    2, HISTOGRAM_SETUP, R_HISTOGRAM_SETUP, histoData));
      myList.push_back(RodPrimitive(4 + sizeof(START_TASK_IN)/4,
                                    3, START_TASK, R_START_TASK, taskData));

      buildSlavePrimList(primList, &myList, slaveNumber, true, true);

      delete [] taskData;
      delete [] histoData;
    }
  }
}

void SctApi::startHistogramTask(const Scan &scan, const ScanEx &ex, RodPrimList &primList) {
  long *ctrlData = new long[sizeof(START_TASK_IN)/4 + scan.getScanPoints1().size() * 2];

  for(unsigned int i=0; i<sizeof(START_TASK_IN)/4; i++) {
    ctrlData[i] = 0;
  }

  struct START_TASK_IN &primData = *(START_TASK_IN*)ctrlData;
  struct HISTOGRAM_CTRL_TASK_IN &taskData = primData.taskStruct.histoCtrlTaskIn; 

#if (R_HISTOGRAM_CTRL_TASK == 107)
  primData.taskType = HISTOGRAM_CTRL_TASK;
  primData.taskRevision = R_HISTOGRAM_CTRL_TASK;
  primData.priority = 1;
  primData.completionFlag = 1; 

  taskData.slvBits = ex.bitFieldDSP;   // Which DSPs to use

  taskData.port = 0;

  taskData.configRegister[0] = scan.getScanVariable1(); // ST_VTHR;
  taskData.configRegister[1] = scan.getScanVariable2(); // ST_VTHR;

  taskData.configSctSet = WORK_MODULE_CONFIG;    // Which conf set to use
  // Which bit of configuration to send, unless this is a trim scan then basic should be enough
  if(!ex.trimScan) 
    taskData.dataSet = MODULE_BASIC; 
  else 
    taskData.dataSet = MODULE_TRIM;               // Which bit of configuration to send

  taskData.groupRangeMap[0] = 0x0f;         // Group for each range list
  taskData.groupRangeMap[1] = 0x0;          // All groups use range list 0

  if(scan.getOption(Scan::DISTSLAVE) == 0) {               // Which groups for each DSP to use
    // All groups to slave 0
    taskData.groupDSPMap[0] = 0xff;
    taskData.groupDSPMap[1] = 0x0;
    taskData.groupDSPMap[2] = 0x0;
    taskData.groupDSPMap[3] = 0x0;
  } else {
    // Each group to a separate DSP
    taskData.groupDSPMap[0] = 0x1;
    taskData.groupDSPMap[1] = 0x2;
    taskData.groupDSPMap[2] = 0x4;
    taskData.groupDSPMap[3] = 0x8;
  }

  taskData.groupSPMap[0] = 0xf;         // Groups 0-3 use port 0
  taskData.groupSPMap[1] = 0xf0;        // Groups 4-7 use port 1

  taskData.globalCtrl = 0;              // Don't want to take control for internal triggers
  taskData.syncLevel = 0;               //  So don't need synchronisation 

  taskData.histoBase = (UINT32*)HISTOGRAM_DEFAULT_BASE;    // 0x20d2000

  taskData.arrangement = scan.getOption(Scan::FORMAT)?HISTOGRAM_SLICE:HISTOGRAM_BLOCK;
  taskData.dataFormat = scan.getOption(Scan::FULL)?HISTOGRAM_FULL:HISTOGRAM_CONDENSED;
  taskData.binSize = scan.getOption(Scan::BITS32)?HISTOGRAM_32BIT:HISTOGRAM_16BIT;
  taskData.unused1 = 0;

  taskData.extSetup = 0xf;                     // From Test Stand (1111b)
  taskData.dataPath = DATA_PATH_NORMAL;        // Other option is INMEM (presumably TestBench based)
  taskData.capture = 0;
  taskData.useRangeList = 0;                   // Major difference from Doug's example!

  taskData.repetitions = scan.getNTrigs();
  taskData.nBins = scan.getScanPoints1().size();
  taskData.bin0 = 0;                           // Start at 0

  // Ignore range list data 
  for(int i=0; i<5; i++) {
    taskData.rangeList[0].xPair[i].x0 = 0.0;
    taskData.rangeList[1].xPair[i].x0 = 0.0;
    taskData.rangeList[0].xPair[i].delta_x = 0.0;
    taskData.rangeList[1].xPair[i].delta_x = 0.0;
  }

  // X data lists
  taskData.dataPtr[0] = (FLOAT32 *)0xffffffff;   // Data for group 1               Append to end of primitive
  taskData.dataPtr[1] = (FLOAT32 *)0xffffffff;   // Data for group 2

  // Clear trigger sequences
  for(int i=0; i<N_CMD_LIST_CMDS; i++) {   // from primParams.h
    taskData.triggerSequence[0].cmd[i] = NO_CMD;        // Set a trigger sequence
    taskData.triggerSequence[1].cmd[i] = NO_CMD;        // Set a trigger sequence
    taskData.triggerSequence[0].data[i] = 0;        // Set a trigger sequence
    taskData.triggerSequence[1].data[i] = 0;        // Set a trigger sequence
  }


//    // Two L1As (illegal without delay?!)
//    taskData.triggerSequence[0].cmd[0] = L1_TRIGGER;
//    taskData.triggerSequence[1].cmd[0] = L1_TRIGGER;
//    taskData.triggerSequence[0].cmd[1] = L1_TRIGGER;
//    taskData.triggerSequence[1].cmd[1] = L1_TRIGGER;

  Trigger::RODTriggers points = scan.getTrigger1()->getRODTriggers();

  for(unsigned int i=0; i<points.size(); i++) {
    taskData.triggerSequence[0].cmd[i] = points[i].first;        // Set a trigger sequence
    taskData.triggerSequence[0].data[i] = points[i].second;        // Set a trigger sequence
  }

  if(scan.getScanPoints2().size() > 0) {
    points = scan.getTrigger2()->getRODTriggers();

    for(unsigned int i=0; i<points.size(); i++) {
      taskData.triggerSequence[1].cmd[i] = points[i].first;        // Set a trigger sequence 2
      taskData.triggerSequence[1].data[i] = points[i].second;        // Set a trigger sequence 2
    }
  } else {
    // Leave as default
  }

//    taskData.triggerSequence[0].cmd[0] = CALIBRATION_PULSE;
//    taskData.triggerSequence[0].cmd[1] = DELAY;
//    taskData.triggerSequence[0].cmd[2] = L1_TRIGGER;

//    taskData.triggerSequence[0].data[1] = 0x81;

  unsigned short cmd, data;

  scan.getTrigger1()->getCommIncr(cmd, data);
  taskData.incCmd[0] = cmd;
  taskData.incData[0] = data;
  scan.getTrigger2()->getCommIncr(cmd, data);
  taskData.incCmd[1] = cmd;
  taskData.incData[1] = data;

  taskData.calLineLoop = scan.getOption(Scan::LOOPCALLINE);
  if(scan.getOption(Scan::DISTSLAVE) == 2) {
    taskData.distributionToggle = ROUTER_DISTRIB;
  } else {
    taskData.distributionToggle = MODULE_GROUP;
  }

  for(int i=0; i<10; i++) {
    taskData.genData[i] = 0;
  }

  taskData.genData[0] = 0xf;
#elif (R_HISTOGRAM_CTRL_TASK == 105)
  primData.taskType = HISTOGRAM_CTRL_TASK;
  primData.taskRevision = R_HISTOGRAM_CTRL_TASK;
  primData.priority = 0; // Top???
  primData.completionFlag = 1; 

  taskData.slvBits = 0xf;             // Use all slaves
  taskData.configSctSet = INIT_MODULE_CONFIG;
  // Not an array yet
//    taskData.configRegister[0] = 0;     // V_THR ??
//    taskData.configRegister[1] = 0;     // V_THR ??
  taskData.dataSet = 0;               // ??
  taskData.useRangeList = 0;          // ??
  taskData.groupRangeMap[0] = 0;      // No range map
  taskData.groupRangeMap[1] = 0;      //  ditto
  taskData.cmdBuff = CMD_BUFFER_BOTH;     // Which serial port to use
  taskData.dataPtr[0] = group1Data;   // Data for group 1
  taskData.dataPtr[1] = group2Data;   // Data for group 2
  taskData.repetitions = 10;          // Make it stop quickly 10 events
  taskData.nBins = 10;                // 10 bins
  taskData.bin0 = 0;                  // Start at 0

  taskData.triggerSequence[0];        // Set a trigger sequence
  taskData.triggerSequence[1];        // Set a trigger sequence

  taskData.incCmd[0] = 0;
  taskData.incCmd[1] = 0;
  taskData.calLineLoop = 0;  // Doesn't work yet??

  taskData.distributionToggle = 0;  // ??
  taskData.incData[0] = 1;
  taskData.incData[1] = 1;

  taskData.groupSPMap[0] = 0;
  taskData.groupSPMap[1] = 0;
#else  // Unknown R_HISTOGRAM_CTRL_TASK
#error "Histogram Task not compiled (new Primitive definition)!" 
#endif

  int offset = sizeof(START_TASK_IN)/4;

  for(unsigned int i=0; i<scan.getScanPoints1().size(); i++) {
    ctrlData[offset + i] = *(UINT32 *)&(scan.getScanPoints1()[i]);
    if(scan.getScanPoints2().size() == 0) {
      ctrlData[offset + scan.getScanPoints1().size() + i] = *(UINT32*)&(scan.getScanPoints1()[i]);
    } else {
      ctrlData[offset + scan.getScanPoints1().size() + i] = *(UINT32*)&(scan.getScanPoints2()[i]);
    }
  }

  primList.push_back(RodPrimitive(4 + sizeof(START_TASK_IN)/sizeof(UINT32) + scan.getScanPoints1().size() * 2, 
                                  0, START_TASK, R_START_TASK, ctrlData));

#warning "Mem leak  Doesn't delete ctrlData"
}

/*
  Stop histogram control in master
  Histogram and event trapping in slaves


  *********** XXX Send them individually as it doesn't seem to work otherwise ***************
*/
void SctApi::stopHistogramming(ScanEx &ex) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Stop histogramming (controlled)\n";
  }
//  stopEvTrap(0);

//  taskOp(16, 0, 0, -1);   // Stop histogram control task

//  taskOp(32, 0, 0, 0);     // Stop histogram task

  RodPrimList primList(2);

  long *taskData = new long[sizeof(TASK_OPERATION_IN) / 4];

  TASK_OPERATION_IN &evOp = *(TASK_OPERATION_IN*)taskData;

#if (R_TASK_OPERATION == 100) 
  // Stop histogram control task
  evOp.taskType = HISTOGRAM_CTRL_TASK;
  evOp.taskOperation = TASK_STOP;
  evOp.data = 0;
#else 
#error "TASK_OPERATION revision change"
#endif

  primList.push_back(RodPrimitive(4 + sizeof(TASK_OPERATION_IN)/4, 
                                  1, TASK_OPERATION, R_TASK_OPERATION, taskData));
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Stop histo control task\n";
  }
  sendPrimList(ex.partition, ex.crate, ex.rod, &primList);

  delete [] taskData;

  awaitResponse(ex.partition, ex.crate, ex.rod, 5);
  primList.clear();

  cout << "Done kill control task\n";

  for(int slaveNumber=0; slaveNumber<4; slaveNumber++) {
    // Kill all tasks!!!!
    RodPrimList slList(1);

    // Stop Slave histogramming task

    // Lots of memory leaks
    taskData= new long[sizeof(TASK_OPERATION_IN) / 4];

    TASK_OPERATION_IN &taskOp = *(TASK_OPERATION_IN*)taskData;

#if (R_TASK_OPERATION == 100) 
    taskOp.taskType = HISTOGRAM_TASK;
    taskOp.taskOperation = TASK_STOP;
    taskOp.data = 0;
#else 
#error "TASK_OPERATION revision change"
#endif

    slList.push_back(RodPrimitive(4 + sizeof(TASK_OPERATION_IN)/4, 
                                  1, TASK_OPERATION, R_TASK_OPERATION, taskData));

    buildSlavePrimList(primList, &slList, slaveNumber, true, true);
    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "Stop slave histogram task " << slaveNumber << endl;
    }
    sendPrimList(ex.partition, ex.crate, ex.rod, &primList);
    awaitResponse(ex.partition, ex.crate, ex.rod, 5);
    slList.clear();
    primList.clear();

    cout << "Done kill histogram task\n";

#if (R_STOP_EVENT_TRAPPING == 100)
    // Stop event trapping
    long *trap = new long[0];
#else
#error "STOP_EVENT_TRAPPING revision change"
#endif

    slList.push_back(RodPrimitive(4, 0, STOP_EVENT_TRAPPING, R_STOP_EVENT_TRAPPING, trap));

    buildSlavePrimList(primList, &slList, slaveNumber, true, true);
    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "Stop Event trapping on slave " << slaveNumber << endl;
    }
    sendPrimList(ex.partition, ex.crate, ex.rod, &primList);

    delete [] trap;
    delete [] taskData;

    awaitResponse(ex.partition, ex.crate, ex.rod, 5);
    primList.clear();

    cout << "Done kill event trapping\n";
  }

  //  sendPrimList(ex.partition, ex.crate, ex.rod, &primList);
  awaitResponse(ex.partition, ex.crate, ex.rod, 2);
}

void SctApi::tidyHistogramming() {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Brutal histogram stopping\n";
  }

  RodPrimList primList(2);

  long *taskData = new long[sizeof(TASK_OPERATION_IN) / 4];

  TASK_OPERATION_IN &evOp = *(TASK_OPERATION_IN*)taskData;

#if (R_TASK_OPERATION == 100)
  // Stop histogram control task
  evOp.taskType = HISTOGRAM_CTRL_TASK;
  evOp.taskOperation = TASK_STOP;
  evOp.data = 0;
#else 
#error "TASK_OPERATION revision change"
#endif

  unsigned int partition, crate, rod;
#warning "Multiple ROD bug"
  partition = crate = rod = 0;

  primList.push_back(RodPrimitive(4 + sizeof(TASK_OPERATION_IN)/4, 
                                  1, TASK_OPERATION, R_TASK_OPERATION, taskData));
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Stop HCT\n";
  }
  sendPrimList(partition, crate, rod, &primList);

  delete [] taskData;

  awaitResponse(partition, crate, rod, 2);
  primList.clear();

  cout << "Done kill control task\n";

  for(int slaveNumber=0; slaveNumber<4; slaveNumber++) {
    RodPrimList slList(1);

    // Stop Slave histogramming task

    // Lots of memory leaks
    taskData= new long[sizeof(TASK_OPERATION_IN) / 4];

    TASK_OPERATION_IN &taskOp = *(TASK_OPERATION_IN*)taskData;

#if (R_TASK_OPERATION == 100)
    taskOp.taskType = HISTOGRAM_TASK;
    taskOp.taskOperation = TASK_STOP;
    taskOp.data = 0;
#else
#error "TASK_OPERATION revision change"
#endif

    slList.push_back(RodPrimitive(4 + sizeof(TASK_OPERATION_IN)/4, 
                                  1, TASK_OPERATION, R_TASK_OPERATION, taskData));

    buildSlavePrimList(primList, &slList, slaveNumber, true, true);
    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "Stop histo task (" << slaveNumber << ")\n";
    }
    sendPrimList(partition, crate, rod, &primList);
    awaitResponse(partition, crate, rod, 2);
    slList.clear();
    primList.clear();

    cout << "Done kill histogram task\n";

#if (R_STOP_EVENT_TRAPPING == 100)
    // Stop event trapping
    long *trap = new long[0];
#else
#error "STOP_EVENT_TRAPPING revision change"
#endif

    slList.push_back(RodPrimitive(4, 0, STOP_EVENT_TRAPPING, R_STOP_EVENT_TRAPPING, trap));

    buildSlavePrimList(primList, &slList, slaveNumber, true, true);
    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "Stop ETS (" << slaveNumber << ")\n";
    }
    sendPrimList(partition, crate, rod, &primList);

    delete [] trap;

    awaitResponse(partition, crate, rod, 2);
    primList.clear();

    cout << "Done kill event trapping\n";
  }

//     sendPrimList(partition, crate, rod, &primList);
//    awaitResponse(partition, crate, rod, 2);
}

scan_result_ptrs SctApi::readHistogramData(const Scan &scan, const ScanEx &ex, UINT32 mid, int index) {
  scan_result_ptrs scanResult;

  int nBins = scan.getScanPoints1().size();

  // There must be some configuration for it to have got this far
  ABCDModule &moduleConfig = *lookupConfig(mid);

  ScanHeader &header = scanResult.header;

  header.version = 0;
  header.length = sizeof(ScanHeader);
  header.runNumber = runNumber;
  header.scanNumber = scanNumber;
  strncpy(header.moduleName, convertToString(mid).c_str(), 16);
  header.moduleName[15] = 0;    // Null terminate the string
//   snprintf(header.moduleName, 16, "%s", convertToString(mid).c_str());
//   cout << "Module name in Scan Results (" << header.moduleName << ")\n" ;
  header.scanType = scan.getScanVariable1();
  header.npoints = nBins;
  header.size = nBins * 2 * 1024 * 2; // (scan.getOption(Scan::BITS32)?2:1);
  header.dataType = SR_DT_SLICE;

  header.width = scan.getOption(Scan::BITS32)?SR_WD_32:SR_WD_16;

  header.config = moduleConfig;

  header.pntPoints = sizeof(ScanHeader);
  header.pntEvents = header.pntPoints + nBins * sizeof(FLOAT32);
  header.pntErrors = header.pntEvents + nBins * sizeof(UINT32);
  header.pntData = header.pntErrors + nBins * sizeof(UINT32);

  // The points
  FLOAT32 *points = new FLOAT32[nBins];
  for(int i=0; i<nBins; i++) {
    points[i] = scan.getScanPoints1()[i];
  }
  scanResult.points = points;

  // The event counts
  UINT32 *events = new UINT32[nBins];
  for(int i=0; i<nBins; i++) {
#warning "Get true event count"
    events[i] = scan.getNTrigs();
  }
  scanResult.nEvents = events;

  // The error counts
  UINT32 *errors = new UINT32[nBins];
  for(int i=0; i<nBins; i++) {
#warning "Get real error count (if possible!)"
    errors[i] = 0;
  }
  scanResult.nErrorEvents = errors;

  // The data!
  //  int binSize = 0x800*(scan.getOption(Scan::BITS32)?4:2);   // In bytes, so works for 16bit histograms
  int binSize = 0x800*4;    // In bytes, so works for 16bit histograms

  char *data = new char[nBins * binSize];

  int nModules = moduleMap.size();

  bool moduleBlock = false;
//    int baseAddress = index*(binSize*nBins);

  unsigned int slaveNumber = 0;

  if(scan.getOption(Scan::DISTSLAVE) == 1) {
    unsigned int partition, crate, rod, channel;
    getpcrc(mid, partition, crate, rod, channel);
    for(int i=0; i<4; i++) {
      if(channel<32) {
        if(ex.dspChannels[i][0] & 1<<channel) {
          slaveNumber = i;
        } else if(ex.dspChannels[i][1] & 1<<(channel-32)) {
          slaveNumber = i;
        }
      }
    }

    // Find the module index on this slave
    index = 0;
    nModules = 0;

    for(unsigned int i=0; i<48; i++) {
      if(channel == i) {
        index = nModules;
      }
      if(i<32) {
        if(ex.dspChannels[slaveNumber][0] & 1<<i) {
          nModules ++;
        }
      } else {
        if(ex.dspChannels[slaveNumber][1] & 1<<(i-32)) {
          nModules ++;
        }
      }
    }
  } else if(scan.getOption(Scan::DISTSLAVE) == 2) {
    cerr << " ****** Well you don't expect me to add them all together do you??? *****\n";
#warning "But maybe I should any way"
  }

  unsigned long slaveAddress = 0x20d2000;

  if(moduleBlock) {
    slaveAddress += binSize * nBins;
  } else {
    slaveAddress += binSize * index;
  }

  bool error = false;

  for(int i=0; i<nBins; i++) {
    unsigned long *chunk = readSlaveDsp(ex.partition, ex.crate, ex.rod, slaveNumber, slaveAddress, binSize/4);
    unsigned long *buffer = (unsigned long *)&(data[i*binSize]);

    if(chunk) {
      for(int b=0; b<binSize/4; b++) {
        buffer[b] = chunk[b];
      }

      // Check End of bin xx Module xx
      int bin, module;
      int count = sscanf(&((char*)buffer)[binSize-32], "End of bin %d Module %d", &bin, &module);
      if(count == 2) {
//          printf("Found End of bin %d module %d\n", bin, module);
      } else {
        if(!error)
          printf("Found bad end of histogram bin (%25s)\n", &((char*)buffer)[binSize-32]);
        error = true;
      }
    } else {
      for(int b=0; b<binSize/4; b++) {
        buffer[b] = 0;
      }
    }

    if(moduleBlock) {
      slaveAddress += binSize;
    } else {
      slaveAddress += binSize * nModules;
    }
  }

  scanResult.data = data;

  return scanResult;
}

void SctApi::readHistogramsToFile(const Scan &scan, const ScanEx &ex) {
  const int BUFF_SIZE = 50;
  char *filename;

  time_t secs;
  secs = time(NULL);
  struct tm broke;
  broke = *(gmtime(&secs));

  int directoryLength;
  int bufferLength;

  const char *histoEnv = "SCT_SCRATCH_DIR";

  if(getenv(histoEnv)) {
    directoryLength = strlen(getenv(histoEnv)) + 1;
    bufferLength = BUFF_SIZE + directoryLength;
    filename = new char[bufferLength];
    strcpy(filename, getenv(histoEnv));
    filename[directoryLength - 1] = '/';
  } else {
    directoryLength = 0;
    bufferLength = BUFF_SIZE + directoryLength;
    filename = new char[bufferLength];
  }

  int index = 0;
  for(map<UINT32, ABCDModule>::const_iterator mi = moduleMap.begin();
      mi!=moduleMap.end();
      mi ++, index ++) {
    UINT32 mid = mi->first;
    string sn = convertToString(mid);

    cout << "Module:  " << mid << " " << sn << endl;

    int written = directoryLength;
    written += snprintf(filename + written, bufferLength - written, "Histogram_%s", sn.c_str());
    //                                                                    20021112143712
    strftime(filename + written, bufferLength - written, "_%Y%m%d%H%M%S.bin", &broke);

    scan_result_ptrs result = readHistogramData(scan, ex, mid, index);

    cout << "Writing histogram data to " << filename << endl;

    saveHistogramToFile(result, filename);

    cout << "done.\n";

    delete [] result.points;
    delete [] result.nEvents;
    delete [] result.nErrorEvents;
    delete [] (char*)result.data;
  }

  delete [] filename;
}

void SctApi::saveHistogramToFile(scan_result_ptrs histo, std::string filename) {
  ofstream histoout(filename.c_str(), ios::binary);

  //    assert(histo.header.length == sizeof(ScanHeader));
  histoout.write((char*)&histo.header, histo.header.length);

  //    assert(histo.header.pntEvents - histo.header.pntPoints == nBins * sizeof(FLOAT32));
  histoout.write((char*)histo.points, histo.header.pntEvents-histo.header.pntPoints);

  //    assert(histo.header.pntErrors - histo.header.pntEvents == nBins * sizeof(FLOAT32));
  histoout.write((char*)histo.nEvents, histo.header.pntErrors-histo.header.pntEvents);

  //    assert(histo.header.pntData - histo.header.pntErrors == nBins * sizeof(FLOAT32));
  histoout.write((char*)histo.nErrorEvents, histo.header.pntData-histo.header.pntErrors);

  //    assert(histo.header.size*2 == (nBins * 0x800 * 4));

  // header.size is a count in 16bit words!
  histoout.write((char*)histo.data, histo.header.size*2);
}

#ifdef USE_IS

void SctApi::readHistogramsToIS(const Scan &scan, const ScanEx &ex) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Read Histogram To IS (" << ex.partition << " " << ex.crate << " " << ex.rod << ")\n";
  }

  cout << "Writing histogram data to IS\n";

  int index = 0;
  for(map<UINT32, ABCDModule>::const_iterator mi = moduleMap.begin();
      mi!=moduleMap.end();
      mi ++, index ++) {
    UINT32 mid = mi->first;
    string sn = convertToString(mid);

    cout << "Module:  " << mid << " " << sn << endl;

    scan_result_ptrs result = readHistogramData(scan, ex, mid, index);

    try {
      SctData::ScanResultWriter::publish(result);
    } catch(Sct::IoException &e) {
      cout << "ScanResult publish failed:\n" << e.what() << endl;
      e.sendToMrs(MRS_ERROR);
    }
    delete [] (char*)result.data;
    delete [] result.points;
    delete [] result.nEvents;
    delete [] result.nErrorEvents;
  }
}

#endif

void SctApi::pollHistogramming(ScanEx &ex, int slave, int bins, int events, int timeout) {
  int partition = ex.partition;
  int crate = ex.crate;
  int rod = ex.rod;

  int currBin = 0;

  bool timedout = false;

  time_t start_time = time(0);

#if USE_IS
  if(m_isDict) {
    ISInfoInt binNumber(0);
    m_isDict->insert("SCTAPIServer.currentBin", binNumber);

    ISInfoInt maxNumber(bins-1);
    m_isDict->insert("SCTAPIServer.maxBin", maxNumber);
  }
#endif

  int lastBin = 0;
  int currEvent = 0;
  int lastEvent = 0;
  while(!timedout) {
    sleep(1);

    long reg = dspSingleRead(partition, crate, rod, 0x80000020, -1);
    // Check text buffers as side effect
    // Possibly use dspSingleRead? + tell MDSP!

    currBin = (reg & 0xff);
    if(currBin != lastBin) {
      lastBin = currBin;
      cout << "Next bin " << currBin << endl;
#if USE_IS
      if(m_isDict) {
        ISInfoInt binNumber(currBin);
        m_isDict->update("SCTAPIServer.currentBin", binNumber);
      }
#endif
      start_time = time(0);

      continue;
    }


    unsigned long *regs = readSlaveDsp(partition, crate, rod, slave, 0x8000002c, 1);
    if(!regs) {
#warning "MRS here"
      cout << "Not able to check slave DSP progress!\n";
      cout << "Histogram in undefined state!\n";
      return;
    }

    currEvent = (int) regs[0];

    if(currEvent != lastEvent) { 
      lastEvent = currEvent;
      cout << "Next event " << currEvent << " (bin " << currBin << ")\n";

      start_time = time(0);

      continue;
    }

    if((currBin == (bins - 1)) && (currEvent == events)) {
      cout << "Histogramming completed successfully\n";
      break;
    }

    if((time(0) - start_time) > timeout) {
      timedout = true;
      cout << "Histogram polling timeout\n";
      break;
    }
  }

#if USE_IS
  if(m_isDict) {
    m_isDict->remove("SCTAPIServer.currentBin");
    m_isDict->remove("SCTAPIServer.maxBin");
  }
#endif

//    if(!timedout) {
//      cout << "Histogramming completed successfully\n";
//    }
}

unsigned long *SctApi::readSlaveDsp(unsigned int partition, unsigned int crate, unsigned int rod,
                                    int s, int add, int words) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Read slave (" << s << ") memory 0x" << hex << add << dec << endl;
  }

  long *data = new long[sizeof(RW_SLAVE_MEMORY_IN)/4];

  RW_SLAVE_MEMORY_IN &rwPrim = *(RW_SLAVE_MEMORY_IN*)data;

#if (R_RW_SLAVE_MEMORY == 100)
  rwPrim.slaveNumber = s;
  rwPrim.readNotWrite = 1;
  rwPrim.slaveAddress = (UINT32 *)add;
  rwPrim.masterAddress = (UINT32 *)0xffffffff;
  rwPrim.numWords = words;
#else
#error "RW_SLAVE_MEMORY revision change"
#endif

  RodPrimList rwList(2);

  rwList.push_back(RodPrimitive(4 + sizeof(RW_SLAVE_MEMORY_IN)/4, 
                                1, RW_SLAVE_MEMORY, R_RW_SLAVE_MEMORY, data));

  sendPrimList(partition, crate, rod, &rwList);

  delete [] data;

  awaitResponse(partition, crate, rod, 1);

  unsigned long length;
  unsigned long *mem = getResponse(partition, crate, rod, length);

#warning "Possible mem leak  Who owns this data??"

  if(mem) {
    unsigned long *result = new unsigned long[words];
    copy(mem+8, mem+8+words, result);
    delete [] mem;
    return result;
  } else {
    return 0;
  }
}

void SctApi::writeRODRegister(unsigned int partition, unsigned int crate, unsigned int rod,
                              int reg, int off, int width, int value) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Write ROD register " << reg << endl;
  }

  long *data = new long[sizeof(RW_REG_FIELD_IN)/4];

  RW_REG_FIELD_IN &regPrim = *(RW_REG_FIELD_IN*)data;

#if (R_RW_REG_FIELD == 104)
  regPrim.registerID = reg;
  regPrim.offset = off;
  regPrim.width = width;
  regPrim.readNotWrite = 0;
  regPrim.dataIn = value;
#elif (R_RW_REG_FIELD == 103)
  regPrim.registerID = reg;
  regPrim.offset = off;
  regPrim.width = width;
  regPrim.readNotWrite = 0;
  regPrim.dataIn = value;
#else
#error "RW_REG_FIELD revision change"
#endif

  RodPrimList regList(2);

  regList.push_back(RodPrimitive(4 + sizeof(RW_REG_FIELD_IN)/4, 
                                 1, RW_REG_FIELD, R_RW_REG_FIELD, data));

  sendPrimList(partition, crate, rod, &regList);

  delete [] data;

  awaitResponse(partition, crate, rod, 1);
}

void SctApi::writeRODRegister(int reg, int off, int width, int value, RodPrimList &parentList) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "+Write ROD register " << reg << endl;
  }

  long *data = new long[sizeof(RW_REG_FIELD_IN)/4];

  RW_REG_FIELD_IN &regPrim = *(RW_REG_FIELD_IN*)data;

#if (R_RW_REG_FIELD == 103 || R_RW_REG_FIELD == 104)
  regPrim.registerID = reg;
  regPrim.offset = off;
  regPrim.width = width;
  regPrim.readNotWrite = 0;
  regPrim.dataIn = value;
#else
#error "RW_REG_FIELD revision change"
#endif

  parentList.push_back(RodPrimitive(4 + sizeof(RW_REG_FIELD_IN)/4, 
                                    1, RW_REG_FIELD, R_RW_REG_FIELD, data));

#warning "Memory leak Didn't delete data!"
}

void SctApi::calib_init() {
  UINT32 txchannels[2];
  UINT32 rxchannels[4];

  txchannels[0] = 0;
  txchannels[1] = 0;
  rxchannels[0] = 0;
  rxchannels[1] = 0;
  rxchannels[2] = 0;
  rxchannels[3] = 0;

  // Build channel map for modules
  for(map<UINT32, ABCDModule>::const_iterator mi = moduleMap.begin();
      mi!=moduleMap.end();
      mi ++) {
    UINT32 currMid = mi->first;

    cout << "Set up module " << currMid << endl;

    unsigned int newPartition, newCrate, newRod;
    // If requires look up redundant channel instead
#warning "How is off-ROD reduncany handled?..."

    unsigned int rcvChannel, sndChannel;
    getpcrc(currMid, newPartition, newCrate, newRod, rcvChannel);

    if(lookupConfig(currMid)->select == 1) {
      unsigned int redPartition, redCrate, redRod;
      getrpcrc(currMid, redPartition, redCrate, redRod, sndChannel);
    } else {
      sndChannel = rcvChannel;
    }

    char *mappings = config->getFibreMappings(newPartition, newCrate, newRod);

    int txChannel = mappings[sndChannel * 3 + 0];
    int rx0Channel = mappings[rcvChannel * 3 + 1];
    int rx1Channel = mappings[rcvChannel * 3 + 2];

    int txi, txval, rx0i, rx0val, rx1i, rx1val;

    if(txChannel<32) { txi = 0; txval = 1<<txChannel; }
    else { txi = 1; txval = 1<<(txChannel-32); }

    if(rx0Channel<32) { rx0i = 0; rx0val = 1<<rx0Channel; }
    else if(rx0Channel<64) { rx0i = 1; rx0val = 1<<(rx0Channel-32); }
    else if(rx0Channel<96) { rx0i = 2; rx0val = 1<<(rx0Channel-64); }
    else { rx0i = 3; rx0val = 1<<(rx0Channel-96); }

    if(rx1Channel<16) { rx1i = 0; rx1val = 1<<rx1Channel; }
    else if(rx1Channel<32) { rx1i = 1; rx1val = 1<<(rx1Channel-32); }
    else if(rx1Channel<64) { rx1i = 2; rx1val = 1<<(rx1Channel-64); }
    else { rx1i = 3; rx1val = 1<<(rx1Channel-96); }

    if(txchannels[txi] & txval) {
      cout << "Module already using this channel in this group\n";
    } else {
      txchannels[txi] |= txval;
    }

    if(rxchannels[rx0i] & rx0val) {
      cout << "Module already using this channel in this group\n";
    } else {
      rxchannels[rx0i] |= rx0val;
    }

    if(rxchannels[rx1i] & rx1val) {
      cout << "Module already using this channel in this group\n";
    } else {
      rxchannels[rx1i] |= rx1val;
    }

    delete [] mappings;
  }

  cout << "Calculated channel masks:\n";
  cout << hex;
  cout << "tx: 0x" << txchannels[0] << " 0x" << txchannels[1] << endl;
  cout << "rx: 0x" << rxchannels[0] << " 0x" << rxchannels[1] << " 0x" << rxchannels[2] << " 0x" << rxchannels[3] << endl;
  cout << dec;

  // Clear router transfer buffer
  //      80008000 2000 words of 0

  RodPrimList calibList(1);

  // Set link static mask on all inputs
  for(int a=0x60; a<=0xbf; a++) {
    writeRODRegister(a, 0, 4, 0x01, calibList);
  }

  // Mask FE Occ counters
  writeRODRegister(0x1df, 0, 32, 0xffffffff, calibList);
  writeRODRegister(0x1e0, 0, 32, 0xffffffff, calibList);
  writeRODRegister(0x1e1, 0, 32, 0xffffffff, calibList);

  // Set RRIF cmd 0&1
  writeRODRegister(0x1c6, 0, 8, 0xff, calibList);    // Reset everything
  writeRODRegister(0x1c6, 0, 8, 0x78, calibList);
  writeRODRegister(0x1c7, 0, 23, 0x0494a9, calibList);

  writeRODRegister(0x1c7, 22, 1, 1, calibList);     // Set static L1

  // Read register 1c7

  writeRODRegister(0x229, 0, 24, 1, calibList);     // Set CAL_L1_ID_0
  writeRODRegister(0x22a, 0, 24, 1, calibList);     // Set CAL_L1_ID_0

  // Set and load output masks
  writeRODRegister(0x1ca, 0, 32, txchannels[0], calibList);
  writeRODRegister(0x1cb, 0, 16, txchannels[1], calibList);
  writeRODRegister(0x1cc, 0, 32, 0, calibList);   // XXX Do we want to use serial port 2
  writeRODRegister(0x1cd, 0, 16, 0, calibList);
  writeRODRegister(0x1c7, 2, 1, 1, calibList);    // Load mask

  // Load default mode bits
  for(int a=0x1ec; a<=0x1f3; a++) {
    writeRODRegister(a, 0, 16, 0, calibList);
  }

  // Load default dynamic mask
  for(int a=0x20c; a<=0x218; a++) {
    writeRODRegister(a, 0, 16, 0, calibList);
  }

  // Set header/trailer limit on all formatters
  for(int a=0xd0; a<=0xd7; a++) {
    writeRODRegister(a, 0, 8, 0xf0, calibList);
  }
  // Set ROD busy limit on all formatters
  for(int a=0xd8; a<=0xdf; a++) {
    writeRODRegister(a, 0, 8, 0xfc, calibList);
  }

  // Normal data path
  writeRODRegister(0x1c7, 0x19, 3, 0x4, calibList);

  // Set Links to condensed mode
  for(int i=0; i<96; i++) {
    if(rxchannels[i/32] & (1 << i%32)) {
      writeRODRegister(0x60+i, 0, 4, 2, calibList);
    }
  }

  // Set input masks
  writeRODRegister(0x1df, 0, 32, ~rxchannels[0], calibList);
  writeRODRegister(0x1e0, 0, 32, ~rxchannels[1], calibList);
  writeRODRegister(0x1e1, 0, 32, ~rxchannels[2], calibList);

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Send calib_init list\n";
  }

  // Send calibList
#warning "Only ROD 0!!!"
  // For each ROD (each ROD is different!)
  sendPrimList(0, 0, 0, &calibList);
  awaitResponse(0, 0, 0, 10);

  if(responseCode != 0) {
    cout << "Calibration setup unsuccessful\n";
  }
}

void SctApi::sendTriggerAll(const Trigger *trig) {
#warning "No sendTriggerAll"
}

void SctApi::sendTrigger(unsigned int partition, unsigned int crate, unsigned int rod, const Trigger *trig) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Send Trigger\n";
  }

  long *data = new long[sizeof(BUILD_STREAM_IN)/4];

  BUILD_STREAM_IN &buildPrim = *(BUILD_STREAM_IN*)data;

#if (R_BUILD_STREAM == 102)
  Trigger::RODTriggers points = trig->getRODTriggers();

  for(unsigned int i=0; i<points.size(); i++) {
    buildPrim.cmdList.cmd[i] = points[i].first;        // Set a trigger sequence
    buildPrim.cmdList.data[i] = points[i].second;      // Set a trigger sequence
  }

  buildPrim.cmdBuff = 0;
  buildPrim.reset = 1;
  buildPrim.chip = 63;
  buildPrim.fibre = 3;
  buildPrim.dataLen = 0;
  buildPrim.data = 0;    // Should be ignored?
 
  RodPrimList sendList(3);

  sendList.push_back(RodPrimitive(4 + sizeof(BUILD_STREAM_IN)/4, 
                                  2, BUILD_STREAM, R_BUILD_STREAM, data));
#elif R_BUILD_STREAM == 101
#error "Build stream not compiled (unsupported old version)" 
#else   // Unknown R_BUILD_STREAM
#error "Build stream not compiled (unknown primitive version, check primParams.h)" 
#endif

  long *sendData = new long[sizeof(SEND_STREAM_IN)/4];

  SEND_STREAM_IN &sendPrim = *(SEND_STREAM_IN*)sendData;

  sendPrim.cmdBuff = 2;
  sendPrim.captureSerOn = 0;

  sendList.push_back(RodPrimitive(4 + sizeof(SEND_STREAM_IN)/4, 
                                  2, SEND_STREAM, R_SEND_STREAM, sendData));

  sendPrimList(partition, crate, rod, &sendList);

  delete [] sendData;
  delete [] data;

  awaitResponse(partition, crate, rod, 2);
}

void SctApi::setupRawData(unsigned int partition, unsigned int crate, unsigned int rod, 
                          int delay, int units, bool setMask) {
  RodPrimList setupList(2);

  // Reset INMEM fifo
  writeRODRegister(0x1c6, 3, 1, 1, setupList);
  writeRODRegister(0x1c6, 3, 1, 0, setupList);

  if(setMask) {
    // Disable formatters
    for(int i=0; i<96; i++) {
#warning "These two are the same (formatters)"
      writeRODRegister(0x60 + i, 0, 4, 0x1, setupList);
    }
  }

  // Enable FE Command Outputs with SP0
  writeRODRegister(0x1c7, 0, 2, 0x1, setupList);

  if(setMask) {
    // Load masks
    writeRODRegister(0x1ca, 0, 32, 0xffffffff, setupList);
    writeRODRegister(0x1cb, 0, 16, 0xffff, setupList);
    writeRODRegister(0x1cc, 0, 32, 0xffffffff, setupList);
    writeRODRegister(0x1cd, 0, 16, 0xffff, setupList);
  }

  unsigned long val = (delay << 16) + units;

  writeRODRegister(0x1d5, 0, 32, val, setupList);

  // Load enable and load
  writeRODRegister(0x1c7, 20, 1, 0x1, setupList);
  writeRODRegister(0x1c7,  2, 1, 0x1, setupList);

  // Configuration read back mode
  writeRODRegister(0x1c7, 25, 3, 0x7, setupList);

  if(setMask) {
    // Enable formatters
    for(int i=0; i<96; i++) {
#warning "These two are the same (formatters)  I've left this the same as the previous link setup!"
      writeRODRegister(0x60 + i, 0, 4, 0x1, setupList);
    }
  }

  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "\"Set up raw data\"\n";
  }
  sendPrimList(partition, crate, rod, &setupList);
  awaitResponse(partition, crate, rod, 10);

  // Do something that might trigger a response
  sendL1A(partition, crate, rod);

  // Poll "Configuration read back" done
//    cout << "Status before poll " << hex << readRODRegister(partition, crate, rod, 0x1c9) << dec << endl;
  pollRegister(partition, crate, rod, 0x1c9, 3, 1, 1);
//    cout << "Status after poll " << hex << readRODRegister(partition, crate, rod, 0x1c9) << dec << endl;

  int status = readRODRegister(partition, crate, rod, 0x1d7);
  cout << "Inmem status: " << hex << status << dec << endl;

  // Set MUX to read out
  writeRODRegister(partition, crate, rod, 0x1c7, 25, 3, 0x1);
}

/// Is there a hit at a position
static int lookupFifoHit(UINT16 *bufA, UINT16 *bufB,
                         int channel, int pos) {
  int word = (channel / 16) % 3;
  int bit = channel % 16;
  if(channel<48) {
    if(bufA)
      return ((bufA[pos*3 + word] >> bit) & 0x1);
  } else {
    if(bufB)
      return ((bufB[pos*3 + word] >> bit) & 0x1);
  }
  return 0;
}

void SctApi::rawData(unsigned int partition, unsigned int crate, unsigned int rod, 
                     int delay, int units, bool setMask) {
  setupRawData(partition, crate, rod, delay, units, setMask);
  
  unsigned long *bufferA = readFifo(partition, crate, rod, 0, 0, units);
  unsigned long *bufferB = readFifo(partition, crate, rod, 0, 1, units);

  for(int ch=0; ch<96; ch++) {
    cout << "Ch ";
    cout.width(2);
    cout << ch << ": ";
    for(int i=0; i<units; i++) {
      // Data is in sets of three 16bit words 
      cout.width(1);
      cout << lookupFifoHit((UINT16*)bufferA, (UINT16*)bufferB, ch, i);
    }
    cout << endl;
  }

  delete [] bufferA;
  delete [] bufferB;

  if(setMask) {
    // Restore to how it was
    calib_init();
  }
}

// Could add do all modules option but then where would the data go?!
void SctApi::doRawScan(Scan &scan, int delay, bool clkBy2) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "doRawScan\n";
  }

  scan.setRunNumber(runNumber);
  scan.setScanNumber(scanNumber);

  scan.print();

  const int units = 768;

#warning "Raw scan only ROD 0"
  int partition = 0;
  int crate = 0;
  int rod = 0;

  if(!getCrate(partition, crate)->checkBOCLasersOn(rod)) {
    cout << "Continuing raw scan using BOC that has its lasers cut out\n";
    if(mrs) {
      *mrs << "BOC_INTERLOCKED" << MRS_WARNING << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<int>("crate", crate) 
           << MRS_PARAM<int>("rod", rod)
           << MRS_TEXT("Continuing raw scan (BOC interlocked)") << ENDM;
    }
  }

  int nBins = scan.getScanPoints1().size();

  map<UINT32, UINT32 *> dataMap;

  for(map<UINT32, ABCDModule> ::const_iterator iter = moduleMap.begin();
      iter != moduleMap.end();
      iter ++) {
    UINT32 *rawData = new UINT32[nBins * 768 * 2];

    for(int i=0; i<nBins*768*2; i++) {
      rawData[i] = 0;
    }

    dataMap.insert(make_pair(iter->first, rawData));
  }

  char *mappings;

  try { 
    mappings = config->getFibreMappings(partition, crate, rod);
  } catch(ConfigurationException &c) {
    cout << "No fibre mappings for appropriate module: \n";
    cout << c.what() << endl;

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "  (Not found)\n";
    }

    if(mrs) {
      *mrs << "SCAN_ABORT" << MRS_ERROR 
           << MRS_QUALIF("SCTAPI") << MRS_QUALIF("doRawScan") 
           << MRS_TEXT("Aborting scan due to missing fibre mappings") 
           << ENDM;
    }
    return;
  }

  if(clkBy2)
    getCrate(partition, crate)->enterBOCClockBy2Mode(rod);

  /**** Start scan ****/

  int bin = 0;
  vector<FLOAT32> scanPoints = scan.getScanPoints1();

  for(vector<FLOAT32>::const_iterator point_iter = scanPoints.begin();
      point_iter != scanPoints.end();
      point_iter++, bin++) {

    cout << "Bin: " << bin << endl;

    modifyABCDVar(scan.getScanVariable1(), *point_iter);
    //partition, crate, rod, 0, 0, bin);

    for(unsigned int t=0; t<scan.getNTrigs(); t++) {
      setupRawData(partition, crate, rod, delay, units, false);

      auto_ptr<unsigned long> bufferA(readFifo(partition, crate, rod, 0, 0, units));
      auto_ptr<unsigned long> bufferB(readFifo(partition, crate, rod, 0, 1, units));

      if(!bufferA.get() || !bufferB.get()) {
        cout << "Failed to read FIFO\n";
        return;
      }

      for(map<UINT32, ABCDModule> ::const_iterator iter = moduleMap.begin();
          iter != moduleMap.end();
          iter ++) {

        UINT32 mid = iter->first;

        unsigned int partition, crate, rod, channel;
        getpcrc(mid, partition, crate, rod, channel);

        UINT32 *rawData = dataMap.find(mid)->second;

        if(!rawData) {
          cout << "Can't find preallocated memory!\n";
          return;
        }

        for(int link=0; link<2; link++) {
          int fibre = mappings[channel * 3 + link + 1];  // Looking for offsets 1 and 2

          for(int i=0; i<units; i++) {
            int val = lookupFifoHit((UINT16*)bufferA.get(), (UINT16*)bufferB.get(), fibre, i);
            rawData[bin * (units*2) + (768 * link) + i] += val;
          }
        }
      }
    }
  }

  delete [] mappings;

  /**** Finished scan ****/

  if(clkBy2)
    getCrate(partition, crate)->leaveBOCClockBy2Mode(rod);


  // Mostly the same data, just copy in the FIFOs for each module
  scan_result_ptrs scanResult;

  ScanHeader &header = scanResult.header;

  // Fill in common data
  header.version = 0;
  header.length = sizeof(ScanHeader);
  header.runNumber = runNumber;
  header.scanNumber = scanNumber;
  header.scanType = scan.getScanVariable1(); 
  header.npoints = nBins;
  header.size = nBins * 2 * 768 * 2;
  header.dataType = SR_DT_SLICE_COMPRESSED;    // SR_DT_RAWHIST
#warning "Gratuitous memory hog!"
  header.width = SR_WD_32;
  header.pntPoints = sizeof(ScanHeader);
  header.pntEvents = header.pntPoints + nBins * sizeof(FLOAT32);
  header.pntErrors = header.pntEvents + nBins * sizeof(UINT32);
  header.pntData = header.pntErrors + nBins * sizeof(UINT32);

  // The points
  FLOAT32 *points = new FLOAT32[nBins];
  for(int i=0; i<nBins; i++) {
    points[i] = scan.getScanPoints1()[i];
  }
  scanResult.points = points;

  // The event counts
  UINT32 *events = new UINT32[nBins];
  for(int i=0; i<nBins; i++) {
    events[i] = scan.getNTrigs();
  }
#warning "Could be counted?"
  scanResult.nEvents = events;

  // The error counts     
#warning "What errors should be recorded?"
  //  (If complete failure then no histogram generated)
  UINT32 *errors = new UINT32[nBins];
  for(int i=0; i<nBins; i++) {
    errors[i] = 0;           // Of course! 
  }
  scanResult.nErrorEvents = errors;

  // Use the same time for all modules
  time_t secs;
  secs = time(NULL);
  struct tm broke;
  broke = *(gmtime(&secs));

  //  for each module
  for(map<UINT32, ABCDModule> ::const_iterator iter = moduleMap.begin();
      iter != moduleMap.end();
      iter ++) {

    // Fill in module specific information

    strncpy(header.moduleName, convertToString(iter->first).c_str(), 16);
    header.moduleName[15] = 0;                        // Null terminate the string
    header.config = iter->second;

    scanResult.data = dataMap.find(iter->first)->second;

    // What to do with it now!

    const int BUFF_SIZE = 50;
    char filename[BUFF_SIZE];

    int written = snprintf(filename, BUFF_SIZE, "RawHistogram_%s", convertToString(iter->first).c_str());
    //                                                  20021112143712
    strftime(filename + written, BUFF_SIZE - written, "_%Y%m%d%H%M%S.bin", &broke);

    cout << "Saving raw histogram: " << filename << endl;
    saveHistogramToFile(scanResult, filename);

    delete [] dataMap.find(iter->first)->second;
    // Make sure its not used again (duplicate module in moduleMap!)
    dataMap.find(iter->first)->second = 0;
  }

  delete [] scanResult.points; 
  delete [] scanResult.nEvents;
  delete [] scanResult.nErrorEvents;

  // Delete any unused memory
  for(map<UINT32, UINT32 *> ::const_iterator iter = dataMap.begin();
      iter != dataMap.end();
      iter ++) {
    if(iter->second) {
      delete [] iter->second;
    }
  }

  if(debugLevel > 3)
    printBOCSetup(partition, crate, rod);

  getCrate(partition, crate)->configureBOC(rod);
}

void SctApi::readRawData(unsigned int partition, unsigned int crate, unsigned int rod, 
                         int delay, bool setMask) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Raw histogram of all ROD data\n";
  }

  int units = 768 * 2;

  scan_result_ptrs scanResult;

  // When the scan started
  time_t secs;
  secs = time(NULL);
  struct tm broke;
  broke = *(gmtime(&secs));

  int nBins = 96; // scan.getScanPoints1().size();

#warning "Assuming module 0 is configured!!!"
  ABCDModule &moduleConfig = *lookupConfig(0);

  ScanHeader &header = scanResult.header;

  header.version = 0;
  header.length = sizeof(ScanHeader);
  header.runNumber = runNumber;
  header.scanNumber = scanNumber;
  strncpy(header.moduleName, "raw link data", 16);
  header.moduleName[15] = 0;                        // Null terminate the string
  header.scanType = 0; 
  header.npoints = 96;
  header.size = nBins * 2 * 1024 * 2;
#warning "Need new type for this"
  header.dataType = SR_DT_SLICE;

#warning  "Gratuitous memory hog! (2)"
  header.width = SR_WD_32;

  header.config = moduleConfig;

  header.pntPoints = sizeof(ScanHeader);
  header.pntEvents = header.pntPoints + nBins * sizeof(FLOAT32);
  header.pntErrors = header.pntEvents + nBins * sizeof(UINT32);
  header.pntData = header.pntErrors + nBins * sizeof(UINT32);

  // The points
  FLOAT32 *points = new FLOAT32[nBins];
  for(int i=0; i<nBins; i++) {
    points[i] = i;
  }
  scanResult.points = points;

  // The event counts
  UINT32 *events = new UINT32[nBins];
  for(int i=0; i<nBins; i++) {
    events[i] = 1;
  }
  scanResult.nEvents = events;

  // The error counts
  UINT32 *errors = new UINT32[nBins];
  for(int i=0; i<nBins; i++) {
#warning "More 0 errors"
    errors[i] = 0;
  }
  scanResult.nErrorEvents = errors;

  // The data!
  int binSize = 0x800*4;   // In bytes, so works for 16bit histograms

  setupRawData(partition, crate, rod, delay, units, setMask);

  unsigned long *bufferA = readFifo(partition, crate, rod, 0, 0, units);
  unsigned long *bufferB = readFifo(partition, crate, rod, 0, 1, units);

  char *data = new char[nBins * binSize];

  unsigned long *longData = (unsigned long *)data;

  for(int ch=0; ch<96; ch++) {
    for(int i=0; i<units; i++) {
      int val = lookupFifoHit((UINT16*)bufferA, (UINT16*)bufferB, ch, i);
      longData[ch * (units) + i] = val;
    }
    cout << endl;
  }

  delete [] bufferA;
  delete [] bufferB;

  scanResult.data = data;

  // Save histogram to file
  const int BUFF_SIZE = 50;
  char filename[BUFF_SIZE];

  int written = snprintf(filename, BUFF_SIZE, "RawHistogramFull");
  //                                                  20021112143712
  strftime(filename + written, BUFF_SIZE - written, "_%Y%m%d%H%M%S.bin", &broke);

  cout << "Saving raw histogram: " << filename << endl;

  saveHistogramToFile(scanResult, filename);
}
 
vector<char> SctApi::probe(unsigned int partition, unsigned int crate, unsigned int rod, signed int harness) {
  int delay = 5;
  int units = 20;

  int startCh = 0;
  int endCh = 47;

  if(harness != -1) {
    startCh = harness*6;
    endCh = harness*6 + 5;
  }

  // Each channel has two links!
  vector<char> result(((endCh+1) - startCh) * 2);

  if(!getCrate(partition, crate)->checkBOCLasersOn(rod)) {
    cout << "Continuing probe using BOC that has its lasers cut out\n";
    if(mrs) {
      *mrs << "BOC_INTERLOCKED" << MRS_WARNING << MRS_QUALIF("SCTAPI") 
           << MRS_PARAM<int>("crate", crate) 
           << MRS_PARAM<int>("rod", rod)
           << MRS_TEXT("Continuing probe (BOC interlocked)") << ENDM;
    }
  }

  // Always set up masks first
  setupRawData(partition, crate, rod, delay, units, true);

  unsigned long *bufferA = readFifo(partition, crate, rod, 0, 0, units);
  unsigned long *bufferB = readFifo(partition, crate, rod, 0, 1, units);

  if(!bufferA && !bufferB) {
    cout << "No data refusing to analyse probe!\n";
    return vector<char>();
  }

  char *data = new char[units];

  for(int index=0; index<(((endCh+1) - startCh)*2); index++) {
    int ch = index + startCh*2;
    bool same = true;
    bool clock = true;
    signed char lastValue = -1;

    int headerState = 0;

    for(int i=0; i<units; i++) {
      int val = lookupFifoHit((UINT16*)bufferA, (UINT16*)bufferB, ch, i);

      if(lastValue == -1) {   // First time around
        lastValue = val;
      } else if(same && lastValue != val) {
        same = false;
      } else if(clock && lastValue == val) {
        clock = false;
      }

      lastValue = val;

      data[i] = val;

      switch(headerState) {
      case 0:
        if(val == 1) headerState = 1;
        break;
      case 1:  // 1xxxxxx
        if(val == 0) headerState = 10; // 10 
        else headerState = 2;          // 11
        break;
      case 2:  // 11xxxxx
        if(val == 0) headerState = 20; // 110 
        else headerState = 3;          // 111
        break;
      case 3:  // 111xxxx
        if(val == 0) headerState = 4;  // 1110 
        else headerState = 99;         // junk
        break;
      case 4:  // 1110xxx
        if(val == 1) headerState = 21; // 11101 - event header
        else headerState = 99;         // junk
        break;
      case 10: // 10xxxxx
        if(val == 1) headerState = 11; // 101 
        else headerState = 99;         // junk
        break;
      case 11: // 101xxxx
        if(val == 0) headerState = 12; // 1010 
        else headerState = 99;         // junk
        break;
      case 12: // 1010xxx
        if(val == 1) headerState = 13; // 10101 
        else headerState = 15;         // 10100
        break;
      case 13: // 10101xx
        if(val == 0) headerState = 14; // 101010 
        else headerState = 99;         // junk
        break;
      case 14: // 10101xx
        if(val == 0) headerState = 22; // 1010100 - soft reset 
        else headerState = 99;         // junk
        break;
      case 15: // 10100xx
        if(val == 1) headerState = 16; // 101001 
        else headerState = 99;         // junk
        break;
      case 16: // 10100xx
        if(val == 0) headerState = 23; // 1010010 - BC reset 
        else headerState = 99;         // junk
        break;
      case 20: // L1A 110
      case 22: // SR  1010100
      case 23: // BCR 1010010
        // Any more "1" and we assume junk
        if(val == 1) headerState = 99;
        break;
      default:
        // including valid event header or junk
        // - do nothing 
        break;
      }
    }

    cout.width(2);
    cout << (ch/2) << " (" << ch%2 << ") ";

    if(same) {
      result[index] = int(lastValue) + '0';
      cout << "All " << int(lastValue) << "'s\n";
    } else if(clock) {
      result[index] = '2';
      cout << "Clock by 2\n";
    } else {
      switch(headerState) {
      case 20:
        result[index] = 'L';
        cout << "L1A\n";
        break;
      case 21:
        result[index] = 'E';
        cout << "Event Header\n";
        break;
      case 22:
        result[index] = 'S';
        cout << "Soft Reset\n";
        break;
      case 23:
        result[index] = 'B';
        cout << "BC Reset\n";
        break;
      default:
        result[index] = 'J';
        cout << "Junk\n";
        break;
      }
      cout << "       ";
      for(int i=0; i<units; i++) {
        cout << " " << int(data[i]);
      }
      cout << endl;
    }
  }

  delete [] bufferA;
  delete [] bufferB;

  delete [] data;

  return result;
}
  
void SctApi::sendL1A(unsigned int partition, unsigned int crate, unsigned int rod) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Send L1A\n";
  }

  long *data = new long[sizeof(BUILD_STREAM_IN)/4];

  BUILD_STREAM_IN &buildPrim = *(BUILD_STREAM_IN*)data;

#if (R_BUILD_STREAM == 102)
  for(int i=0; i<6; i++) {
    buildPrim.cmdList.cmd[i] = 0x64;
    buildPrim.cmdList.data[i] = 0;
  }

  buildPrim.cmdList.cmd[0] = 0x65;

  buildPrim.cmdBuff = 0;
  buildPrim.reset = 1;
  buildPrim.chip = 63;
  buildPrim.fibre = 3;
  buildPrim.dataLen = 0;
  buildPrim.data = 0;    // Should be ignored?
 
  RodPrimList sendList(3);

  sendList.push_back(RodPrimitive(4 + sizeof(BUILD_STREAM_IN)/4, 
                                  2, BUILD_STREAM, R_BUILD_STREAM, data));
#elif R_BUILD_STREAM == 101
#error "Build stream not compiled (unsupported old version)" 
#else   // Unknown R_BUILD_STREAM
#error "Build stream not compiled (unknown primitive version, check primParams.h)" 
#endif

  long *sendData = new long[sizeof(SEND_STREAM_IN)/4];

  SEND_STREAM_IN &sendPrim = *(SEND_STREAM_IN*)sendData;

  sendPrim.cmdBuff = 2;
  sendPrim.captureSerOn = 0;

  sendList.push_back(RodPrimitive(4 + sizeof(SEND_STREAM_IN)/4, 
                                  2, SEND_STREAM, R_SEND_STREAM, sendData));

  sendPrimList(partition, crate, rod, &sendList);

  delete [] sendData;
  delete [] data;

  awaitResponse(partition, crate, rod, 2);
}

unsigned int SctApi::readRODRegister(unsigned int partition, unsigned int crate, unsigned int rod, int r) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Read ROD reg\n";
  }
  //    long d[] = {r, 0, 32, 1, 0};

  long *data = new long[sizeof(RW_REG_FIELD_IN) / 4];

  RW_REG_FIELD_IN &readPrim = *(RW_REG_FIELD_IN*)data;

#if (R_RW_REG_FIELD == 103 || R_RW_REG_FIELD == 104)
  readPrim.registerID = r;
  readPrim.offset = 0;
  readPrim.width = 32;
  readPrim.readNotWrite = 1;
  readPrim.dataIn = 0;
#else
#error "RW_REG_FIELD revision change"
#endif


  RodPrimList readList(2);
 
  readList.push_back(RodPrimitive(4 + sizeof(RW_REG_FIELD_IN)/4,
                                  1, RW_REG_FIELD, R_RW_REG_FIELD, data));

  sendPrimList(partition, crate, rod, &readList);

  delete [] data;

  awaitResponse(partition, crate, rod, 2);

  unsigned long length;
  unsigned long *result = getResponse(partition, crate, rod, length);
  unsigned int val = 0;
  if(result) {
    val = result[8];

    delete [] result;
  } else {
    cout << "ReadRegister " << hex << r << dec << " failed\n";
  }

  return val;
}

int SctApi::pollRegister(unsigned int partition, unsigned int crate, unsigned int rod, 
                         int r, int off, int width, int val, int timeout) {
  {
    boost::mutex::scoped_lock lock(logMutex);
    log << "Poll ROD register\n";
  }

  // Timeout in 1000000 usec
  //    long poll[] = {r, off, width, val, timeout};

  long *data = new long[sizeof(POLL_REG_FIELD_IN) / 4];

  POLL_REG_FIELD_IN &pollPrim = *(POLL_REG_FIELD_IN*)data;

#if (R_RW_REG_FIELD == 103 || R_RW_REG_FIELD == 104)
  pollPrim.registerID = r;
  pollPrim.offset = off;
  pollPrim.width = width;
  pollPrim.desiredValue = val;
  pollPrim.timeoutInUsec = timeout;
#else
#error "RW_REG_FIELD revision change"
#endif

  RodPrimList pollList(2);
 
  pollList.push_back(RodPrimitive(4 + sizeof(POLL_REG_FIELD_IN)/4,
                                  1, POLL_REG_FIELD, R_POLL_REG_FIELD, data));

  sendPrimList(partition, crate, rod, &pollList);

  delete [] data;

  awaitResponse(partition, crate, rod, 5);

  unsigned long length;

  unsigned long *response = getResponse(partition, crate, rod, length);

  int result = 0;

  if(response) {
    if(response[8] == 1) {
      cout << "Success\n";
      result = 1;
    } else {
      cout << "Failure (timeout)\n";
      result = 0;
    }

    delete [] response;
  }

  return result;
}

unsigned long *SctApi::readFifo(unsigned int partition, unsigned int crate, unsigned int rod, int id, int bank, int elems) {
  {
    boost::mutex::scoped_lock lock(logMutex); 
    log << "readFIFO\n";
  }

  long *data = new long[sizeof(RW_FIFO_IN) / 4];

  RW_FIFO_IN &fifoPrim = *(RW_FIFO_IN*)data;

#if (R_RW_FIFO == 104)
  fifoPrim.fifoId = id;
  fifoPrim.bank = bank;
  fifoPrim.readNotWrite = 1;
  fifoPrim.numElements = elems;
  fifoPrim.dataBaseAdr = (UINT32 *)0xffffffff;
#else
#error "RW_FIFO revision change"
#endif

  RodPrimList fifoList(2);
 
  fifoList.push_back(RodPrimitive(4 + sizeof(RW_FIFO_IN)/4,
                                  1, RW_FIFO, R_RW_FIFO, data));

  sendPrimList(partition, crate, rod, &fifoList);

  delete [] data;

  awaitResponse(partition, crate, rod, 2);

  unsigned long length;
  unsigned long *response = getResponse(partition, crate, rod, length);

  unsigned long *result;
  if(response) {
    unsigned long myLength = length-9-2;

    result = new unsigned long[myLength];

    for(int i=0; i<myLength; i++) {
      result[i] = response[i+9];
    }
  } else {
    result = 0;
  }

  return result;
}

void SctApi::printABCDModule(int mid) {
  ABCDModule *mConf = lookupConfig(mid);

  if(mConf) {
    config->printModuleConfig(*mConf);
  } else {
    cout << "Invalid module id " << mid << endl;
  }
}

void SctApi::printABCDRodModule(int mid, int bank) {
  getABCDModule(mid, bank);

  ABCDModule *mConf = lookupConfig(mid);

  config->printModuleConfig(*mConf);
}

void SctApi::modifyBOCParam(unsigned int partition, unsigned int crate, unsigned int rod,
                            unsigned int channel, unsigned int type, unsigned int val) {
  getCrate(partition, crate)->modifyBOCParam(rod, channel, type, val);
}

void SctApi::printBOCSetup(unsigned int partition, unsigned int crate, unsigned int rod) {
  getCrate(partition, crate)->printBOCSetup(rod);
}

void SctApi::printBOCRegisters(unsigned int partition, unsigned int crate, unsigned int rod) {
  getCrate(partition, crate)->printBOCRegisters(rod);
}

void SctApi::lasersOff() {
  for(map<pair<unsigned int, unsigned int>, Crate* >::const_iterator ci = crateMap.begin();
      ci != crateMap.end();
      ci++) {
    ci->second->lasersOff();
  }
}

void SctApi::freeTriggers(unsigned int partition, unsigned int crate,
                          float freq) {
  SctPixelRod::TimMaskFrequency tFreq = SctPixelRod::TIM_MASK_FER_0_010HZ;

  // std:: needed because of ROOT (I think)
  int power = (int)std::log(freq);

  switch(power) {
  case -2: tFreq = SctPixelRod::TIM_MASK_FER_0_010HZ; break;
  case -1: tFreq = SctPixelRod::TIM_MASK_FER_0_100HZ; break;
  case  0: tFreq = SctPixelRod::TIM_MASK_FER_1_000HZ; break;
  case  1: tFreq = SctPixelRod::TIM_MASK_FER_10_00HZ; break;
  case  2: tFreq = SctPixelRod::TIM_MASK_TRIG_0_10KHZ; break;
  case  3: tFreq = SctPixelRod::TIM_MASK_TRIG_1_00KHZ; break;
  case  4: tFreq = SctPixelRod::TIM_MASK_TRIG_10_0KHZ; break;
  case  5: tFreq = SctPixelRod::TIM_MASK_TRIG_100_KHZ; break;
  }

  getCrate(partition, crate)->freeTriggers(tFreq);
}

void SctApi::stopTriggers(unsigned int partition, unsigned int crate) {
  RodPrimList timList(1);

  // Use TIM triggers
  writeRODRegister(0x1c7, 1, 1, 1, timList);
  // Enable TIM trigger event ID decoding
  writeRODRegister(0x1c7, 8, 1, 1, timList);

  sendPrimListAll(partition, crate, &timList);

  awaitResponseAll(partition, crate, 2);

  getCrate(partition, crate)->stopTriggers();
}

void SctApi::timL1A(unsigned int partition, unsigned int crate) {
  getCrate(partition, crate)->timL1A();
}

void SctApi::timCalL1A(unsigned int partition, unsigned int crate, int delay) {
  getCrate(partition, crate)->timCalL1A(delay);
}

void SctApi::decodeEvent(unsigned int partition, unsigned int crate, unsigned int rod, 
                         int sl, int index, bool extFlag, bool errorType) {
  const int eventSize = 0x400;
  long base;
  if(extFlag) 
    base = 0x2092000;
  else
    base = 0x80008000;

  //  long add = base + eventSize * index;
  const int eventWords = eventSize / 4;
  int frameCount = 0;

  // Storage for 32 frames
  UINT32 frameBuffer[eventWords * 32];

  while(frameCount < 16) {
    unsigned long *mem = readSlaveDsp(partition, crate, rod, sl, base + eventSize * ((index + frameCount) % 32), eventWords);

    if(mem) {
      for(int i=0; i<eventWords; i++) {
        frameBuffer[eventWords*frameCount + i] = mem[i];
      }

      frameCount ++;
    } else {
      break;
    }

    if((frameBuffer[eventWords*frameCount - 1] & 0xff0e0000) == 0x400e0000) {
      break;
    } 
  }

  if(frameCount == 0) {
    cout << "Couldn't load memory\n";
    return;
  } else {
    cout << "Found " << frameCount << " frames of event data\n";
  }

  if((frameBuffer[eventWords*frameCount - 1] & 0xff0e0000) != 0x400e0000) {
    cout << "Event trailer marker not found " << frameBuffer[eventWords*frameCount - 1] << endl;
    return;
  } 

  if(frameBuffer[0] == 0xb0f00000) { 
    cout << "Found header\n";
  } else {
    cout << "Bad header 0x" << hex << frameBuffer[0] << dec << endl;
    return;
  }

  if(frameBuffer[1] == 0xee1234ee) {
    cout << "Valid header\n";
  } else {
    cout << "Bad check 0x" << hex << frameBuffer[1] << dec << endl;
  }

  UINT32 headerLength;     // Including bof
  UINT32 trailerLength;    // Excluding eof

  if(errorType) {
    headerLength = 6;
    trailerLength = 8;

    cout << "L1ID = " << frameBuffer[2] << " BCID = " << frameBuffer[3] << endl;
    cout << "Trigger type = " << frameBuffer[4] << " Event type = " << frameBuffer[5] << endl;
  } else {
    headerLength = 9;
    trailerLength = 5;

    if(frameBuffer[2] != headerLength) {
      cout << "Unknown header length (" << frameBuffer[2] << ")\n";
    } 

    cout << "Version: " << frameBuffer[3] << " ID: " << frameBuffer[4] << endl;
    cout << "L1ID = " << frameBuffer[5] << " BCID = " << frameBuffer[6] 
         << " TType = " << frameBuffer[7] << " det type = " << frameBuffer[8] << endl;

  } 

  int eofAt = headerLength;

  while(frameBuffer[eofAt] != 0xe0f00000 && eofAt< (eventWords * frameCount)) eofAt ++;

  if(eofAt < (eventWords * frameCount)) {
    cout << "Found eof!\n";
  } else {
    cout << "EOF not found\n";
    return;
  }

  int eventLength = eofAt-headerLength-trailerLength;

  UINT32 *rawEvent = frameBuffer+headerLength;

  // Something different to the first word
  UINT16 lastWord = 1 + ((rawEvent[0] & 0xffff0000) >> 16);
  UINT32 repeats = 0;

  for(int i=0; i<eventLength * 2; i++) {
    UINT16 currWord;
    if(i&1) {
      currWord = rawEvent[i/2] & 0x00ffff;
    } else {
      currWord = (rawEvent[i/2] & 0xffff0000) >> 16;
    }

    if(currWord == lastWord) {
      repeats ++;
      continue;
    } else if(repeats) {
      cout << "        Repeated " << repeats << " times\n";
      repeats = 0;
    }

    lastWord = currWord;

    cout.width(4);
    cout.fill('0');
    cout << hex << currWord << dec;
    cout.fill(' ');

    switch((currWord & 0xe000) >> 13) {
    case 0:
      {
        if(currWord & 0x1f80) 
          cout << " INVALID" << endl;
        else {
          // Flagged error
          cout << " Flagged error on " << ((currWord & 0x78) >> 3) 
               << " (" << ((currWord & 0x7)) << ")\n";
        }
      }
      break;
    case 1:
      {
        // Header
        cout << " Header: ";
        if(errorType) {
          cout << "L1 " << ((currWord & 0x0f00) >> 8) 
               << " BCID " << ((currWord & 0xff)) 
               << ((currWord & 0x1000)?" Preamble err ":"")
               << endl;
        } else {
          cout << "Link " << (currWord & 0x7f) << " " 
             << ((currWord & 0x100)?"Condensed ":"") 
             << ((currWord & 0x200)?"BC err ":"") 
             << ((currWord & 0x400)?"L1 err ":"") 
             << ((currWord & 0x800)?"Time out err ":"") 
             << ((currWord & 0x1000)?"Preamble err ":"")
             << endl;
        }
      }
      break;
    case 2:
      {
        if(currWord & 0x3ff) 
          cout << " INVALID\n";
        else {
          // Trailer
          cout << " Trailer: " 
               << ((currWord & 0x400)?"Data overflow err ":"") 
               << ((currWord & 0x800)?"H/T limit err ":"") 
               << ((currWord & 0x1000)?"Trailer bit err ":"");
          cout << endl;
        }
      }
      break;
    case 3:
      {
        if(currWord & 0x300) 
          cout << " INVALID\n";
        else {
          // Raw data
          cout << " Raw: " 
               << ((currWord & 0x1c00) >> 10) + 1 << " bits " 
               << hex << (currWord & 0xff) << dec;
          cout << endl;
        }
      }
      break;
    default:
      // Everything else
      //          cout << " Cluster or condensed\n";
      {
        if((currWord & 0x2) == 0) {
          cout << "  Condensed: " ;
          cout << "Chip " << ((currWord & 0x7e00) >> 11)
               << " address 0x" << hex << ((currWord & 0x7f0) >> 4) << dec;
          if(currWord & 1) {
            cout << " hits " << ((currWord & 0x4)?"1":"0") 
                 << ((currWord & 0x8)?"1":"0");
          } else {
            cout << " hit " << ((currWord & 0x4)?"1":"0"); 
          }
          cout << endl;
        }

        if(currWord & 0x8 == 0) {
          cout << "  1st hit clust exp: " 
               << "Chip " << ((currWord & 0x7e00) >> 11)
               << " address 0x" << hex << ((currWord & 0x7f0) >> 4) << dec;
          cout << hex << " 0x" << (currWord & 0x7) << dec << endl;
        } else {
          if((currWord & 0x7f00) == 0) {
            cout << "  Clust exp: ";
            cout << hex << " 0x" << (currWord & 0x7) << dec;
            if(currWord & 0x80) {
              cout << hex << " 0x" << ((currWord & 0x70) >> 4) << dec;
            }
            cout << endl;
          }
        }
      }
    }
  }

  if(repeats) {
    cout << "        Repeated " << repeats << " times\n";
    repeats = 0;
  }

  cout << "Error count = " << frameBuffer[eofAt-trailerLength] << endl;
  cout << "Error flags = 0x" << hex << frameBuffer[eofAt-trailerLength + 1] << dec << endl;

  int flags = frameBuffer[eofAt-trailerLength + 1];
  if(flags != 0) {
    if(flags & 0x1) {
      cout << "HEADER ";
    }
    if(flags & 0x2) {
      cout << "TRAILER ";
    }
    if(flags & 0x4) {
      cout << "FLAGGED ";
    }
    if(flags & 0x8) {
      cout << "SYNC ";
    }
    if(flags & 0x10) {
      cout << "HIT PATTERN ";
    }
    if(flags & 0x20) {
      cout << "L1ID ";
    }
    if(flags & 0x40) {
      cout << "BCID ";
    }
    if(flags & 0x80) {
      cout << "TIMEOUT ";
    }
    if(flags & 0x100) {
      cout << "Almost Full ";
    }
    if(flags & 0x200) {
      cout << "OVERFLOW ";
    }
    cout << endl;
  }

  if(errorType) {
    cout << "Error Status: \n" << hex;

    for(int es = 0; es < 6; es++) {
      UINT16 esWord = frameBuffer[eofAt-trailerLength + 2 + es];
      for(int i=0; i<8; i++) {
        cout << (esWord & 0x3) << " ";
        esWord >>= 2;
      }
    }

    cout << dec << endl;
  } else {
    cout << "nData = " << frameBuffer[eofAt-2] << " words found = " << (eofAt-9-5) << endl;
  }

  cout << "EOF = 0x" << hex << frameBuffer[eofAt] << dec << endl;

  if(debugLevel > 1) {
    // Print out the raw data afterwards
    int wordsPerLine = 8;

    printMemoryBlock(cout, (long unsigned *)rawEvent, eventLength, wordsPerLine);

    cout << frameCount << " frames finished in frame " << (frameCount + index) << " with 0x";
    cout.fill('0'); cout.width(8);
    cout << hex << int(frameBuffer[eventWords*frameCount - 1]) << dec << endl;
    cout.fill(' ');
  }
}

void SctApi::printHex(UINT32 num, int width) {
  cout << "0x";
  char saveFill = cout.fill('0');
  cout.width(width);
  cout.setf(ios::right);
  cout << hex << num << dec;
  cout.fill(saveFill);
}

void SctApi::getrpcrc(UINT32 mid, unsigned int &rpartition, unsigned int &rcrate, unsigned int &rrod, unsigned int &rchannel) {
  unsigned int partition, crate, rod, channel;

  getpcrc(mid, partition, crate, rod, channel);

  unsigned int mur, module;

  try {
    if(debugLevel > 1)
      cout << "Looking for module in partition " << partition << " crate " << crate << " rod " << rod << " channel " << channel << endl;
    config->translateFromROD(partition, crate, rod, channel, 
                             mur, module);

    if(debugLevel > 2)
      cout << "Found MUR " << mur << " module number " << module << endl;
  } catch(ConfigurationException &c) {
    cout << "Can't find module in MUR map: \n";
    cout << c.what() << endl;

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "  (Not found)\n";
    }
    return;
  }

  try {
    config->translateToRROD(mur, module, 
                            rpartition, rcrate, rrod, rchannel);
  } catch(ConfigurationException &c) {
    cout << "Module redundancy not found: \n";
    cout << c.what() << endl;

    {
      boost::mutex::scoped_lock lock(logMutex);
      log << "  (Not found)\n";
    }
  }
}


void SctApi::requestHardResetAll() 
{
  cout << "Unimplemented!\n";
}

void SctApi::requestHardReset(UINT32 mid)
{
  unsigned int MUR, module;

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  config->translateFromROD(partition, crate, rod, channel,
                           MUR, module);

  unsigned int ppartition, pcrate, pchannel;

  config->translateToPowerSupply(MUR, module, 
                                 ppartition, pcrate, pchannel);

  std::cout << "Should do hard reset on module in " 
            << " partition: " << ppartition 
            << " crate: " << pcrate 
            << " channel: " << pchannel << std::endl;
}

void SctApi::requestIVCurve(UINT32 mid, FLOAT32 start, FLOAT32 stop, FLOAT32 step, UINT16 delay, FLOAT32 currentLim)
{
  log << "requestIVCurve: " << mid << ", " << start << ", " << stop << ", " 
      << step << " " << delay << " " << currentLim << std::endl;

  unsigned int MUR, module;

  unsigned int partition, crate, rod, channel;
  getpcrc(mid, partition, crate, rod, channel);

  config->translateFromROD(partition, crate, rod, channel,
                           MUR, module);

  unsigned int ppartition, pcrate, pchannel;

  config->translateToPowerSupply(MUR, module, 
                                 ppartition, pcrate, pchannel);

  if(!ddcCmd) initDDC();

  log << "ddc->requestIVCurve: " << crate << ", " << channel 
      << ", " << start << ", " << stop << ", " << step 
      << " " << 0 << " " << currentLim << " " << delay << " 10" << std::endl;

#warning "These should be changeable"
  float Vend = 0.0;
  float Iwarn = currentLim;

  bool result = ddcCmd->requestIVCurve(crate, channel, start, stop, step, Vend, 
                                       Iwarn, currentLim, delay, 10);

  log << "iv curve success got: " << result << std::endl;
  
}

void SctApi::requestIVCurveAll(FLOAT32 start, FLOAT32 stop, FLOAT32 step, UINT16 delay, FLOAT32 currentLim)
{
}

void SctApi::ddcGetChannelParameter(int crate, int channel, string dpeName,
                                    int& dpeValue, int timeout) 
{
  log << "ddcGetChannelParameter: " << crate << ", " << channel << ", (" << dpeName << "), " << dpeName.size() << " " << timeout << std::endl;

// #warning "Shouldn't have to do this every time!"
//   string *ctrls [2];
//   ctrls[0] = new string("ddc_ct1");
//   ctrls[1] = 0;

//   IPCPartition *ddcPartition = new IPCPartition("SCT");
//   SCTDCSCommand *ddcCmd = new SCTDCSCommand(*ddcPartition, ctrls);

  if(!ddcCmd) initDDC();

  ddcCmd->getChannelParameter(crate, channel, dpeName, dpeValue, timeout);

  log << "channel parameter got: " << dpeValue << std::endl;
}

void SctApi::setSelect(UINT32 mid, bool value)
{
}

void SctApi::changeRunState(int state)
{
}



void SctApi::setDebugLevel(unsigned int level) 
{
  debugLevel = level;
}

void SctApi::initDDC() 
{
  // Initialise DDC command
  ddcPartition = new IPCPartition("SCT");
  ddcCtrls[0] = new string("ddc_ct1");
  ddcCtrls[1] = 0;

  ddcCmd = new SCTDCSCommand(*ddcPartition, ddcCtrls);
}

unsigned int SctApi::getDebugLevel()
{
  return debugLevel;
}

Scan::~Scan() {}
Trigger::~Trigger() {}

}  // End of namespace SctApi

/// Convert mid to p/c/r/c notation
static void getpcrc(UINT32 mid, unsigned int &partition, unsigned int &crate, unsigned int &rod, unsigned int &channel) {
  partition = (mid & 0xff000000) >> 24;
  crate = (mid & 0xff0000) >> 16;
  rod = (mid & 0xff00) >> 8;
  channel = (mid & 0xff) >> 0;
}

/// Convert p/c/r/c notation to mid
static UINT32 packpcrc(unsigned int &partition, unsigned int &crate, unsigned int &rod, unsigned int &channel) {
  UINT32 mid = 0;
  mid |= (partition & 0xff) << 24;
  mid |= (crate & 0xff) << 16;
  mid |= (rod & 0xff) << 8;
  mid |= (channel & 0xff) << 0;

  return mid;
}

// Some static exception handlers documented only in this file
/**
   Catch various exceptions (just so its known what happened, can't recover anything)
   Hopefully VME will get shut down properly???

   *** Only works for exception prototypes with missing exceptions ***
 */
static void handle_unexpected(void) {
  cerr << "Unexpected exception thrown in unknown place\n";

  try {
    throw;
  } catch (RodException &r) {
    cerr << "Rod Exception\n";
    cerr << r.getDescriptor() << ", " << r.getData1() << ", " << r.getData2() << endl;
  } catch (NoImageFile &f) {
    cerr << "No image file exception " << f.getFileName() << endl;
  } catch (VmeException &v) {
    decodeVme(v);
  } catch (HpiException &h) {
    cerr << "HpiException:\n";
    hex(cerr);
    cerr << h.getDescriptor() << '\n';
    cerr << "calcAddr: " << h.getCalcAddr() << ", readAddr: " <<
      h.getReadAddr() << '\n';
    dec(cerr);
  } catch (PrimListException &p) {
    cerr << "Primlist Exception:\n";
    cerr << p.getDescriptor() << " " << p.getData1() << ", " << p.getData2() << endl;
  } catch(std::exception &e) {
    cerr << "Exception of type " << e.what() << endl;
  } catch(...) {
    cerr << "Unknown exception type\n";
  }

  // New version of VmeInterface should cope and shutdown RCC interface properly
  exit(1);

  //  Rethrow exception, function _must_ catch (and therefore method with specification, 
  //   if no exceptions specified then no problem in the first place...) std::bad_exception to recover.
// throw;
}

static void decodeVme(VmeException &e) {
  cout << "VmeException Code: " << e.getErrorCode() << " Class: ";

  switch(e.getErrorClass()) {
  case VmeException::OK: cout << "OK\n"; break;
  case VmeException::INIT_ERROR: cout << "Init error\n"; break;
  case VmeException::BUS_ERROR: cout << "Bus error\n"; break;
  case VmeException::MAP_ERROR: cout << "Map error\n"; break;
  case VmeException::DMA_ERROR: cout << "Dma error\n"; break;
  default: cout << "Unknown error class\n"; break;
  }

  VmePort *p = e.getPort();

  cout << "Port: code: " << p->getLastErrcode() << " " 
       << p->getErrorMessage(p->getLastErrcode()) << endl;
}

static void printMemoryBlock(ostream &os, unsigned long *mem, unsigned long words, int wordsPerLine, int level) {
  unsigned int lines = words/wordsPerLine;
  int skipping = 0;

  os.fill('0');

  os << hex;
  for(unsigned int i=0; i<lines; i++) {
    int val = 0;
    for(int j=0; j<wordsPerLine; j++) {
      val+=mem[i*wordsPerLine+j];
    }
    if(val == 0) {
      if(!skipping) 
        os << string("        ", 0, level) << ".";
      skipping++;
    } else {
      if(skipping) os << " Skipped " << dec << skipping << hex << " blank lines\n";
      skipping = 0;

      os << string("        ", 0, level);
      for(int j=0; j<wordsPerLine; j++) {
        os.fill('0');
        os.width(8);
        os << mem[i*wordsPerLine+j] << " ";
      }
      os << endl;
    }
  }
  if(skipping) os << " Skipped " << dec << skipping << hex << " blank lines\n";
  if(lines*wordsPerLine < words) {
    os << string("        ", 0, level);
    for(unsigned int i=lines*wordsPerLine; i<words; i++) {
      os.fill('0');
      os.width(8);
      os << mem[i] << " ";
    }
    os << endl;
  }
  os.fill(' ');

  os << dec;
}

#ifdef HAVE_PRIM_NAMES

const char *getName(int id) {
  for(int i=0; primNames[i].id != -1; i++) {
    if(primNames[i].id == id) {
      return primNames[i].name;
    }
  }

  return 0;
}

const char *getTaskName(int id) {
  for(int i=0; taskNames[i].id != -1; i++) {
    if(taskNames[i].id == id) {
      return taskNames[i].name;
    }
  }

  return 0;
}

#endif
